@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     04.10.99
-// RCS-ID:      $Id: init.cpp,v 1.69 2006/12/13 19:03:34 VZ Exp $
+// RCS-ID:      $Id: init.cpp 51336 2008-01-22 13:59:45Z SC $
 // Copyright:   (c) Vadim Zeitlin
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -179,7 +179,11 @@ static void ConvertArgsToUnicode(int argc, char **argv)
     int wargc = 0;
     for ( int i = 0; i < argc; i++ )
     {
+#ifdef __DARWIN__
+        wxWCharBuffer buf(wxConvFileName->cMB2WX(argv[i]));
+#else
         wxWCharBuffer buf(wxConvLocal.cMB2WX(argv[i]));
+#endif
         if ( !buf )
         {
             wxLogWarning(_("Command line argument %d couldn't be converted to Unicode and will be ignored."),
@@ -220,6 +224,9 @@ static void FreeConvertedArgs()
 static bool DoCommonPreInit()
 {
 #if wxUSE_LOG
+    // Reset logging in case we were cleaned up and are being reinitialized.
+    wxLog::DoCreateOnDemand();
+
     // install temporary log sink: we can't use wxLogGui before wxApp is
     // constructed and if we use wxLogStderr, all messages during
     // initialization simply disappear under Windows
@@ -391,8 +398,12 @@ void wxEntryCleanup()
     {
         wxTheApp->CleanUp();
 
-        delete wxTheApp;
+        // reset the global pointer to it to NULL before destroying it as in
+        // some circumstances this can result in executing the code using
+        // wxTheApp and using half-destroyed object is no good
+        wxAppConsole * const app = wxApp::GetInstance();
         wxApp::SetInstance(NULL);
+        delete app;
     }
 
 
