@@ -2,7 +2,7 @@
 // Name:        src/gtk/dnd.cpp
 // Purpose:     wxDropTarget class
 // Author:      Robert Roebling
-// Id:          $Id: dnd.cpp,v 1.117 2006/12/01 14:38:39 JJ Exp $
+// Id:          $Id: dnd.cpp 49608 2007-11-03 21:46:55Z VZ $
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -901,6 +901,16 @@ wxDragResult wxDropSource::DoDragDrop(int flags)
                 g_lastButtonNumber,  // number of mouse button which started drag
                 (GdkEvent*) g_lastMouseEvent );
 
+    if ( !context )
+    {
+        // this can happen e.g. if gdk_pointer_grab() failed
+        g_blockEventsOnDrag = false;
+
+        UnregisterWindow();
+
+        return wxDragError;
+    }
+
     m_dragContext = context;
 
     PrepareIcon( action, context );
