@@ -6,7 +6,7 @@
  * Author:      Guillermo Rodriguez Garcia <guille@iies.es>
  * Purpose:     GSocket main MSW file
  * Licence:     The wxWindows licence
- * CVSID:       $Id: gsocket.cpp 44864 2007-03-17 02:15:31Z VZ $
+ * CVSID:       $Id: gsocket.cpp 52492 2008-03-14 14:17:14Z JS $
  * -------------------------------------------------------------------------
  */
 
@@ -870,9 +870,9 @@ GSocketEventFlags GSocket::Select(GSocketEventFlags flags)
       result |= GSOCK_INPUT_FLAG;
 
       if (m_server && m_stream)
-      { 
-        /* This is a TCP server socket that detected a connection. 
-           While the INPUT_FLAG is also set, it doesn't matter on 
+      {
+        /* This is a TCP server socket that detected a connection.
+           While the INPUT_FLAG is also set, it doesn't matter on
            this kind of  sockets, as we can only Accept() from them. */
         result |= GSOCK_CONNECTION_FLAG;
         m_detected |= GSOCK_CONNECTION_FLAG;
@@ -1451,7 +1451,7 @@ GSocketError GAddress_INET_SetPortName(GAddress *address, const char *port,
   se = getservbyname(port, protocol);
   if (!se)
   {
-    if (isdigit(port[0]))
+    if (isdigit((unsigned char) port[0]))
     {
       int port_int;
 
