@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     2003-07-12
-// RCS-ID:      $Id: tbarwce.cpp,v 1.35 2006/09/07 19:01:45 ABX Exp $
+// RCS-ID:      $Id: tbarwce.cpp 44268 2007-01-20 14:13:52Z VZ $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -491,19 +491,21 @@ bool wxToolMenuBar::MSWCommand(WXUINT WXUNUSED(cmd), WXWORD id)
     wxToolBarToolBase *tool = FindById((int)id);
     if ( !tool )
     {
-        if (m_menuBar)
+        bool checked = false;
+        if ( m_menuBar )
         {
             wxMenuItem *item = m_menuBar->FindItem(id);
-            if (item && item->IsCheckable())
+            if ( item && item->IsCheckable() )
             {
                 item->Toggle();
+                checked = item->IsChecked();
             }
         }
 
         wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED);
         event.SetEventObject(this);
         event.SetId(id);
-        event.SetInt(id);
+        event.SetInt(checked);
 
         return GetEventHandler()->ProcessEvent(event);
     }
