@@ -8,7 +8,7 @@
  *              Guillermo Rodriguez Garcia <guille@iies.es>
  * Purpose:     GSocket main Unix and OS/2 file
  * Licence:     The wxWindows licence
- * CVSID:       $Id: gsocket.cpp,v 1.67 2007/01/07 21:38:08 SN Exp $
+ * CVSID:       $Id: gsocket.cpp 58386 2009-01-25 12:45:29Z VZ $
  * -------------------------------------------------------------------------
  */
 
@@ -251,13 +251,13 @@ static struct hostent * deepCopyHostent(struct hostent *h,
   misalign = sizeof(char *) - pos%sizeof(char *);
   if(misalign < sizeof(char *))
     pos += misalign;
-  
+
   /* leave space for pointer list */
   p = h->h_aliases;
   char **h_aliases = (char **)(buffer + pos);
   while(*(p++) != 0)
     pos += sizeof(char *);
-  
+
   /* copy aliases and fill new pointer list */
   for (p = h->h_aliases, q = h_aliases; *p != 0; p++, q++)
   {
@@ -389,13 +389,13 @@ static struct servent * deepCopyServent(struct servent *s,
   unsigned int misalign = sizeof(char *) - pos%sizeof(char *);
   if(misalign < sizeof(char *))
     pos += misalign;
-  
+
   /* leave space for pointer list */
   char **p = s->s_aliases, **q;
   char **s_aliases = (char **)(buffer + pos);
   while(*(p++) != 0)
     pos += sizeof(char *);
-  
+
   /* copy addresses and fill new pointer list */
   for (p = s->s_aliases, q = s_aliases; *p != 0; p++, q++){
     len = strlen(*p);
@@ -545,10 +545,17 @@ GSocket::GSocket()
 void GSocket::Close()
 {
     gs_gui_functions->Disable_Events(this);
-    /* gsockosx.c calls CFSocketInvalidate which closes the socket for us */
-#if !(defined(__DARWIN__) && (defined(__WXMAC__) || defined(__WXCOCOA__)))
-    close(m_fd);
-#endif
+
+    /*  When running on OS X, the gsockosx implementation of GSocketGUIFunctionsTable
+        will close the socket during Disable_Events.  However, it will only do this
+        if it is being used.  That is, it won't do it in a console program.  To
+        ensure we get the right behavior, we have gsockosx set m_fd = INVALID_SOCKET
+        if it has closed the socket which indicates to us (at runtime, instead of
+        at compile time as this had been before) that the socket has already
+        been closed.
+     */
+    if(m_fd != INVALID_SOCKET)
+        close(m_fd);
     m_fd = INVALID_SOCKET;
 }
 
@@ -587,7 +594,7 @@ void GSocket::Shutdown()
   /* If socket has been created, shutdown it */
   if (m_fd != INVALID_SOCKET)
   {
-    shutdown(m_fd, 2);
+    shutdown(m_fd, 1);
     Close();
   }
 
@@ -762,7 +769,7 @@ GSocketError GSocket::SetServer()
 
   /* FreeBSD variants can't use MSG_NOSIGNAL, and instead use a socket option */
 #ifdef SO_NOSIGPIPE
-  setsockopt(m_fd, SOL_SOCKET, SO_NOSIGPIPE, (const char*)&arg, sizeof(u_long));
+  setsockopt(m_fd, SOL_SOCKET, SO_NOSIGPIPE, (const char*)&arg, sizeof(arg));
 #endif
 
   ioctl(m_fd, FIONBIO, &arg);
@@ -773,9 +780,9 @@ GSocketError GSocket::SetServer()
    */
   if (m_reusable)
   {
-    setsockopt(m_fd, SOL_SOCKET, SO_REUSEADDR, (const char*)&arg, sizeof(u_long));
+    setsockopt(m_fd, SOL_SOCKET, SO_REUSEADDR, (const char*)&arg, sizeof(arg));
 #ifdef SO_REUSEPORT
-    setsockopt(m_fd, SOL_SOCKET, SO_REUSEPORT, (const char*)&arg, sizeof(u_long));
+    setsockopt(m_fd, SOL_SOCKET, SO_REUSEPORT, (const char*)&arg, sizeof(arg));
 #endif
   }
 
@@ -967,7 +974,7 @@ GSocketError GSocket::Connect(GSocketStream stream)
 
   /* FreeBSD variants can't use MSG_NOSIGNAL, and instead use a socket option */
 #ifdef SO_NOSIGPIPE
-  setsockopt(m_fd, SOL_SOCKET, SO_NOSIGPIPE, (const char*)&arg, sizeof(u_long));
+  setsockopt(m_fd, SOL_SOCKET, SO_NOSIGPIPE, (const char*)&arg, sizeof(arg));
 #endif
 
 #if defined(__EMX__) || defined(__VISAGECPP__)
@@ -979,9 +986,9 @@ GSocketError GSocket::Connect(GSocketStream stream)
   // If the reuse flag is set, use the applicable socket reuse flags(s)
   if (m_reusable)
   {
-    setsockopt(m_fd, SOL_SOCKET, SO_REUSEADDR, (const char*)&arg, sizeof(u_long));
+    setsockopt(m_fd, SOL_SOCKET, SO_REUSEADDR, (const char*)&arg, sizeof(arg));
 #ifdef SO_REUSEPORT
-    setsockopt(m_fd, SOL_SOCKET, SO_REUSEPORT, (const char*)&arg, sizeof(u_long));
+    setsockopt(m_fd, SOL_SOCKET, SO_REUSEPORT, (const char*)&arg, sizeof(arg));
 #endif
   }
 
@@ -1111,6 +1118,14 @@ GSocketError GSocket::SetNonOriented()
 #endif
   gs_gui_functions->Enable_Events(this);
 
+  if (m_reusable)
+  {
+    setsockopt(m_fd, SOL_SOCKET, SO_REUSEADDR, (const char*)&arg, sizeof(arg));
+#ifdef SO_REUSEPORT
+    setsockopt(m_fd, SOL_SOCKET, SO_REUSEPORT, (const char*)&arg, sizeof(arg));
+#endif
+  }
+
   /* Bind to the local address,
    * and retrieve the actual address bound.
    */
@@ -1160,17 +1175,21 @@ int GSocket::Read(char *buffer, int size)
     else
       ret = Recv_Dgram(buffer, size);
 
-    /* If recv returned zero, then the connection is lost, and errno is not set.
-     * Otherwise, recv has returned an error (-1), in which case we have lost the
-     * socket only if errno does _not_ indicate that there may be more data to read.
+    /*
+     * If recv returned zero for a TCP socket (if m_stream == NULL, it's an UDP
+     * socket and empty datagrams are possible), then the connection has been
+     * gracefully closed.
+     *
+     * Otherwise, recv has returned an error (-1), in which case we have lost
+     * the socket only if errno does _not_ indicate that there may be more data
+     * to read.
      */
-    if (ret == 0)
+    if ((ret == 0) && m_stream)
     {
-      m_error = GSOCK_IOERR;
+      /* Make sure wxSOCKET_LOST event gets sent and shut down the socket */
       m_detected = GSOCK_LOST_FLAG;
-      Close();
-      // Signal an error for return
-      return -1;
+      Detected_Read();
+      return 0;
     }
     else if (ret == -1)
     {
@@ -1319,7 +1338,7 @@ GSocketEventFlags GSocket::Select(GSocketEventFlags flags)
       {
         /* This is a TCP server socket that detected a connection.
           While the INPUT_FLAG is also set, it doesn't matter on
-          this kind of  sockets, as we can only Accept() from them. */        
+          this kind of  sockets, as we can only Accept() from them. */
         result |= GSOCK_CONNECTION_FLAG;
         m_detected |= GSOCK_CONNECTION_FLAG;
       }
@@ -1744,6 +1763,20 @@ void GSocket::Detected_Read()
     {
       CALL_CALLBACK(this, GSOCK_CONNECTION);
     }
+    else if (num == 0)
+    {
+      if (m_stream)
+      {
+        /* graceful shutdown */
+        CALL_CALLBACK(this, GSOCK_LOST);
+        Shutdown();
+      }
+      else
+      {
+        /* Empty datagram received */
+        CALL_CALLBACK(this, GSOCK_INPUT);
+      }
+    }
     else
     {
       /* Do not throw a lost event in cases where the socket isn't really lost */
@@ -2035,6 +2068,7 @@ GSocketError GAddress_INET_SetHostName(GAddress *address, const char *hostname)
     struct hostent h;
 #if defined(HAVE_FUNC_GETHOSTBYNAME_R_3)
     struct hostent_data buffer;
+    memset(&buffer, 0, sizeof(buffer));
 #else
     char buffer[1024];
 #endif
@@ -2092,6 +2126,7 @@ GSocketError GAddress_INET_SetPortName(GAddress *address, const char *port,
 
 #if defined(HAVE_FUNC_GETSERVBYNAME_R_4)
     struct servent_data buffer;
+    memset(&buffer, 0, sizeof(buffer));
 #else
   char buffer[1024];
 #endif
@@ -2150,6 +2185,7 @@ GSocketError GAddress_INET_GetHostName(GAddress *address, char *hostname, size_t
   struct hostent temphost;
 #if defined(HAVE_FUNC_GETHOSTBYNAME_R_3)
   struct hostent_data buffer;
+  memset(&buffer, 0, sizeof(buffer));
 #else
   char buffer[1024];
 #endif
