@@ -5,7 +5,7 @@
 // Author:      Robin Dunn
 //
 // Created:     1-July-1997
-// RCS-ID:      $Id: helpers.cpp 49695 2007-11-06 20:52:28Z RD $
+// RCS-ID:      $Id: helpers.cpp 60451 2009-05-01 00:24:29Z RD $
 // Copyright:   (c) 1998 by Total Control Software
 // Licence:     wxWindows license
 /////////////////////////////////////////////////////////////////////////////
@@ -98,7 +98,63 @@ static PyObject* wxPyNoAppError = NULL;
 PyObject* wxPyPtrTypeMap = NULL;
 
 
-#ifdef __WXMSW__             // If building for win32...
+#ifdef __WXMSW__             // If building for Windows...
+
+//----------------------------------------------------------------------
+// Use an ActivationContext to ensure that the new (themed) version of
+// the comctl32 DLL is loaded.
+//----------------------------------------------------------------------
+
+// Note that the use of the ISOLATION_AWARE_ENABLED define replaces the
+// activation context APIs with wrappers that dynamically load the API
+// pointers from the kernel32 DLL so we don't have to do that ourselves.
+// Using ISOLATION_AWARE_ENABLED also causes the manifest resource to be put
+// in slot #2 as expected for DLLs. (See wx/msw/wx.rc)
+#ifdef ISOLATION_AWARE_ENABLED
+
+static ULONG_PTR wxPySetActivationContext()
+{
+
+    OSVERSIONINFO info;
+    wxZeroMemory(info);
+    info.dwOSVersionInfoSize = sizeof(OSVERSIONINFO); 
+    GetVersionEx(&info);
+    if (info.dwMajorVersion < 5)
+        return 0;
+    
+    ULONG_PTR cookie = 0;
+    HANDLE h;
+    ACTCTX actctx;
+    TCHAR modulename[MAX_PATH];
+
+    GetModuleFileName(wxGetInstance(), modulename, MAX_PATH);
+    wxZeroMemory(actctx);
+    actctx.cbSize = sizeof(actctx);
+    actctx.lpSource = modulename;
+    actctx.lpResourceName = MAKEINTRESOURCE(2);
+    actctx.hModule = wxGetInstance();
+    actctx.dwFlags = ACTCTX_FLAG_HMODULE_VALID | ACTCTX_FLAG_RESOURCE_NAME_VALID;
+    
+    h = CreateActCtx(&actctx);
+    if (h == INVALID_HANDLE_VALUE) {
+        wxLogLastError(wxT("CreateActCtx"));
+        return 0;
+    }
+
+    if (! ActivateActCtx(h, &cookie))
+        wxLogLastError(wxT("ActivateActCtx"));
+    
+    return cookie;
+}
+
+static void wxPyClearActivationContext(ULONG_PTR cookie)
+{
+    if (! DeactivateActCtx(0, cookie))
+        wxLogLastError(wxT("DeactivateActCtx"));
+}
+
+#endif
+
 //----------------------------------------------------------------------
 // This gets run when the DLL is loaded.  We just need to save a handle.
 //----------------------------------------------------------------------
@@ -114,7 +170,8 @@ BOOL WINAPI DllMain(
     // the instance has already been set.
     if (! wxGetInstance())
         wxSetInstance(hinstDLL);
-    return true;
+
+    return TRUE;
 }
 #endif
 
@@ -128,6 +185,7 @@ IMPLEMENT_ABSTRACT_CLASS(wxPyApp, wxApp);
 wxPyApp::wxPyApp() {
     m_assertMode = wxPYAPP_ASSERT_EXCEPTION;
     m_startupComplete = false;
+    m_callFilterEvent = false;
 }
 
 
@@ -182,6 +240,20 @@ int wxPyApp::OnExit() {
 }
 
 
+#if wxUSE_EXCEPTIONS
+bool wxPyApp::OnExceptionInMainLoop() {
+    bool rval=false;
+    bool found;
+    wxPyBlock_t blocked = wxPyBeginBlockThreads();
+    if ((found = wxPyCBH_findCallback(m_myInst, "OnExceptionInMainLoop")))
+        rval = wxPyCBH_callCallback(m_myInst, Py_BuildValue("()"));
+    wxPyEndBlockThreads(blocked);
+    if (! found)
+        rval = wxApp::OnExceptionInMainLoop(); 
+    return rval;
+}
+#endif
+
 
 void wxPyApp::ExitMainLoop() {
     bool found;
@@ -194,6 +266,23 @@ void wxPyApp::ExitMainLoop() {
 }  
 
 
+int wxPyApp::FilterEvent(wxEvent& event) {
+    int result = -1;
+
+    if (m_callFilterEvent) {
+        wxPyBlock_t blocked = wxPyBeginBlockThreads();
+        if (wxPyCBH_findCallback(m_myInst, "FilterEvent")) {
+            wxString className = event.GetClassInfo()->GetClassName();
+            PyObject* eventObject = wxPyConstructObject((void*)&event, className, 0);
+            result = wxPyCBH_callCallback(m_myInst, Py_BuildValue("(O)", eventObject));
+            Py_DECREF(eventObject);
+        }
+        wxPyEndBlockThreads(blocked);
+    }
+    return result;
+}
+
+
 #ifdef __WXDEBUG__
 void wxPyApp::OnAssertFailure(const wxChar *file,
                               int line,
@@ -405,11 +494,11 @@ void wxPyApp::_BootstrapApp()
 {
     static      bool haveInitialized = false;
     bool        result;
-    wxPyBlock_t     blocked;
+    wxPyBlock_t blocked;
     PyObject*   retval = NULL;
     PyObject*   pyint  = NULL;
 
-
+    
     // Only initialize wxWidgets once
     if (! haveInitialized) {
 
@@ -459,6 +548,7 @@ void wxPyApp::_BootstrapApp()
 //        wxSystemOptions::SetOption(wxT("mac.textcontrol-use-mlte"), 1);
         
         wxPyEndBlockThreads(blocked);
+
         haveInitialized = true;
     }
     else {
@@ -572,7 +662,9 @@ inline const char* dropwx(const char* name) {
 // wxPyApp::_BootstrapApp
 void __wxPyPreStart(PyObject* moduleDict)
 {
-
+#ifdef ISOLATION_AWARE_ENABLED
+    wxPySetActivationContext();
+#endif
 #ifdef __WXMSW__
 //     wxCrtSetDbgFlag(_CRTDBG_LEAK_CHECK_DF
 //                     | _CRTDBG_CHECK_ALWAYS_DF
@@ -1944,7 +2036,7 @@ wxPyEvtSelfRef::wxPyEvtSelfRef() {
 
 wxPyEvtSelfRef::~wxPyEvtSelfRef() {
     wxPyBlock_t blocked = wxPyBeginBlockThreads();
-    if (m_cloned)
+    if ( !wxPyDoingCleanup && m_cloned)
         Py_DECREF(m_self);
     wxPyEndBlockThreads(blocked);
 }
@@ -2008,29 +2100,6 @@ wxPyCommandEvent::~wxPyCommandEvent() {
 
 //---------------------------------------------------------------------------
 //---------------------------------------------------------------------------
-// Convert a wxList to a Python List, only works for lists of wxObjects
-
-PyObject* wxPy_ConvertList(wxListBase* listbase) {
-    wxList*     list = (wxList*)listbase;  // this is probably bad...
-    PyObject*   pyList;
-    PyObject*   pyObj;
-    wxObject*   wxObj;
-    wxNode*     node = list->GetFirst();
-
-    wxPyBlock_t blocked = wxPyBeginBlockThreads();
-    pyList = PyList_New(0);
-    while (node) {
-        wxObj = node->GetData();
-        pyObj = wxPyMake_wxObject(wxObj,false);
-        PyList_Append(pyList, pyObj);
-        Py_DECREF(pyObj);  // the Append also does an INCREF, that's one more than we need.
-        node = node->GetNext();
-    }
-    wxPyEndBlockThreads(blocked);
-    return pyList;
-}
-
-//----------------------------------------------------------------------
 
 long wxPyGetWinHandle(wxWindow* win) {
 
@@ -2070,8 +2139,9 @@ wxString* wxString_in_helper(PyObject* source) {
     target = new wxString();
     size_t len = PyUnicode_GET_SIZE(uni);
     if (len) {
-        PyUnicode_AsWideChar((PyUnicodeObject*)uni, target->GetWriteBuf(len), len);
-        target->UngetWriteBuf(len);
+        PyUnicode_AsWideChar((PyUnicodeObject*)uni,
+                             wxStringBuffer(*target, len),
+                             len);
     }
 
     if (PyString_Check(source))
@@ -2123,8 +2193,9 @@ wxString Py2wxString(PyObject* source)
     }
     size_t len = PyUnicode_GET_SIZE(uni);
     if (len) {
-        PyUnicode_AsWideChar((PyUnicodeObject*)uni, target.GetWriteBuf(len), len);
-        target.UngetWriteBuf();
+        PyUnicode_AsWideChar((PyUnicodeObject*)uni,
+                             wxStringBuffer(target, len),
+                             len);
     }
 
     if (!PyUnicode_Check(source))
@@ -2839,18 +2910,34 @@ bool wxColour_helper(PyObject* source, wxColour** obj) {
     // otherwise check for a string
     else if (PyString_Check(source) || PyUnicode_Check(source)) {
         wxString spec = Py2wxString(source);
-        if (spec.GetChar(0) == '#' && spec.Length() == 7) {  // It's  #RRGGBB
+        if (spec.GetChar(0) == '#' 
+            && (spec.length() == 7 || spec.length() == 9)) {  // It's  #RRGGBB[AA]
             long red, green, blue;
             red = green = blue = 0;
             spec.Mid(1,2).ToLong(&red,   16);
             spec.Mid(3,2).ToLong(&green, 16);
             spec.Mid(5,2).ToLong(&blue,  16);
 
-            **obj = wxColour(red, green, blue);
+            if (spec.length() == 7)         // no alpha
+                **obj = wxColour(red, green, blue);
+            else {                          // yes alpha
+                long alpha;
+                spec.Mid(7,2).ToLong(&alpha, 16);
+                **obj = wxColour(red, green, blue, alpha);
+            }
             return true;
         }
         else {                                       // it's a colour name
-            **obj = wxColour(spec);
+            // check if alpha is there too
+            int pos;
+            if (((pos = spec.Find(':', true)) != wxNOT_FOUND) && (pos == spec.length()-3)) {
+                long alpha;
+                spec.Right(2).ToLong(&alpha, 16);
+                wxColour c = wxColour(spec.Left(spec.length()-3));
+                **obj = wxColour(c.Red(), c.Green(), c.Blue(), alpha);
+            }
+            else
+                **obj = wxColour(spec);
             return true;
         }
     }
