@@ -43,37 +43,44 @@ a solid color it may cause the control to loose its transparent appearance.
 Other attributes can be configured after the control has been created. The
 settings that are currently available are as follows:
 
-SetBitmap: Change/Add the bitmap at any time and the control will resize and
-           refresh to display it.
-SetLabelColor: Explicitly set text colors
-SetMenu: Set the button to have a popupmenu. When a menu is set a small drop
-            arrow will be drawn on the button that can then be clicked to show
-            a menu.
-SetPressColor: Use a custom highlight color
+  - SetBitmap: Change/Add the bitmap at any time and the control will resize and
+               refresh to display it.
+  - SetLabelColor: Explicitly set text colors
+  - SetMenu: Set the button to have a popupmenu. When a menu is set a small drop
+             arrow will be drawn on the button that can then be clicked to show
+             a menu.
+  - SetPressColor: Use a custom highlight color
 
 
 Overridden Methods Inherited from PyControl:
 
-SetFont: Changing the font is one way to set the size of the button, by default
-         the control will inherit its font from its parent.
+  - SetFont: Changing the font is one way to set the size of the button, by
+             default the control will inherit its font from its parent.
 
-SetWindowVariant: Setting the window variant will cause the control to resize to
-                  the corresponding variant size. However if the button is using
-                  a bitmap the bitmap will remain unchanged and only the font
-                  will be adjusted.
+  - SetWindowVariant: Setting the window variant will cause the control to
+                      resize to the corresponding variant size. However if the
+                      button is using a bitmap the bitmap will remain unchanged
+                      and only the font will be adjusted.
 
 Requirements:
-    This module requires python2.4 or higher and wxPython2.8 or higher
+  - python2.4 or higher
+  - wxPython2.8 or higher
 
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: platebtn.py 51224 2008-01-15 12:51:16Z CJP $"
-__revision__ = "$Revision: 51224 $"
+__svnid__ = "$Id: platebtn.py 57713 2009-01-01 23:36:15Z CJP $"
+__revision__ = "$Revision: 57713 $"
+
+__all__ = ["PlateButton", "AdjustAlpha", "AdjustColor", "GetHighlightColor",
+           "PLATE_NORMAL", "PLATE_PRESSED", "PLATE_HIGHLIGHT", 
+           "PB_STYLE_DEFAULT", "PB_STYLE_GRADIENT", "PB_STYLE_SQUARE",
+           "PB_STYLE_NOBG" ]
 
 #-----------------------------------------------------------------------------#
 # Imports
 import wx
+import  wx.lib.newevent
 
 # Used on OSX to get access to carbon api constants
 if wx.Platform == '__WXMAC__':
@@ -91,65 +98,14 @@ PB_STYLE_GRADIENT = 2   # Gradient Filled Background
 PB_STYLE_SQUARE   = 4   # Use square corners instead of rounded
 PB_STYLE_NOBG     = 8   # Usefull on Windows to get a transparent appearance
                         # when the control is shown on a non solid background
+PB_STYLE_DROPARROW = 16 # Draw drop arrow and fire EVT_PLATEBTN_DROPRROW_PRESSED event
+
+PlateBtnDropArrowPressed, EVT_PLATEBTN_DROPARROW_PRESSED = wx.lib.newevent.NewEvent()
 
 #-----------------------------------------------------------------------------#
-# Utility Functions
-def AdjustAlpha(colour, alpha):
-    """Adjust the alpha of a given colour"""
-    return wx.Colour(colour.Red(), colour.Green(), colour.Blue(), alpha)
+# Utility Functions, moved to their own module
 
-def AdjustColour(color, percent, alpha=wx.ALPHA_OPAQUE):
-    """ Brighten/Darken input colour by percent and adjust alpha
-    channel if needed. Returns the modified color.
-    @param color: color object to adjust
-    @type color: wx.Color
-    @param percent: percent to adjust +(brighten) or -(darken)
-    @type percent: int
-    @keyword alpha: amount to adjust alpha channel
-
-    """
-    radj, gadj, badj = [ int(val * (abs(percent) / 100.))
-                         for val in color.Get() ]
-
-    if percent < 0:
-        radj, gadj, badj = [ val * -1 for val in [radj, gadj, badj] ]
-    else:
-        radj, gadj, badj = [ val or 255 for val in [radj, gadj, badj] ]
-
-    red = min(color.Red() + radj, 255)
-    green = min(color.Green() + gadj, 255)
-    blue = min(color.Blue() + badj, 255)
-    return wx.Colour(red, green, blue, alpha)
-
-def BestLabelColour(color):
-    """Get the best color to use for the label that will be drawn on
-    top of the given color.
-    @param color: background color that text will be drawn on
-
-    """
-    avg = sum(color.Get()) / 3
-    if avg > 192:
-        txt_color = wx.BLACK
-    elif avg > 128:
-        txt_color = AdjustColour(color, -95)
-    elif avg < 64:
-        txt_color = wx.WHITE
-    else:
-        txt_color = AdjustColour(color, 95)
-    return txt_color
-
-def GetHighlightColour():
-    """Get the default highlight color
-    @return: wx.Color
-
-    """
-    if wx.Platform == '__WXMAC__':
-        brush = wx.Brush(wx.BLACK)
-        # kThemeBrushButtonPressedLightHighlight
-        brush.MacSetTheme(Carbon.Appearance.kThemeBrushFocusHighlight)
-        return brush.GetColour()
-    else:
-        return wx.SystemSettings.GetColour(wx.SYS_COLOUR_HIGHLIGHT)
+from wx.lib.colourutils import *
 
 #-----------------------------------------------------------------------------#
 
@@ -234,7 +190,7 @@ class PlateButton(wx.PyControl):
         @param ypos: y cord to start at
 
         """
-        if self._menu is not None:
+        if self._menu is not None or self._style & PB_STYLE_DROPARROW:
             # Positioning needs a little help on Windows
             if wx.Platform == '__WXMSW__':
                 xpos -= 2
@@ -319,7 +275,7 @@ class PlateButton(wx.PyControl):
             self.__DrawHighlight(gc, width, height)
 
         elif self._state['cur'] == PLATE_PRESSED:
-            gc.SetTextForeground(self._color['ptxt'])
+            gc.SetTextForeground(self._color['htxt'])
             if wx.Platform == '__WXMAC__':
                 brush = wx.Brush(wx.BLACK)
                 brush.MacSetTheme(Carbon.Appearance.kThemeBrushFocusHighlight)
@@ -328,10 +284,10 @@ class PlateButton(wx.PyControl):
                 pen = wx.Pen(AdjustColour(self._color['press'], -80, 220), 1)
             gc.SetPen(pen)
 
+            self.__DrawHighlight(gc, width, height)
             txt_x = self.__DrawBitmap(gc)
             gc.DrawText(self.GetLabel(), txt_x + 2, txt_y)
             self.__DrawDropArrow(gc, txt_x + tw + 6, (height / 2) - 2)
-            self.__DrawHighlight(gc, width, height)
 
         else:
             if self.IsEnabled():
@@ -349,12 +305,11 @@ class PlateButton(wx.PyControl):
     def __InitColors(self):
         """Initialize the default colors"""
         color = GetHighlightColour()
-        pcolor = AdjustColour(color, -10, 160)
+        pcolor = AdjustColour(color, -12)
         colors = dict(default=True,
                       hlight=color, 
                       press=pcolor,
-                      htxt=BestLabelColour(self.GetForegroundColour()),
-                      ptxt=self.GetForegroundColour())
+                      htxt=BestLabelColour(self.GetForegroundColour()))
         return colors
 
     #---- End Private Member Function ----#
@@ -406,7 +361,7 @@ class PlateButton(wx.PyControl):
         else:
             width += 10
 
-        if self._menu is not None:
+        if self._menu is not None or self._style & PB_STYLE_DROPARROW:
             width += 12
 
         best = wx.Size(width, height)
@@ -522,8 +477,14 @@ class PlateButton(wx.PyControl):
         pos = evt.GetPositionTuple()
         self.SetState(PLATE_PRESSED)
         size = self.GetSizeTuple()
-        if self._menu is not None and pos[0] >= size[0] - 16:
-            self.ShowMenu()
+        if pos[0] >= size[0] - 16:
+            if self._menu is not None:
+                self.ShowMenu()
+            elif self._style & PB_STYLE_DROPARROW:
+                event = PlateBtnDropArrowPressed()
+                event.SetEventObject(self)
+                wx.PostEvent(self, event)
+        
         self.SetFocus()
             
     def OnLeftUp(self, evt):
@@ -533,7 +494,10 @@ class PlateButton(wx.PyControl):
 
         """
         if self._state['cur'] == PLATE_PRESSED:
-            self.__PostEvent()
+            pos = evt.GetPositionTuple()
+            size = self.GetSizeTuple()
+            if not (self._style & PB_STYLE_DROPARROW and pos[0] >= size[0] - 16):
+                self.__PostEvent()
         self.SetState(PLATE_HIGHLIGHT)
 
     def OnMenuClose(self, evt):
@@ -594,7 +558,7 @@ class PlateButton(wx.PyControl):
         wx.PyControl.SetLabel(self, label)
         self.InvalidateBestSize()
 
-    def SetLabelColor(self, normal, hlight=wx.NullColor, press=wx.NullColor):
+    def SetLabelColor(self, normal, hlight=wx.NullColor):
         """Set the color of the label. The optimal label color is usually
         automatically selected depending on the button color. In some
         cases the colors that are choosen may not be optimal.
@@ -606,7 +570,6 @@ class PlateButton(wx.PyControl):
 
         @param normal: Label color for normal state
         @keyword hlight: Color for when mouse is hovering over
-        @keyword press: Color for when button is pressed
 
         """
         self._color['default'] = False
@@ -618,13 +581,10 @@ class PlateButton(wx.PyControl):
             else:
                 self._color['htxt'] = BestLabelColour(normal)
 
-        if press is not None:
-            if press.IsOk():
-                self._color['ptxt'] = press
-            else:
-                self._color['ptxt'] = BestLabelColour(self._color['press'])
-
-        self.GetParent().RefreshRect(self.GetRect(), False)
+        if wx.Platform == '__WXMSW__':
+            self.GetParent().RefreshRect(self.GetRect(), False)
+        else:
+            self.Refresh()
 
     def SetMenu(self, menu):
         """Set the menu that can be shown when clicking on the
@@ -653,7 +613,6 @@ class PlateButton(wx.PyControl):
             self._color['hlight'] = color
         self._color['press'] = AdjustColour(color, -10, 160)
         self._color['htxt'] = BestLabelColour(self._color['hlight'])
-        self._color['ptxt'] = BestLabelColour(self._color['htxt'])
         self.Refresh()
 
     def SetState(self, state):
@@ -664,7 +623,10 @@ class PlateButton(wx.PyControl):
         """
         self._state['pre'] = self._state['cur']
         self._state['cur'] = state
-        self.GetParent().RefreshRect(self.GetRect(), False)
+        if wx.Platform == '__WXMSW__':
+            self.GetParent().RefreshRect(self.GetRect(), False)
+        else:
+            self.Refresh()
 
     def SetWindowStyle(self, style):
         """Sets the window style bytes, the updates take place
