@@ -10,8 +10,8 @@
 """Launch Configuration Dialog"""
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: cfgdlg.py 55064 2008-08-12 03:27:17Z CJP $"
-__revision__ = "$Revision: 55064 $"
+__svnid__ = "$Id: cfgdlg.py 60140 2009-04-14 15:12:34Z CJP $"
+__revision__ = "$Revision: 60140 $"
 
 #-----------------------------------------------------------------------------#
 # Imports
@@ -25,11 +25,10 @@ import zlib
 import handlers
 
 # Editra Libraries
-import eclib.colorsetter as colorsetter
-import eclib.elistmix as elistmix
-from profiler import Profile_Get, Profile_Set
-import ed_msg
+import eclib
 import util
+import ed_msg
+from profiler import Profile_Get, Profile_Set
 
 #-----------------------------------------------------------------------------#
 # Globals
@@ -110,9 +109,6 @@ class ConfigDialog(wx.Frame):
         """
         wx.Frame.__init__(self, parent, title=_("Launch Configuration"))
 
-        # Attributes
-        self.ftype = ftype
-
         # Layout
         util.SetWindowIcon(self)
         self.__DoLayout()
@@ -130,37 +126,59 @@ class ConfigDialog(wx.Frame):
         hsizer = wx.BoxSizer(wx.HORIZONTAL)
         vsizer = wx.BoxSizer(wx.VERTICAL)
         panel = wx.Panel(self)
-        noteb = wx.Notebook(panel)
-        noteb.AddPage(ConfigPanel(noteb), _("General"))
-        noteb.AddPage(MiscPanel(noteb), _("Misc"))
+        noteb = ConfigNotebook(panel)
         hsizer.AddMany([((5, 5), 0), (noteb, 1, wx.EXPAND), ((5, 5), 0)])
         vsizer.AddMany([((5, 5), 0), (hsizer, 1, wx.EXPAND), ((10, 10), 0)])
         panel.SetSizer(vsizer)
-
         sizer.Add(panel, 1, wx.EXPAND)
         self.SetSizer(sizer)
         self.SetAutoLayout(True)
         self.SetInitialSize()
         self.SetMinSize((420, 345))
 
-    def GetLangId(self):
-        return self.ftype
-
     def OnClose(self, evt):
         """Unregister the window when its closed"""
         wx.GetApp().UnRegisterWindow(repr(self))
-        ed_msg.PostMessage(EDMSG_LAUNCH_CFG_EXIT)
         evt.Skip()
 
 #-----------------------------------------------------------------------------#
 
+class ConfigNotebook(wx.Notebook):
+    """Notebook for holding config pages"""
+    def __init__(self, parent):
+        wx.Notebook.__init__(self, parent)
+
+        # Make sure config has been intialized
+        prefs = Profile_Get(LAUNCH_PREFS, default=None)
+        if prefs is None:
+            buff = eclib.OutputBuffer(self)
+            buff.Hide()
+            Profile_Set(LAUNCH_PREFS,
+                        dict(autoclear=False,
+                             defaultf=buff.GetDefaultForeground().Get(),
+                             defaultb=buff.GetDefaultBackground().Get(),
+                             errorf=buff.GetErrorForeground().Get(),
+                             errorb=buff.GetErrorBackground().Get(),
+                             infof=buff.GetInfoForeground().Get(),
+                             infob=buff.GetInfoBackground().Get(),
+                             warnf=buff.GetWarningForeground().Get(),
+                             warnb=buff.GetWarningBackground().Get()))
+            buff.Destroy()
+
+        # Setup
+        self.AddPage(ConfigPanel(self), _("General"))
+        self.AddPage(MiscPanel(self), _("Misc"))
+
+    def __del__(self):
+        ed_msg.PostMessage(EDMSG_LAUNCH_CFG_EXIT)
+
+#-----------------------------------------------------------------------------#
+
 class ConfigPanel(wx.Panel):
     """Configuration panel that holds the controls for configuration"""
     def __init__(self, parent):
         wx.Panel.__init__(self, parent)
 
-        # Attributes
-
         # Layout
         self.__DoLayout()
 
@@ -174,7 +192,7 @@ class ConfigPanel(wx.Panel):
         msizer = wx.BoxSizer(wx.VERTICAL)
 
         lsizer = wx.BoxSizer(wx.HORIZONTAL)
-        ftype = self.GetTopLevelParent().GetLangId()
+        ftype = wx.GetApp().GetCurrentBuffer().GetLangId()
         ftype = handlers.GetHandlerById(ftype).GetName()
         htypes = GetHandlerTypes()
         lang_ch = wx.Choice(self, ID_LANGUAGE, choices=htypes)
@@ -208,15 +226,14 @@ class ConfigPanel(wx.Panel):
                         (def_ch, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)])
 
         # Executables List
-        exelist = AutoWidthListCtrl(self, ID_EXECUTABLES,
-                                    style=wx.LC_EDIT_LABELS|\
-                                          wx.BORDER|\
-                                          wx.LC_REPORT)
-        exelist.SetToolTipString(_("Click on an item to edit"))
-        exelist.InsertColumn(0, _("Alias"))
-        exelist.InsertColumn(1, _("Executable Commands"))
+        exelist = CommandListCtrl(self, ID_EXECUTABLES,
+                                  style=wx.LC_EDIT_LABELS|\
+                                        wx.BORDER|wx.LC_REPORT|\
+                                        wx.LC_SINGLE_SEL)
+#        exelist.SetToolTipString(_("Click on an item to edit"))
+#        exelist.InsertColumn(0, _("Alias"))
+#        exelist.InsertColumn(1, _("Executable Commands"))
         self.SetListItems(chandler.GetCommands())
-        exelist.SetToolTipString(_("Click on an item to edit"))
         addbtn = wx.BitmapButton(self, wx.ID_ADD, GetPlusBitmap())
         addbtn.SetToolTipString(_("Add a new executable"))
         delbtn = wx.BitmapButton(self, wx.ID_REMOVE, GetMinusBitmap())
@@ -376,7 +393,7 @@ class MiscPanel(wx.Panel):
 
         # Event Handlers
         self.Bind(wx.EVT_CHECKBOX, self.OnCheck)
-        self.Bind(colorsetter.EVT_COLORSETTER, self.OnColor)
+        self.Bind(eclib.EVT_COLORSETTER, self.OnColor)
 
     def __DoLayout(self):
         """Layout the controls"""
@@ -395,7 +412,7 @@ class MiscPanel(wx.Panel):
         # Colors
         colors = dict()
         for btn in COLOR_MAP.iteritems():
-            cbtn = colorsetter.ColorSetter(self, btn[0], color=cfg.get(btn[1]))
+            cbtn = eclib.ColorSetter(self, btn[0], color=cfg.get(btn[1]))
             colors[btn[0]] = cbtn
 
         flexg = wx.FlexGridSizer(5, 5, 5, 5)
@@ -469,17 +486,107 @@ class MiscPanel(wx.Panel):
             evt.Skip()
 
 #-----------------------------------------------------------------------------#
+ID_BROWSE = wx.NewId()
 
-class AutoWidthListCtrl(listmix.ListCtrlAutoWidthMixin,
-                        listmix.TextEditMixin,
-                        elistmix.ListRowHighlighter,
-                        wx.ListCtrl):
+class CommandListCtrl(listmix.ListCtrlAutoWidthMixin,
+                      listmix.TextEditMixin,
+                      eclib.ListRowHighlighter,
+#                      listmix.CheckListCtrlMixin,
+                      wx.ListCtrl):
     """Auto-width adjusting list for showing editing the commands"""
     def __init__(self, *args, **kwargs):
         wx.ListCtrl.__init__(self, *args, **kwargs)
         listmix.ListCtrlAutoWidthMixin.__init__(self)
+#        listmix.CheckListCtrlMixin.__init__(self)
+        eclib.ListRowHighlighter.__init__(self)
+
+        # Attributes
+        self._menu = None
+        self._cindex = -1
+
+        # Setup
+        self.SetToolTipString(_("Click on an item to edit"))
+#        pcol = _("Dir")
+#        self.InsertColumn(0, pcol)
+        self.InsertColumn(0, _("Alias"))
+        self.InsertColumn(1, _("Executable Commands"))
+#        self.SetColumnWidth(0, self.GetTextExtent(pcol)[0] + 5)
+
         listmix.TextEditMixin.__init__(self)
-        elistmix.ListRowHighlighter.__init__(self)
+
+        # Event Handlers
+        self.Bind(wx.EVT_LIST_ITEM_RIGHT_CLICK, self.OnContextClick)
+        self.Bind(wx.EVT_MENU, self.OnMenu, id=ID_BROWSE)
+
+    def OnContextClick(self, evt):
+        """Handle right clicks"""
+        if not self.GetSelectedItemCount():
+            evt.Skip()
+            return
+
+        if self._menu is None:
+            # Lazy init of menu
+            self._menu = wx.Menu()
+            self._menu.Append(ID_BROWSE, _("Browse"))
+
+        self._cindex = evt.GetIndex()
+        self.PopupMenu(self._menu)
+
+    def OnMenu(self, evt):
+        """Handle Menu events"""
+        e_id = evt.GetId()
+        if e_id == ID_BROWSE:
+            dlg = wx.FileDialog(self, _("Choose and executable"))
+            if dlg.ShowModal() == wx.ID_OK:
+                path = dlg.GetPath()
+                if self._cindex >= 0:
+                    self.SetStringItem(self._cindex, 1, path)
+                    levt = wx.ListEvent(wx.wxEVT_COMMAND_LIST_END_LABEL_EDIT,
+                                        self.GetId())
+# TODO: ERR ><! there are no setters for index, column, and label...
+#                    levt.Index = self._cindex
+#                    levt.SetInt(self._cindex)
+#                    levt.Column = 1
+#                    levt.Label = path
+                    # HACK set the member variables directly...
+                    levt.m_itemIndex = self._cindex
+                    levt.m_col = 1
+                    levt.SetString(path)
+                    wx.PostEvent(self.GetParent(), levt)
+        else:
+            evt.Skip()
+
+#    def Append(self, entry):
+#        """Append a row to the list. Overrides ListCtrl.Append to allow
+#        for setting a bool on the first object to check or uncheck the
+#        checkbox on the first column.
+#        @param entry: tuple (bool, string, string)
+
+#        """
+#        check, alias, cmd = entry
+#        wx.ListCtrl.Append(self, ('', alias, cmd))
+#        self.CheckItem(self.GetItemCount() - 1, check)
+
+#    def OpenEditor(self, col, row):
+#        """Disable the editor for the first column
+#        @param col: Column to edit
+#        @param row: Row to edit
+
+#        """
+#        if col != 0:
+#            listmix.TextEditMixin.OpenEditor(self, col, row)
+#        else:
+#            # Handle the checkbox click
+#            self.CheckItem(row, not self.IsChecked(row))
+
+#    def OnCheckItem(self, index, flag):
+#        """Override CheckListMixin to update handlers
+#        @param index: list index
+#        @param flag: check or uncheck
+
+#        """
+#        parent = self.GetParent()
+#        parent.UpdateCurrentHandler(index)
 
 #-----------------------------------------------------------------------------#
 
