@@ -31,8 +31,8 @@ StyleText : Perform custom styling on the text as its added, line by line
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: handlers.py 54027 2008-06-08 02:59:09Z CJP $"
-__revision__ = "$Revision: 54027 $"
+__svnid__ = "$Id: handlers.py 59439 2009-03-08 22:43:06Z CJP $"
+__revision__ = "$Revision: 59439 $"
 
 __all__ = ['GetHandlerById', 'GetHandlerByName', 'GetState',
            'SetState', 'DEFAULT_HANDLER']
@@ -89,7 +89,7 @@ RE_PROC_SE = re.compile('>{3,3}.*' + os.linesep)
 # Public Handler Api for use outside this module
 def GetHandlerById(lang_id):
     """Get a handler for the specified language id"""
-    if HANDLERS.has_key(lang_id):
+    if lang_id in HANDLERS:
         return HANDLERS[lang_id]
     else:
         return HANDLERS[0]
@@ -136,9 +136,7 @@ class FileTypeHandler(object):
         self.commands = dict()
         self.default = ''
 
-    @property
-    def __name__(self):
-        return DEFAULT_HANDLER
+    __name__ = DEFAULT_HANDLER
 
     def AppendCommand(self, cmd):
         """Add a command to the list of commands known to this handler
@@ -218,7 +216,7 @@ class FileTypeHandler(object):
                 self.commands.update(sdict)
 
             # Reset default if it has been removed
-            if not self.commands.has_key(self.default):
+            if self.default not in self.commands:
                 keys = self.commands.keys()
                 if len(keys):
                     self.default = keys[0]
@@ -258,23 +256,20 @@ class AdaHandler(FileTypeHandler):
         self.commands = {'gcc -c' : 'gcc -c'}
         self.default = 'gcc -c'
 
-    @property
-    def __name__(self):
-        return 'ada'
+    __name__ = 'ada'
 
 #-----------------------------------------------------------------------------#
 
 class BashHandler(FileTypeHandler):
     """FileTypeHandler for Bash scripts"""
     RE_BASH_ERROR = re.compile('(.+): line ([0-9]+): .*' + os.linesep)
+
     def __init__(self):
         FileTypeHandler.__init__(self)
         self.commands = dict(bash='bash')
         self.default = 'bash'
 
-    @property
-    def __name__(self):
-        return 'bash shell'
+    __name__ = 'bash shell'
 
     def FilterInput(self, txt):
         """Filter out ansi escape sequences from input"""
@@ -287,13 +282,13 @@ class BashHandler(FileTypeHandler):
 
         """
         txt = outbuffer.GetLine(line)
-        match = self.RE_BASH_ERROR.findall(txt)
+        match = BashHandler.RE_BASH_ERROR.findall(txt)
         ifile = None
         if len(match):
             ifile = match[0][0].split()[-1]
             try:
                 line = max(int(match[0][1]) - 1, 0)
-            except:
+            except IndexError:
                 line = 0
 
         # If not an absolute path then the error is in the current script
@@ -305,12 +300,23 @@ class BashHandler(FileTypeHandler):
 
     def StyleText(self, stc, start, txt):
         """Style NSIS output messages"""
-        if _StyleError(stc, start, txt, self.RE_BASH_ERROR):
+        if _StyleError(stc, start, txt, BashHandler.RE_BASH_ERROR):
             return
         else:
             # Highlight Start end lines this is what the
             # base classes method does.
-            FileTypeHandler.StyleText(self, stc, start, txt)
+            super(BashHandler, self).StyleText(stc, start, txt)
+
+#-----------------------------------------------------------------------------#
+
+class BatchHandler(FileTypeHandler):
+    """FileTypeHandler for Dos batch files"""
+    def __init__(self):
+        FileTypeHandler.__init__(self)
+        self.commands = dict(cmd='cmd /c')
+        self.default = 'cmd'
+
+    __name__ = 'Batch'
 
 #-----------------------------------------------------------------------------#
 
@@ -321,9 +327,38 @@ class BooHandler(FileTypeHandler):
         self.commands = dict(booi='booi')
         self.default = 'booi'
 
-    @property
-    def __name__(self):
-        return 'boo'
+    __name__ = 'boo'
+
+#-----------------------------------------------------------------------------#
+
+class CamlHandler(FileTypeHandler):
+    """FileTypeHandler for Caml"""
+    RE_CAML_ERROR = re.compile(r'File "(.+)", line (.+), characters .+:')
+
+    def __init__(self):
+        FileTypeHandler.__init__(self)
+        self.commands = dict(ocaml='ocaml')
+        self.default = 'ocaml'
+
+    __name__ = 'Caml'
+
+    def HandleHotSpot(self, mainw, outbuffer, line, fname):
+        """Hotspots are error messages, find the file/line of the
+        error in question and open the file to that point in the buffer.
+
+        """
+        ifile, line = _FindFileLine(outbuffer, line, fname,
+                                    CamlHandler.RE_CAML_ERROR)
+        _OpenToLine(ifile, line, mainw)
+
+    def StyleText(self, stc, start, txt):
+        """Style OCaml Information and Error messages from script output."""
+        if _StyleError(stc, start, txt, CamlHandler.RE_CAML_ERROR):
+            return
+        else:
+            # Highlight Start end lines this is what the
+            # base classes method does.
+            super(CamlHandler, self).StyleText(stc, start, txt)
 
 #-----------------------------------------------------------------------------#
 
@@ -334,9 +369,18 @@ class CSHHandler(FileTypeHandler):
         self.commands = dict(csh='csh')
         self.default = 'csh'
 
-    @property
-    def __name__(self):
-        return 'c shell'
+    __name__ = 'c shell'
+
+#-----------------------------------------------------------------------------#
+
+class DHandler(FileTypeHandler):
+    """FileTypeHandler for D"""
+    def __init__(self):
+        FileTypeHandler.__init__(self)
+        self.commands = dict(dmd='dmd -run')
+        self.default = 'dmd'
+
+    __name__ = 'd'
 
 #-----------------------------------------------------------------------------#
 
@@ -347,53 +391,104 @@ class FeriteHandler(FileTypeHandler):
         self.commands = dict(ferite='ferite')
         self.default = 'ferite'
 
-    @property
-    def __name__(self):
-        return 'ferite'
+    __name__ = 'ferite'
 
 #-----------------------------------------------------------------------------#
 
 class HaskellHandler(FileTypeHandler):
     """FileTypeHandler for Haskell"""
+    RE_HASKELL_ERROR = re.compile('(.+):(.+):[0-9]+:.+ error .+')
+
     def __init__(self):
         FileTypeHandler.__init__(self)
         self.commands = {'ghc --make' : 'ghc --make'}
         self.default = 'ghc --make'
 
-    @property
-    def __name__(self):
-        return 'haskell'
-
-#-----------------------------------------------------------------------------#
-
-class HaxeHandler(FileTypeHandler):
-    """FileTypeHandler for haXe"""
-    RE_HAXE_ERROR = re.compile('([a-zA-Z_.]+)\(([0-9]+)\):.*')
-    def __init__(self):
-        FileTypeHandler.__init__(self)
-        self.commands = dict(neko='neko', nekoc='nekoc')
-        self.default = 'nekoc'
-
-    @property
-    def __name__(self):
-        return 'haxe'
+    __name__ = 'haskell'
 
     def HandleHotSpot(self, mainw, outbuffer, line, fname):
         """Hotspots are error messages, find the file/line of the
         error in question and open the file to that point in the buffer.
 
         """
-        ifile, line = _FindFileLine(outbuffer, line, fname, self.RE_HAXE_ERROR)
+        ifile, line = _FindFileLine(outbuffer, line, fname,
+                                    HaskellHandler.RE_HASKELL_ERROR)
         _OpenToLine(ifile, line, mainw)
 
     def StyleText(self, stc, start, txt):
-        """Style haXe output messages"""
-        if _StyleError(stc, start, txt, self.RE_HAXE_ERROR):
+        """Style GHC Information and Error messages from script output."""
+        if _StyleError(stc, start, txt, HaskellHandler.RE_HASKELL_ERROR):
             return
         else:
             # Highlight Start end lines this is what the
             # base classes method does.
-            FileTypeHandler.StyleText(self, stc, start, txt)
+            super(HaskellHandler, self).StyleText(stc, start, txt)
+
+#-----------------------------------------------------------------------------#
+
+class HaxeHandler(FileTypeHandler):
+    """FileTypeHandler for haXe"""
+    RE_HAXE_ERROR = re.compile('([a-zA-Z_.]+)\(([0-9]+)\):.*')
+
+    def __init__(self):
+        FileTypeHandler.__init__(self)
+        self.commands = dict(neko='neko', nekoc='nekoc')
+        self.default = 'nekoc'
+
+    __name__ = 'haxe'
+
+    def HandleHotSpot(self, mainw, outbuffer, line, fname):
+        """Hotspots are error messages, find the file/line of the
+        error in question and open the file to that point in the buffer.
+
+        """
+        ifile, line = _FindFileLine(outbuffer, line, fname,
+                                    HaxeHandler.RE_HAXE_ERROR)
+        _OpenToLine(ifile, line, mainw)
+
+    def StyleText(self, stc, start, txt):
+        """Style haXe output messages"""
+        if _StyleError(stc, start, txt, HaxeHandler.RE_HAXE_ERROR):
+            return
+        else:
+            # Highlight Start end lines this is what the
+            # base classes method does.
+            super(HaxeHandler, self).StyleText(stc, start, txt)
+
+#-----------------------------------------------------------------------------#
+
+class HTMLHandler(FileTypeHandler):
+    """FileTypeHandler for HTML"""
+    def __init__(self):
+        FileTypeHandler.__init__(self)
+        if u'darwin' in sys.platform:
+            self.commands = dict(Safari='open -a Safari.app',
+                                 Camino='open -a Camino.app',
+                                 Firefox='open -a Firefox.app',
+                                 Opera='open -a Opera.app')
+            self.default = 'Safari'
+        elif sys.platform.startswith('win'):
+            self.commands = dict(ie='iexplorer.exe',
+                                 firefox='firefox.exe',
+                                 opera='opera.exe')
+            self.default = 'ie'
+        else:
+            self.commands = dict(firefox='firefox',
+                                 opera='opera')
+            self.default = 'firefox'
+
+    __name__ = 'html'
+
+#-----------------------------------------------------------------------------#
+
+class InnoSetupHandler(FileTypeHandler):
+    """FileTypeHandler for Inno Setup Scripts"""
+    def __init__(self):
+        FileTypeHandler.__init__(self)
+        self.commands = dict(iscc='iscc.exe', Compil32='Compil32.exe /cc')
+        self.default = 'iscc'
+
+    __name__ = 'inno setup script'
 
 #-----------------------------------------------------------------------------#
 
@@ -404,92 +499,115 @@ class KornHandler(FileTypeHandler):
         self.commands = dict(ksh='ksh')
         self.default = 'ksh'
 
-    @property
-    def __name__(self):
-        return 'korn shell'
+    __name__ = 'korn shell'
+
+#-----------------------------------------------------------------------------#
+
+class LatexHandler(FileTypeHandler):
+    """FileTypeHandler for LaTex"""
+    def __init__(self):
+        FileTypeHandler.__init__(self)
+        self.commands = dict(latex='latex', dvips='dvips',
+                             pdflatex='pdflatex', ps2pdf='ps2pdf',
+                             dvipng='dvipng', latex2html='latex2html')
+        self.default = 'latex'
+
+    __name__ = 'LaTex'
 
 #-----------------------------------------------------------------------------#
 
 class LuaHandler(FileTypeHandler):
     """FileTypeHandler for Lua"""
     RE_LUA_ERROR = re.compile('.*: (.+):([0-9]+):.*')
+
     def __init__(self):
         FileTypeHandler.__init__(self)
         self.commands = dict(lua='lua', luac='luac')
         self.default = 'lua'
 
-    @property
-    def __name__(self):
-        return 'lua'
+    __name__ = 'lua'
 
     def HandleHotSpot(self, mainw, outbuffer, line, fname):
         """Hotspots are error messages, find the file/line of the
         error in question and open the file to that point in the buffer.
 
         """
-        ifile, line = _FindFileLine(outbuffer, line, fname, self.RE_LUA_ERROR)
+        ifile, line = _FindFileLine(outbuffer, line, fname,
+                                    LuaHandler.RE_LUA_ERROR)
         _OpenToLine(ifile, line, mainw)
 
     def StyleText(self, stc, start, txt):
         """Style NSIS output messages"""
-        if _StyleError(stc, start, txt, self.RE_LUA_ERROR):
+        if _StyleError(stc, start, txt, LuaHandler.RE_LUA_ERROR):
             return
         else:
             # Highlight Start end lines this is what the
             # base classes method does.
-            FileTypeHandler.StyleText(self, stc, start, txt)
+            super(LuaHandler, self).StyleText(stc, start, txt)
+
+#-----------------------------------------------------------------------------#
+
+class NewLispHandler(FileTypeHandler):
+    """FileTypeHandler for newLisp"""
+
+    def __init__(self):
+        FileTypeHandler.__init__(self)
+        self.commands = dict(newlisp='newlisp')
+        self.default = 'newlisp'
+
+    __name__ = 'newlisp'
 
 #-----------------------------------------------------------------------------#
 
 class NSISHandler(FileTypeHandler):
     """FileTypeHandler for NSIS scripts"""
     RE_NSIS_ERROR = re.compile(r'Error .* "(.+)" on line ([0-9]+) ')
+
     def __init__(self):
         FileTypeHandler.__init__(self)
         self.commands = dict(makensis='makensis')
         self.default = 'makensis'
 
-    @property
-    def __name__(self):
-        return 'nsis'
+    __name__ = 'nsis'
 
     def HandleHotSpot(self, mainw, outbuffer, line, fname):
         """Hotspots are error messages, find the file/line of the
         error in question and open the file to that point in the buffer.
 
         """
-        ifile, line = _FindFileLine(outbuffer, line, fname, self.RE_NSIS_ERROR)
+        ifile, line = _FindFileLine(outbuffer, line, fname,
+                                    NSISHandler.RE_NSIS_ERROR)
         _OpenToLine(ifile, line, mainw)
 
     def StyleText(self, stc, start, txt):
         """Style NSIS output messages"""
-        if _StyleError(stc, start, txt, self.RE_NSIS_ERROR):
+        if _StyleError(stc, start, txt, NSISHandler.RE_NSIS_ERROR):
             return
         else:
             # Highlight Start end lines this is what the
             # base classes method does.
-            FileTypeHandler.StyleText(self, stc, start, txt)
+            super(NSISHandler, self).StyleText(stc, start, txt)
 
 #-----------------------------------------------------------------------------#
 
 class PhpHandler(FileTypeHandler):
     """FileTypeHandler for Php"""
     RE_PHP_ERROR = re.compile(r'[a-zA-Z]+ error: .* in (.+) on line ([0-9]+).*')
+
     def __init__(self):
         FileTypeHandler.__init__(self)
         self.commands = dict(php='php -f')
         self.default = 'php'
 
-    @property
-    def __name__(self):
-        return 'php'
+    __name__ = 'php'
 
     def HandleHotSpot(self, mainw, outbuffer, line, fname):
         """Hotspots are error messages, find the file/line of the
         error in question and open the file to that point in the buffer.
 
         """
-        ifile, line = _FindFileLine(outbuffer, line, fname, self.RE_PHP_ERROR)
+        ifile, line = _FindFileLine(outbuffer, line, fname,
+                                    PhpHandler.RE_PHP_ERROR)
         _OpenToLine(ifile, line, mainw)
 
     def StyleText(self, stc, start, txt):
@@ -497,12 +615,12 @@ class PhpHandler(FileTypeHandler):
         output.
 
         """
-        if _StyleError(stc, start, txt, self.RE_PHP_ERROR):
+        if _StyleError(stc, start, txt, PhpHandler.RE_PHP_ERROR):
             return
         else:
             # Highlight Start end lines this is what the
             # base classes method does.
-            FileTypeHandler.StyleText(self, stc, start, txt)
+            super(PhpHandler, self).StyleText(stc, start, txt)
 
 #-----------------------------------------------------------------------------#
 
@@ -513,30 +631,28 @@ class PikeHandler(FileTypeHandler):
         self.commands = dict(pike='pike')
         self.default = 'pike'
 
-    @property
-    def __name__(self):
-        return 'pike'
+    __name__ = 'pike'
 
 #-----------------------------------------------------------------------------#
 
 class PerlHandler(FileTypeHandler):
     """FileTypeHandler for Perl scripts"""
     RE_PERL_ERROR = re.compile(r'.+ at (.+) line ([0-9]+)[,\.].*')
+
     def __init__(self):
         FileTypeHandler.__init__(self)
         self.commands = dict(perl='perl')
         self.default = 'perl'
 
-    @property
-    def __name__(self):
-        return 'perl'
+    __name__ = 'perl'
 
     def HandleHotSpot(self, mainw, outbuffer, line, fname):
         """Hotspots are error messages, find the file/line of the
         error in question and open the file to that point in the buffer.
 
         """
-        ifile, line = _FindFileLine(outbuffer, line, fname, self.RE_PERL_ERROR)
+        ifile, line = _FindFileLine(outbuffer, line, fname,
+                                    PerlHandler.RE_PERL_ERROR)
         _OpenToLine(ifile, line, mainw)
 
     def StyleText(self, stc, start, txt):
@@ -544,12 +660,12 @@ class PerlHandler(FileTypeHandler):
         output.
 
         """
-        if _StyleError(stc, start, txt, self.RE_PERL_ERROR):
+        if _StyleError(stc, start, txt, PerlHandler.RE_PERL_ERROR):
             return
         else:
             # Highlight Start end lines this is what the
             # base classes method does.
-            FileTypeHandler.StyleText(self, stc, start, txt)
+            super(PerlHandler, self).StyleText(stc, start, txt)
 
 #-----------------------------------------------------------------------------#
 
@@ -567,24 +683,22 @@ class PostScriptHandler(FileTypeHandler):
             self.commands = dict(gs='gs')
             self.default = 'gs'
 
-    @property
-    def __name__(self):
-        return 'postscript'
+    __name__ = 'postscript'
 
 #-----------------------------------------------------------------------------#
 
 class PythonHandler(FileTypeHandler):
     """FileTypeHandler for Python"""
-    RE_PY_ERROR = re.compile('File "(.+)", line ([0-9]+)')
+    RE_PY_ERROR = re.compile('File "(.+)", line ([0-9]+)') # Python Exceptions
+    RE_PL_ERR = re.compile(r'\s*([REWCF]):\s*([0-9]+):.*') # Pylint Output
 
     def __init__(self):
         FileTypeHandler.__init__(self)
-        self.commands = dict(python='python', pylint='pylint')
+        self.commands = dict(python='python -u', pylint='pylint',
+                             pylinterr='pylint -e')
         self.default = 'python'
 
-    @property
-    def __name__(self):
-        return 'python'
+    __name__ = 'python'
 
     def GetEnvironment(self):
         """Get the environment to run the python script in"""
@@ -601,65 +715,85 @@ class PythonHandler(FileTypeHandler):
         error in question and open the file to that point in the buffer.
 
         """
-        ifile, line = _FindFileLine(outbuffer, line, fname, self.RE_PY_ERROR)
-        _OpenToLine(ifile, line, mainw)
+        match = PythonHandler.RE_PL_ERR.match(outbuffer.GetLine(line))
+        if match:
+            eline = max(0, int(match.group(2)) - 1)
+            _OpenToLine(fname, eline, mainw)
+        else:
+            ifile, line = _FindFileLine(outbuffer, line, fname,
+                                        PythonHandler.RE_PY_ERROR)
+            _OpenToLine(ifile, line, mainw)
 
     def StyleText(self, stc, start, txt):
         """Style python Information and Error messages from script
         output.
 
         """
-        if _StyleError(stc, start, txt, self.RE_PY_ERROR):
+        if _StyleError(stc, start, txt, PythonHandler.RE_PY_ERROR) or \
+           _StyleError(stc, start, txt, PythonHandler.RE_PL_ERR):
             return
         else:
             # Highlight Start end lines this is what the
             # base classes method does.
-            FileTypeHandler.StyleText(self, stc, start, txt)
+            super(PythonHandler, self).StyleText(stc, start, txt)
+
+#-----------------------------------------------------------------------------#
+
+class RHandler(FileTypeHandler):
+    """FileTypeHandler for R"""
+    def __init__(self):
+        FileTypeHandler.__init__(self)
+        self.commands = {'r' : 'R',
+                         'Rterm' : 'Rterm',
+                         'Rgui' : 'Rgui',
+                         'Rscript' : 'Rscript'}
+        self.default = 'Rscript'
+
+    __name__ = 'R'
 
 #-----------------------------------------------------------------------------#
 
 class RubyHandler(FileTypeHandler):
     """FileTypeHandler for Ruby scripts"""
     RE_RUBY_ERROR = re.compile('(.+):([0-9]+)[:]{0,1}.*')
+
     def __init__(self):
         FileTypeHandler.__init__(self)
         self.commands = dict(ruby='ruby')
         self.default = 'ruby'
 
-    @property
-    def __name__(self):
-        return 'ruby'
+    __name__ = 'ruby'
 
     def HandleHotSpot(self, mainw, outbuffer, line, fname):
         """Hotspots are error messages, find the file/line of the
         error in question and open the file to that point in the buffer.
 
         """
-        ifile, line = _FindFileLine(outbuffer, line, fname, self.RE_RUBY_ERROR)
+        ifile, line = _FindFileLine(outbuffer, line, fname,
+                                    RubyHandler.RE_RUBY_ERROR)
         _OpenToLine(ifile, line, mainw)
 
     def StyleText(self, stc, start, txt):
         """Style NSIS output messages"""
-        if _StyleError(stc, start, txt, self.RE_RUBY_ERROR):
+        if _StyleError(stc, start, txt, RubyHandler.RE_RUBY_ERROR):
             return
         else:
             # Highlight Start end lines this is what the
             # base classes method does.
-            FileTypeHandler.StyleText(self, stc, start, txt)
+            super(RubyHandler, self).StyleText(stc, start, txt)
 
 #-----------------------------------------------------------------------------#
 
 class TCLHandler(FileTypeHandler):
     """FileTypeHandler for TCL/TK"""
     RE_TCL_ERROR = re.compile('\(file "(.+)" line ([0-9]+)\)')
+
     def __init__(self):
         FileTypeHandler.__init__(self)
         self.commands = dict(wish='wish')
         self.default = 'wish'
 
-    @property
-    def __name__(self):
-        return 'tcl/tk'
+    __name__ = 'tcl/tk'
 
     def HandleHotSpot(self, mainw, outbuffer, line, fname):
         """Hotspots are error messages, find the file/line of the
@@ -667,13 +801,13 @@ class TCLHandler(FileTypeHandler):
 
         """
         txt = outbuffer.GetLine(line)
-        match = self.RE_TCL_ERROR.findall(txt)
+        match = TCLHandler.RE_TCL_ERROR.findall(txt)
         ifile = None
         if len(match):
             ifile = match[0][0]
             try:
                 line = max(int(match[0][1]) - 1, 0)
-            except:
+            except IndexError:
                 line = 0
 
         # If not an absolute path then the error is in the current script
@@ -685,34 +819,76 @@ class TCLHandler(FileTypeHandler):
 
     def StyleText(self, stc, start, txt):
         """Style NSIS output messages"""
-        if _StyleError(stc, start, txt, self.RE_TCL_ERROR):
+        if _StyleError(stc, start, txt, TCLHandler.RE_TCL_ERROR):
             return
         else:
             # Highlight Start end lines this is what the
             # base classes method does.
-            FileTypeHandler.StyleText(self, stc, start, txt)
+            super(TCLHandler, self).StyleText(stc, start, txt)
+
+#-----------------------------------------------------------------------------#
+
+class VBScriptHandler(FileTypeHandler):
+    """FileTypeHandler for VBScript"""
+    RE_VBS_ERROR = re.compile('(.+)\(([0-9]+).*' + os.linesep)
+
+    def __init__(self):
+        FileTypeHandler.__init__(self)
+        self.commands = dict(cscript='CSCRIPT.exe', wscript='WSCRIPT.exe')
+        self.default = 'cscript'
+
+    __name__ = 'VBScript'
+
+    def HandleHotSpot(self, mainw, outbuffer, line, fname):
+        """Hotspots are error messages, find the file/line of the
+        error in question and open the file to that point in the
+        buffer.
+
+        """
+        ifile, line = _FindFileLine(outbuffer, line, fname,
+                                    VBScriptHandler.RE_VBS_ERROR)
+        _OpenToLine(ifile, line, mainw)
+
+    def StyleText(self, stc, start, txt):
+        """Style VBScript Information and Error messages from script output."""
+        if _StyleError(stc, start, txt, VBScriptHandler.RE_VBS_ERROR):
+            return
+        else:
+            # Highlight Start end lines this is what the
+            # base classes method does.
+            super(VBScriptHandler, self).StyleText(stc, start, txt)
 
 #-----------------------------------------------------------------------------#
 # Handler Object Dictionary
-# Create an instance of each Handler to use as a persistant object
-HANDLERS = { 0 : FileTypeHandler(), # Null Hanlder
-            synglob.ID_LANG_ADA : AdaHandler(),
-            synglob.ID_LANG_BASH : BashHandler(),
-            synglob.ID_LANG_BOO : BooHandler(),
-            synglob.ID_LANG_CSH : CSHHandler(),
-            synglob.ID_LANG_FERITE : FeriteHandler(),
-            synglob.ID_LANG_KSH : KornHandler(),
+# Create an instance of each Handler to use as a persistent object
+HANDLERS = { 0                      : FileTypeHandler(), # Null Handler
+            synglob.ID_LANG_ADA     : AdaHandler(),
+            synglob.ID_LANG_BASH    : BashHandler(),
+            synglob.ID_LANG_BATCH   : BatchHandler(),
+            synglob.ID_LANG_BOO     : BooHandler(),
+            synglob.ID_LANG_CAML    : CamlHandler(),
+            synglob.ID_LANG_CSH     : CSHHandler(),
+            synglob.ID_LANG_D       : DHandler(),
+            synglob.ID_LANG_FERITE  : FeriteHandler(),
+            synglob.ID_LANG_KSH     : KornHandler(),
             synglob.ID_LANG_HASKELL : HaskellHandler(),
-            synglob.ID_LANG_HAXE : HaxeHandler(),
-            synglob.ID_LANG_LUA : LuaHandler(),
-            synglob.ID_LANG_NSIS : NSISHandler(),
-            synglob.ID_LANG_PERL : PerlHandler(),
-            synglob.ID_LANG_PHP : PhpHandler(),
-            synglob.ID_LANG_PIKE : PikeHandler(),
-            synglob.ID_LANG_PS   : PostScriptHandler(),
-            synglob.ID_LANG_PYTHON : PythonHandler(),
-            synglob.ID_LANG_RUBY : RubyHandler(),
-            synglob.ID_LANG_TCL : TCLHandler() }
+            synglob.ID_LANG_HAXE    : HaxeHandler(),
+            synglob.ID_LANG_HTML    : HTMLHandler(),
+            synglob.ID_LANG_INNO    : InnoSetupHandler(),
+            synglob.ID_LANG_LATEX   : LatexHandler(),
+            synglob.ID_LANG_LUA     : LuaHandler(),
+            synglob.ID_LANG_NEWLISP : NewLispHandler(),
+            synglob.ID_LANG_NSIS    : NSISHandler(),
+            synglob.ID_LANG_PERL    : PerlHandler(),
+            synglob.ID_LANG_PHP     : PhpHandler(),
+            synglob.ID_LANG_PIKE    : PikeHandler(),
+            synglob.ID_LANG_PS      : PostScriptHandler(),
+            synglob.ID_LANG_PYTHON  : PythonHandler(),
+            synglob.ID_LANG_R       : RHandler(),
+            synglob.ID_LANG_RUBY    : RubyHandler(),
+            synglob.ID_LANG_TEX     : LatexHandler(),
+            synglob.ID_LANG_TCL     : TCLHandler(),
+            synglob.ID_LANG_VBSCRIPT : VBScriptHandler() }
 
 #-----------------------------------------------------------------------------#
 # Local utility functions
@@ -735,7 +911,7 @@ def _FindFileLine(outbuffer, line, fname, regex):
         ifile = match[0][0]
         try:
             line = max(int(match[0][1]) - 1, 0)
-        except:
+        except (IndexError, TypeError):
             line = 0
 
     # If not an absolute path then the error is relative to the
@@ -754,15 +930,15 @@ def _OpenToLine(fname, line, mainw):
     @param mainw: MainWindow instance to open the file in
 
     """
-    nb = mainw.GetNotebook()
-    buffers = [ page.GetFileName() for page in nb.GetTextControls() ]
+    nbook = mainw.GetNotebook()
+    buffers = [ page.GetFileName() for page in nbook.GetTextControls() ]
     if fname in buffers:
         page = buffers.index(fname)
-        nb.SetSelection(page)
-        nb.GetPage(page).GotoLine(line)
+        nbook.ChangePage(page)
+        nbook.GetPage(page).GotoLine(line)
     else:
-        nb.OnDrop([fname])
-        nb.GetPage(nb.GetSelection()).GotoLine(line)
+        nbook.OnDrop([fname])
+        nbook.GetPage(nbook.GetSelection()).GotoLine(line)
 
 def _StyleError(stc, start, txt, regex):
     """Style Error message groups
