@@ -18,8 +18,8 @@ element is defined in the file.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: cbrowser.py 54518 2008-07-06 19:40:25Z CJP $"
-__revision__ = "$Revision: 54518 $"
+__svnid__ = "$Id: cbrowser.py 60379 2009-04-26 05:24:41Z CJP $"
+__revision__ = "$Revision: 60379 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -52,7 +52,8 @@ STRINGS = ( _("Class Definitions"), _("Defines"), _("Function Definitions"),
             _("Procedure Definitions"), _("Programs"), _("Protocols"),
             _("Sections"), _("Style Tags"), _("Subroutines"),
             _("Subroutine Declarations"), _("Task Definitions"), 
-            _("Modules") )
+            _("Modules"), _("Functions"), _("Public Functions"),
+            _("Public Subroutines") )
 del STRINGS
 
 #--------------------------------------------------------------------------#
@@ -73,6 +74,10 @@ class CodeBrowserTree(wx.TreeCtrl):
         self.icons = dict()
         self.il = None
 
+        self._timer = wx.Timer(self)
+        self._cpage = None
+        self._force = False
+
         # Setup
         self._SetupImageList()
         viewm = self._mw.GetMenuBar().GetMenuByName("view")
@@ -90,6 +95,7 @@ class CodeBrowserTree(wx.TreeCtrl):
         self.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.OnActivated)
         self.Bind(wx.EVT_TREE_ITEM_RIGHT_CLICK, self.OnContext)
         self.Bind(wx.EVT_MENU, self.OnMenu)
+        self.Bind(wx.EVT_TIMER, self.OnStartJob)
         self.Bind(EVT_JOB_FINISHED, self.OnTagsReady)
         ed_msg.Subscribe(self.OnThemeChange, ed_msg.EDMSG_THEME_CHANGED)
         ed_msg.Subscribe(self.OnUpdateTree, ed_msg.EDMSG_UI_NB_CHANGED)
@@ -115,21 +121,27 @@ class CodeBrowserTree(wx.TreeCtrl):
     def _SetupImageList(self):
         """Setup the image list for the tree"""
         imglst = wx.ImageList(16, 16)
-        if Profile_Get('ICONS', 'Default') != 'Default':
-            globe = wx.ArtProvider.GetBitmap(str(ed_glob.ID_WEB), wx.ART_MENU)
-            self.icons['globals'] = imglst.Add(globe)
-        else:
-            self.icons['globals'] = imglst.Add(IconFile.GetGlobalBitmap())
-        self.icons['class'] = imglst.Add(IconFile.GetBricksBitmap())
-        self.icons['section'] = imglst.Add(IconFile.GetBrickAddBitmap())
-        self.icons['function'] = imglst.Add(IconFile.GetBrickGoBitmap())
-        self.icons['method'] = self.icons['function']
+
+        globe = wx.ArtProvider.GetBitmap(str(ed_glob.ID_WEB), wx.ART_MENU)
+        self.icons['globals'] = imglst.Add(globe)
+
+        bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_CLASS_TYPE), wx.ART_MENU)
+        self.icons['class'] = imglst.Add(bmp)
+        self.icons['section'] = imglst.Add(IconFile.Brick_Add.GetBitmap())
+        bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_FUNCT_TYPE), wx.ART_MENU)
+        self.icons['function'] = imglst.Add(bmp)
+        bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_METHOD_TYPE), wx.ART_MENU)
+        self.icons['method'] = imglst.Add(bmp)
         self.icons['subroutine'] = self.icons['function']
         self.icons['procedure'] = self.icons['function']
-        self.icons['task'] = imglst.Add(IconFile.GetBrickGo2Bitmap())
+        self.icons['task'] = imglst.Add(bmp)
         self.icons['function2'] = self.icons['task']
-        self.icons['variable'] = imglst.Add(IconFile.GetBrickBitmap())
-        self.icons['namespace'] = imglst.Add(IconFile.GetBrickBricksBitmap())
+        bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_VARIABLE_TYPE), wx.ART_MENU)
+        self.icons['variable'] = imglst.Add(bmp)
+        self.icons['namespace'] = imglst.Add(IconFile.Brick_Bricks.GetBitmap())
+        self.icons['tag_red'] = imglst.Add(IconFile.Tag_Red.GetBitmap())
+        self.icons['tag_blue'] = imglst.Add(IconFile.Tag_Blue.GetBitmap())
+        self.icons['tag_green'] = imglst.Add(IconFile.Tag_Green.GetBitmap())
         self.SetImageList(imglst)
         # NOTE: Must save reference to the image list or tree will crash!!!
         self.il = imglst
@@ -275,6 +287,24 @@ class CodeBrowserTree(wx.TreeCtrl):
             self._menu.Append(ID_GOTO_ELEMENT, _("Goto \"%s\"") % txt)
             self.PopupMenu(self._menu)
 
+    def OnCompareItems(self, item1, item2):
+        """Compare two tree items for sorting.
+        @param item1: wx.TreeItem
+        @param item2: wx.TreeItem
+        @return: -1, 0, 1
+
+        """
+        txt1 = self.GetItemText(item1).lower()
+        txt2 = self.GetItemText(item2).lower()
+#        txt1 = self.GetPyData(item1)
+#        txt2 = self.GetPyData(item2)
+        if txt1 < txt2:
+            return -1
+        elif txt1 == txt2:
+            return 0
+        else:
+            return 1
+
     def OnMenu(self, evt):
         """Handle the context menu events"""
         if evt.GetId() == ID_GOTO_ELEMENT:
@@ -308,6 +338,9 @@ class CodeBrowserTree(wx.TreeCtrl):
             ed_msg.PostMessage(ed_msg.EDMSG_PROGRESS_STATE,
                                (self._mw.GetId(), 0, 0))
 
+            if not self._timer.IsRunning():
+                self._cpage = None
+
     def OnUpdateFont(self, msg):
         """Update the ui font when a message comes saying to do so."""
         font = msg.GetData()
@@ -323,36 +356,37 @@ class CodeBrowserTree(wx.TreeCtrl):
         pane = self._mw.GetFrameManager().GetPane(PANE_NAME)
         evt.Check(pane.IsShown())
 
-    def OnUpdateTree(self, msg=None):
-        """Update the tree when an action message is sent
-        @param keyword: Message Object
+    def OnStartJob(self, evt):
+        """Start the tree update job
+        @param evt: wxTimerEvent
 
         """
-        # Don't update when this window is not Active
-        if self._mw != wx.GetApp().GetActiveWindow():
-            return
-
-        page = self._GetCurrentCtrl()
-
-        # If document job is same as current don't start a new one
-        if self._lastjob == page.GetFileName():
+        if self._cpage is None or not isinstance(self._cpage, wx.Window):
+            self._cpage = None
             return
         else:
-            self._lastjob = page.GetFileName()
+            # Check if its still the current page
+            parent = self._cpage.GetParent()
+            if self._cpage != parent.GetCurrentPage():
+                return
 
         # Get the generator method
-        genfun = TagLoader.GetGenerator(page.GetLangId())
+        genfun = TagLoader.GetGenerator(self._cpage.GetLangId())
         self._cjob += 1 # increment job Id
 
         # Check if we need to do updates
-        if genfun is not None and self._ShouldUpdate():
+        if genfun is not None and (self._force or self._ShouldUpdate()):
+            self._force = False
+
+            # Start progress indicator in pulse mode
             ed_msg.PostMessage(ed_msg.EDMSG_PROGRESS_SHOW,
                                (self._mw.GetId(), True))
-            # Start progress indicator in pulse mode
             ed_msg.PostMessage(ed_msg.EDMSG_PROGRESS_STATE,
                                (self._mw.GetId(), -1, -1))
+
+            # Create and start the worker thread
             thread = TagGenThread(self, self._cjob, genfun,
-                                  StringIO.StringIO(page.GetText()))
+                                  StringIO.StringIO(self._cpage.GetText()))
             wx.CallLater(75, thread.start)
         else:
             self._cdoc = None
@@ -361,6 +395,39 @@ class CodeBrowserTree(wx.TreeCtrl):
                                (self._mw.GetId(), False))
             return
 
+    def OnUpdateTree(self, msg=None, force=False):
+        """Update the tree when an action message is sent
+        @keyword msg: Message Object
+        @keyword force: Force update
+
+        """
+        # Don't update when this window is not Active
+        istop = wx.GetApp().GetTopWindow() == self._mw
+        if not force and not self._mw.IsActive() and not istop:
+            return
+
+        page = self._GetCurrentCtrl()
+        cfname = page.GetFileName()
+
+        # If its a blank document just clear out
+        if not len(cfname):
+            self._cdoc = None
+            self.DeleteChildren(self.root)
+            return
+
+        # If document job is same as current don't start a new one
+        if force or self._lastjob != cfname:
+            self._lastjob = cfname
+            if self._timer.IsRunning():
+                self._timer.Stop()
+
+            # Cache the current job information
+            self._cpage = page
+            self._force = force
+
+            # Start the oneshot timer for beginning the tag generator job
+            self._timer.Start(300, True)
+
     def OnShowBrowser(self, evt):
         """Show the browser pane
         @param evt: wx.MenuEvent
@@ -371,7 +438,7 @@ class CodeBrowserTree(wx.TreeCtrl):
             pane = mgr.GetPane(PANE_NAME)
             pane.Show(not pane.IsShown())
             mgr.Update()
-            self.OnUpdateTree()
+            self.OnUpdateTree(force=True)
         else:
             evt.Skip()
 
