@@ -14,8 +14,8 @@ FileBrowser Plugin.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: browser.py 53543 2008-05-10 18:32:25Z CJP $"
-__revision__ = "$Revision: 53543 $"
+__svnid__ = "$Id: browser.py 60285 2009-04-22 18:25:17Z CJP $"
+__revision__ = "$Revision: 60285 $"
 
 #-----------------------------------------------------------------------------#
 # Imports
@@ -32,13 +32,13 @@ import wx
 import ed_glob
 import ed_msg
 import ed_menu
+import ed_mdlg
 import syntax.syntax
 import util
-from profiler import Profile_Get, Profile_Set
-from eclib import platebtn
+import eclib
+import ebmlib
 
 # Local Imports
-import FileInfo
 import Trash
 
 #-----------------------------------------------------------------------------#
@@ -66,14 +66,19 @@ def TrashString():
 _ = wx.GetTranslation
 #-----------------------------------------------------------------------------#
 
-class BrowserMenuBar(wx.Panel):
+ID_MARK_PATH = wx.NewId()
+ID_OPEN_MARK = wx.NewId()
+ID_REMOVE_MARK = wx.NewId()
+
+class BrowserMenuBar(eclib.ControlBar):
     """Creates a menubar with """
-    ID_MARK_PATH = wx.NewId()
-    ID_OPEN_MARK = wx.NewId()
-    ID_REMOVE_MARK = wx.NewId()
 
     def __init__(self, parent):
-        wx.Panel.__init__(self, parent, style=wx.NO_BORDER)
+        eclib.ControlBar.__init__(self, parent,
+                                  style=eclib.CTRLBAR_STYLE_GRADIENT)
+
+        if wx.Platform == '__WXGTK__':
+            self.SetWindowStyle(eclib.CTRLBAR_STYLE_DEFAULT)
 
         # Attributes
         self._saved = ed_menu.EdMenu()
@@ -92,37 +97,24 @@ class BrowserMenuBar(wx.Panel):
 
         # Build Menus
         menu = ed_menu.EdMenu()
-        menu.Append(self.ID_MARK_PATH, _("Save Selected Paths"))
-        menu.AppendMenu(self.ID_OPEN_MARK, 
-                        _("Jump to Saved Path"), self._saved)
+        menu.Append(ID_MARK_PATH, _("Save Selected Paths"))
+        menu.AppendMenu(ID_OPEN_MARK, _("Jump to Saved Path"), self._saved)
         menu.AppendSeparator()
-        menu.AppendMenu(self.ID_REMOVE_MARK, 
-                        _("Remove Saved Path"), self._rmpath)
+        menu.AppendMenu(ID_REMOVE_MARK, _("Remove Saved Path"), self._rmpath)
 
         # Button
         bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_ADD_BM), wx.ART_MENU)
-        self.menub = platebtn.PlateButton(self, bmp=bmp,
-                                          style=platebtn.PB_STYLE_NOBG)
+        self.menub = eclib.PlateButton(self, bmp=bmp,
+                                       style=eclib.PB_STYLE_NOBG)
         self.menub.SetToolTipString(_("Pathmarks"))
         self.menub.SetMenu(menu)
 
-
         # Layout bar
-        sizer = wx.BoxSizer(wx.VERTICAL)
-        sizer.Add((1, 1))
-        men_sz = wx.BoxSizer(wx.HORIZONTAL)
-        men_sz.Add((6, 6))
-        men_sz.Add(self.menub, 0, wx.ALIGN_LEFT)
-        sizer.Add(men_sz)
-        sizer.Add((1, 1))
-        self.SetSizer(sizer)
+        self.AddControl(self.menub, wx.ALIGN_LEFT)
 
         # Event Handlers
         ed_msg.Subscribe(self.OnThemeChanged, ed_msg.EDMSG_THEME_CHANGED)
         self.Bind(wx.EVT_BUTTON, lambda evt: self.menub.ShowMenu(), self.menub)
-        # Due to transparency issues dont do painting on gtk
-        if wx.Platform != '__WXGTK__':
-            self.Bind(wx.EVT_PAINT, self.OnPaint)
 
     def __del__(self):
         """Unsubscribe from messages"""
@@ -168,25 +160,6 @@ class BrowserMenuBar(wx.Panel):
         """Returns the menu containg the saved items"""
         return self._saved
 
-    def OnPaint(self, evt):
-        """Paints the background of the menubar"""
-        dc = wx.PaintDC(self)
-        gc = wx.GraphicsContext.Create(dc)
-        col1 = util.AdjustColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE), -15)
-        col2 = util.AdjustColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE), 40)
-        rect = self.GetRect()
-        grad = gc.CreateLinearGradientBrush(0, 1, 0, rect.height, col2, col1)
-
-        # Create the background path
-        path = gc.CreatePath()
-        path.AddRectangle(0, 0, rect.width - 0.5, rect.height - 0.5)
-
-        gc.SetPen(wx.Pen(util.AdjustColour(col1, -20), 1))
-        gc.SetBrush(grad)
-        gc.DrawPath(path)
-
-        evt.Skip()
-
     def OnThemeChanged(self, msg):
         """Update the buttons icon when the icon theme changes
         @param msg: Message Object
@@ -213,34 +186,46 @@ class BrowserMenuBar(wx.Panel):
 
 #-----------------------------------------------------------------------------#
 
-class BrowserPane(wx.Panel):
+class BrowserPane(eclib.ControlBox):
     """Creates a filebrowser Pane"""
     ID_SHOW_HIDDEN = wx.NewId()
 
     def __init__(self, parent, id, pos=wx.DefaultPosition,
-                 size=wx.DefaultSize, style=wx.NO_BORDER):
-        wx.Panel.__init__(self, parent, id, pos, size, style)
+                 size=wx.DefaultSize, style=wx.NO_BORDER|wx.TAB_TRAVERSAL):
+        eclib.ControlBox.__init__(self, parent, id, pos, size, style)
         
         # Attributes
         self._mw = parent
-        self._sizer = wx.BoxSizer(wx.VERTICAL)
-        filters = "".join(syntax.syntax.GenFileFilters())
+        filters = u"".join(syntax.syntax.GenFileFilters())
         self._menbar = BrowserMenuBar(self)
         self._browser = FileBrowser(self, ID_FILEBROWSE, 
-                                    dir=wx.GetHomeDir(), 
-                                    size=(200, -1),
+                                    dir=wx.GetHomeDir(),
                                     style=wx.DIRCTRL_SHOW_FILTERS | 
                                           wx.DIRCTRL_EDIT_LABELS | 
                                           wx.BORDER_SUNKEN,
                                     filter=filters)
+        self._browser.SetMainWindow(self._mw)
         self._config = PathMarkConfig(ed_glob.CONFIG['CACHE_DIR'])
         for item in self._config.GetItemLabels():
             self._menbar.AddItem(item)
-        self._showh_cb = wx.CheckBox(self, self.ID_SHOW_HIDDEN, 
+
+        # Setup hidden files checkbox
+        bbar = eclib.ControlBar(self, style=eclib.CTRLBAR_STYLE_GRADIENT)
+        if wx.Platform == '__WXGTK__':
+            bbar.SetWindowStyle(eclib.CTRLBAR_STYLE_DEFAULT)
+
+        self._showh_cb = wx.CheckBox(bbar, self.ID_SHOW_HIDDEN, 
                                      _("Show Hidden Files"))
         self._showh_cb.SetValue(False)
-        if wx.Platform == '__WXMAC__':
-            self._showh_cb.SetWindowVariant(wx.WINDOW_VARIANT_SMALL)
+        bbar.AddControl(self._showh_cb, wx.ALIGN_LEFT)
+
+        # Layout
+        self.SetWindow(self._browser)
+        self.SetControlBar(self._menbar, wx.TOP)
+        self.SetControlBar(bbar, wx.BOTTOM)
+        if wx.Platform == '__WXMSW__':
+            bbar.Hide()
+        self.Layout()
 
         #---- Add Menu Items ----#
         viewm = self._mw.GetMenuBar().GetMenuByName("view")
@@ -249,21 +234,9 @@ class BrowserPane(wx.Panel):
                                      wx.ITEM_CHECK,
                                      after=ed_glob.ID_PRE_MARK)
 
-        # Layout Pane
-        self._sizer.AddMany([(self._menbar, 0, wx.EXPAND),
-                             (self._browser, 1, wx.EXPAND),
-                             ((2, 2))])
-        cb_sz = wx.BoxSizer(wx.HORIZONTAL)
-        cb_sz.Add((4, 4))
-        cb_sz.Add(self._showh_cb, 0, wx.ALIGN_LEFT)
-        self._sizer.Add(cb_sz, 0, wx.ALIGN_LEFT)
-        self._sizer.Add((3, 3))
-        self.SetSizer(self._sizer)
-
         # Event Handlers
         self.Bind(wx.EVT_CHECKBOX, self.OnCheck)
         self.Bind(wx.EVT_MENU, self.OnMenu)
-        self.Bind(wx.EVT_PAINT, self.OnPaint)
 
         # Messages
         ed_msg.Subscribe(self.OnUpdateFont, ed_msg.EDMSG_DSP_FONT)
@@ -273,6 +246,10 @@ class BrowserPane(wx.Panel):
         self._config.Save()
         ed_msg.Unsubscribe(self.OnUpdateFont)
 
+    def GetMainWindow(self):
+        """Get the MainWindow that owns this panel"""
+        return self._mw
+
     def OnCheck(self, evt):
         """Toggles visibility of hidden files on and off"""
         e_id = evt.GetId()
@@ -296,7 +273,7 @@ class BrowserPane(wx.Panel):
         e_id = evt.GetId()
         o_ids = self._menbar.GetOpenIds()
         d_ids = self._menbar.GetRemoveIds()
-        if e_id == self._menbar.ID_MARK_PATH:
+        if e_id == ID_MARK_PATH:
             items = self._browser.GetPaths()
             for item in items:
                 self._menbar.AddItem(item)
@@ -306,7 +283,7 @@ class BrowserPane(wx.Panel):
             pmark = self._menbar.GetItemText(e_id)
             path = self._config.GetPath(pmark)
             self._browser.ExpandPath(path)
-            self._browser.SetFocus()
+            self._browser.GetTreeCtrl().SetFocus()
         elif e_id in d_ids:
             plabel = self._menbar.GetItemText(e_id)
             self._menbar.RemoveItemById(e_id)
@@ -315,30 +292,6 @@ class BrowserPane(wx.Panel):
         else:
             evt.Skip()
 
-    def OnPaint(self, evt):
-        """Paints the background of the panel"""
-        dc = wx.PaintDC(self)
-        gc = wx.GraphicsContext.Create(dc)
-        col1 = util.AdjustColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE), -15)
-        col2 = util.AdjustColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE), 40)
-        rect = self.GetRect()
-        x = 0
-        y = rect.height - (self._showh_cb.GetSize()[1] + 6)
-        grad = gc.CreateLinearGradientBrush(x, y, x, 
-                                            y + self._showh_cb.GetSize()[1] + 6,
-                                            col2, col1)
-
-        # Create the background path
-        path = gc.CreatePath()
-        path.AddRectangle(x, y, rect.width - 0.5, 
-                          self._showh_cb.GetSize()[1] + 6)
-
-        gc.SetPen(wx.Pen(util.AdjustColour(col1, -20), 1))
-        gc.SetBrush(grad)
-        gc.DrawPath(path)
-
-        evt.Skip()
-
     def OnShowBrowser(self, evt):
         """Shows the filebrowser"""
         if evt.GetId() == ID_FILEBROWSE:
@@ -348,6 +301,10 @@ class BrowserPane(wx.Panel):
                 pane.Hide()
             else:
                 pane.Show()
+                cbuff = self._mw.GetNotebook().GetCurrentCtrl()
+                path = cbuff.GetFileName()
+                if path:
+                    self._browser.GotoPath(path)
             mgr.Update()
         else:
             evt.Skip()
@@ -369,10 +326,15 @@ class BrowserPane(wx.Panel):
         pane = self._mw.GetFrameManager().GetPane(PANE_NAME)
         evt.Check(pane.IsShown())
 
+    def SetFocus(self):
+        """Set the focus to this window"""
+        self._browser.GetTreeCtrl().SetFocus()
+
 #-----------------------------------------------------------------------------#
 # Menu Id's
 ID_EDIT = wx.NewId()
 ID_OPEN = wx.NewId()
+ID_SEARCH_DIR = wx.NewId()
 ID_REVEAL = wx.NewId()
 ID_GETINFO = wx.NewId()
 ID_RENAME = wx.NewId()
@@ -392,9 +354,16 @@ class FileBrowser(wx.GenericDirCtrl):
                                    size, style, filter, defaultFilter)
 
         # Attributes
+        self._mw = None
         self._tree = self.GetTreeCtrl()
         self._treeId = 0                # id of TreeItem that was last rclicked
         self._fmenu = self._MakeMenu()
+        self._drag_img = None
+
+        # Show hidden files by default on windows since the option is disabled
+        # due to it crashing out the program...
+        if wx.Platform == '__WXMSW__':
+            self.ShowHidden(True)
         
         # Set custom styles
         self._tree.SetWindowStyle(self._tree.GetWindowStyle() | wx.TR_MULTIPLE)
@@ -408,13 +377,14 @@ class FileBrowser(wx.GenericDirCtrl):
         self.Bind(wx.EVT_MENU, self.OnMenu)
         ed_msg.Subscribe(self.OnThemeChanged, ed_msg.EDMSG_THEME_CHANGED)
         ed_msg.Subscribe(self.OnPageChange, ed_msg.EDMSG_UI_NB_CHANGED)
-#         self.Bind(wx.EVT_TREE_BEGIN_DRAG, self.OnDragStart)
-#         self.Bind(wx.EVT_TREE_END_DRAG, self.OnDragEnd)
+#        self.Bind(wx.EVT_TREE_BEGIN_DRAG, self.OnDragStart)
+#        self.Bind(wx.EVT_MOUSE_EVENTS, self.OnDragging)
+#        self.Bind(wx.EVT_TREE_END_DRAG, self.OnDragEnd)
 
     def __del__(self):
         """Unsubscribe from messages"""
-        ed_msg.Unsubscribe(self.OnThemeChanged)
         ed_msg.Unsubscribe(self.OnPageChange)
+        ed_msg.Unsubscribe(self.OnThemeChanged)
 
     def _SetupIcons(self):
         """If a custom theme is in use by Editra set the icons of the
@@ -432,7 +402,8 @@ class FileBrowser(wx.GenericDirCtrl):
             if bmp.IsOk():
                 self._imglst.Replace(idx, bmp)
 
-    def _MakeMenu(self):
+    @staticmethod
+    def _MakeMenu():
         """Setup the context menu"""
         menu = wx.Menu()
 
@@ -441,6 +412,11 @@ class FileBrowser(wx.GenericDirCtrl):
                  (ID_OPEN, _("Open with " + FILEMAN), ed_glob.ID_OPEN),
                  (ID_REVEAL, _("Reveal in " + FILEMAN), None),
                  (wx.ID_SEPARATOR, '', None),
+                 (ID_MARK_PATH, _("Bookmark Selected Path(s)"),
+                  ed_glob.ID_ADD_BM),
+                 (wx.ID_SEPARATOR, '', None),
+                 (ID_SEARCH_DIR, _("Search in directory"), ed_glob.ID_FIND),
+                 (wx.ID_SEPARATOR, '', None),
                  (ID_GETINFO, _("Get Info"), None),
                  (ID_RENAME, _("Rename"), None),
                  (wx.ID_SEPARATOR, '', None),
@@ -478,6 +454,10 @@ class FileBrowser(wx.GenericDirCtrl):
         self.Refresh()
         return rval
 
+    def AcceptsFocus(self):
+        print "HELLO1"
+        return True
+
     def GetItemPath(self, itemId):
         """Get and return the path of the given item id
         @param itemId: TreeItemId
@@ -499,18 +479,29 @@ class FileBrowser(wx.GenericDirCtrl):
         if wx.Platform == '__WXMSW__':
             r_txt = u''
         else:
+            # Handle localized paths for special nodes on gtk version
             if wx.Platform == '__WXGTK__':
-                if path[0].lower() == 'home directory':
+                if path[0] == _("Home directory"):
                     path[0] = wx.GetHomeDir()
-                elif path[0].lower() == 'desktop':
+                elif path[0] == _("Desktop"):
                     path.insert(0, wx.GetHomeDir())
                 else:
                     pass
 
-            if wx.Platform == '__WXMAC__':
+            elif wx.Platform == '__WXMAC__':
                 if path[0] != "/":
-                    path.pop(0)
+                    if path == 'Macintosh HD':
+                        path.pop(0)
+                    else:
+                        path.insert(0, 'Volumes')
+
+                if u'Macintosh HD' in path:
+                    idx = path.index(u'Macintosh HD')
+                    if len(path) > idx:
+                        path = path[idx+1:]
+
             r_txt = os.path.sep
+
         return r_txt + os.sep.join(path)
 
     def GetPaths(self):
@@ -528,6 +519,18 @@ class FileBrowser(wx.GenericDirCtrl):
         """Returns the trees current style"""
         return self._tree.GetWindowStyle()
 
+    def GotoPath(self, path):
+        """Move the selection to the given path
+        @param path: string:
+
+        """
+        style = self.GetTreeStyle()
+        self.SetTreeStyle(wx.TR_SINGLE)
+        self.Refresh()
+        self.SetPath(path)
+        self.SetTreeStyle(style)
+        self.Refresh()        
+
     def OnContext(self, evt):
         """Show the popup context menu"""
         self._fmenu.Destroy()
@@ -560,10 +563,17 @@ class FileBrowser(wx.GenericDirCtrl):
         elif e_id == ID_REVEAL:
             worker = OpenerThread([os.path.dirname(fname) for fname in paths])
             worker.start()
+        elif e_id == ID_SEARCH_DIR:
+            path = self.GetPath()
+            if path:
+                if not os.path.isdir(path):
+                    path = os.path.dirname(path)
+                mdata = dict(mainw=self._mw, lookin=path)
+                ed_msg.PostMessage(ed_msg.EDMSG_FIND_SHOW_DLG, mdata) 
         elif e_id == ID_GETINFO:
             last = None
             for fname in paths:
-                info = FileInfo.FileInfoDlg(self.GetTopLevelParent(), fname)
+                info = ed_mdlg.EdFileInfoDlg(self.GetTopLevelParent(), fname)
                 if last is None:
                     info.CenterOnParent()
                 else:
@@ -574,9 +584,9 @@ class FileBrowser(wx.GenericDirCtrl):
         elif e_id == ID_RENAME:
             self._tree.EditLabel(self._treeId)
         elif e_id == ID_NEW_FOLDER:
-            ok = util.MakeNewFolder(path, _("Untitled_Folder"))
+            ok = ebmlib.MakeNewFolder(path, _("Untitled_Folder"))
         elif e_id == ID_NEW_FILE:
-            ok = util.MakeNewFile(path, _("Untitled_File") + ".txt")
+            ok = ebmlib.MakeNewFile(path, _("Untitled_File") + ".txt")
         elif e_id == ID_DUPLICATE:
             for fname in paths:
                 ok = DuplicatePath(fname)
@@ -604,19 +614,16 @@ class FileBrowser(wx.GenericDirCtrl):
         files = self.GetPaths()
         item = self._SCommand(self.GetPath)
         if os.path.isdir(item):
-            # Unlike Mac/Gtk, need to skip the event here to allow for
-            # expanding paths with double clicks. This however disables the
-            # ability to use Return as a key for expanding. It also doesn't
-            # seem to be possible to tell whether the activation event was
-            # caused by a double click or a key sequence.
-            if wx.Platform == '__WXMSW__':
-                evt.Skip()
-            else:
-                self.ExpandPath(item)
+            evt.Skip()
             return
         else:
             files = [ fname for fname in files if not os.path.isdir(fname) ]
+
+        # Disconnect from page change notifications while opening files
+        # from the browser.
+        ed_msg.Unsubscribe(self.OnPageChange)
         self.OpenFiles(files)
+        ed_msg.Subscribe(self.OnPageChange, ed_msg.EDMSG_UI_NB_CHANGED)
 
     def OnPageChange(self, msg):
         """Syncronize selection with the notebook page changes
@@ -626,15 +633,24 @@ class FileBrowser(wx.GenericDirCtrl):
         """
         
         nbdata = msg.GetData()
+        if nbdata[0].GetPageCount() < nbdata[1]:
+            return # must have closed all pages in the notebook
+
         page = nbdata[0].GetPage(nbdata[1])
         path = page.GetFileName()
-        if path:
-            style = self.GetTreeStyle()
-            self.SetTreeStyle(wx.TR_SINGLE)
-            self.Refresh()
-            self.SetPath(path)
-            self.SetTreeStyle(style)
-            self.Refresh()
+        if len(path) and os.path.exists(path):
+            if wx.Platform == '__WXMAC__' and \
+               path.startswith('/Volumes/Macintosh HD'):
+                path = path.replace('/Volumes/Macintosh HD', '', 1)
+            elif wx.Platform == '__WXGTK__':
+                home = wx.GetHomeDir()
+                desk = home + _("Desktop")
+                if path.startswith(desk):
+                    path = path.replace(desk, '', 1)
+                elif path.startswith(home):
+                    path = path.replace(home, '', 1)
+
+            self.GotoPath(path)
 
     def OnThemeChanged(self, msg):
         """Update the icons when the icon theme has changed
@@ -644,7 +660,8 @@ class FileBrowser(wx.GenericDirCtrl):
         self._SetupIcons()
         self._tree.Refresh()
 
-    def OpenFiles(self, files):
+    @staticmethod
+    def OpenFiles(files):
         """Open the list of files in Editra for editing
         @param files: list of file names
 
@@ -663,12 +680,43 @@ class FileBrowser(wx.GenericDirCtrl):
             win.GetNotebook().OnDrop(to_open)
 
       # TODO implement drag and drop from the control to the editor
-#     def OnDragEnd(self, evt):
-#         evt.Skip()
+#    def OnDragStart(self, evt):
+#        """Start dragging a file from the tree"""
+#        item = self._tree.GetSelection()
+#        img = self._imglst.GetBitmap(self._tree.GetItemImage(item))
+#        lbl = self._tree.GetItemText(item)
+#        fdo = wx.FileDataObject()
+#        fdo.AddFile(self.GetPath())
+#        self._drag_img = FileDragImage(self._tree, lbl, img, fdo)
+#        mw = self.GetParent().GetMainWindow()
+#        self._drag_img.BeginDrag(wx.GetMousePosition(), self,
+#                                 rect=mw.GetClientRect())
+#        self._drag_img.Show()
+#        self._drag_img.Move(wx.GetMousePosition())
 
-#     def OnDragStart(self, evt):
-#         print evt.GetLabel()
-#         evt.Skip()
+#    def OnDragging(self, evt):
+#        """Move the drag image"""
+#        if evt.Dragging() and self._drag_img is not None:
+#            self._drag_img.Move(evt.GetPosition())#wx.GetMousePosition())
+#        elif self._drag_img is not None and evt.LeftUp():
+#            print "LEFTUP"
+#            self._drag_img.EndDrag()
+#            self._drag_img.Destroy()
+#            self._drag_img = None
+#        else:
+#            evt.Skip()
+
+#    def OnDragEnd(self, evt):
+#        """End the drag"""
+#        print "END DRAG"
+#        if self._drag_img is not None:
+#            self._drag_img.EndDrag()
+#            self._drag_img.Destroy()
+#            self._drag_img = None
+#        else:
+#            evt.Skip()
+
+#        evt.Skip()
 
 #     def SelectPath(self, path):
 #         """Selects the given path"""
@@ -687,12 +735,21 @@ class FileBrowser(wx.GenericDirCtrl):
 #                 print i_txt
 #             item, cookie = self._tree.GetNextChild(item, cookie)
 
+    def SetMainWindow(self, mainw):
+        """Set the main window this browser belongs to.
+        @param mainw: MainWindow or None
+
+        """
+        self._mw = mainw
+
     def SetTreeStyle(self, style):
         """Sets the style of directory controls tree"""
         self._tree.SetWindowStyle(style)
 
 #-----------------------------------------------------------------------------#
 
+# TODO maybe switch to storing this info in the user profile instead of
+#      managing it here.
 class PathMarkConfig(object):
     """Manages the saving of pathmarks to make them usable from
     one session to the next.
@@ -744,7 +801,7 @@ class PathMarkConfig(object):
 
     def RemovePathMark(self, pmark):
         """Removes a path mark from the config"""
-        if self._pmarks.has_key(pmark):
+        if pmark in self._pmarks:
             del self._pmarks[pmark]
 
     def Save(self):
@@ -770,7 +827,7 @@ def DuplicatePath(path):
     """
     head, tail = os.path.split(path)
     if os.path.isdir(path):
-        name = util.GetUniqueName(head, tail + "_Copy")
+        name = ebmlib.GetUniqueName(head, tail + "_Copy")
         copy = shutil.copytree
     else:
         tmp = [ part for part in tail.split('.') if len(part) ]
@@ -786,7 +843,7 @@ def DuplicatePath(path):
             tmp = '.'.join(tmp[:-1]) + "_Copy." + tmp[-1]
 
         if len(tmp) > 1:
-            name = util.GetUniqueName(head, tmp)
+            name = ebmlib.GetUniqueName(head, tmp)
         copy = shutil.copy2
 
     try:
@@ -808,7 +865,7 @@ def MakeArchive(path):
     ok = True
     name = ''
     if dname and fname:
-        name = util.GetUniqueName(dname, fname + ".zip")
+        name = ebmlib.GetUniqueName(dname, fname + ".zip")
         files = list()
         cwd = os.getcwd()
         head = dname
@@ -835,6 +892,7 @@ def MakeArchive(path):
     return (ok, name)
 
 #-----------------------------------------------------------------------------#
+
 class OpenerThread(threading.Thread):
     """Job runner thread for opening files with the systems filemanager"""
     def __init__(self, files):
@@ -845,3 +903,73 @@ class OpenerThread(threading.Thread):
         """Do the work of opeing the files"""
         for fname in self._files:
             subprocess.call([FILEMAN_CMD, fname])
+
+#-----------------------------------------------------------------------------#
+
+class FileDragImage(wx.DragImage):
+    def __init__(self, treeCtrl, lbl, img, fdo):
+        """Create the drag image
+        @param treeCtrl: Tree control drag started from
+        @param lbl: drag image label
+        @param img: file image
+
+        """
+        bmp = self.MakeBitmap(treeCtrl, lbl, img)
+        wx.DragImage.__init__(self, bmp, wx.StockCursor(wx.CURSOR_COPY_ARROW))
+
+        # Attributes
+        self._backgroundColour = None
+
+    def MakeBitmap(self, treeCtrl, lbl, img):
+        """Draw and create the drag image
+        @param treeCtrl: tree control drag started from
+        @param lbl: label to draw
+        @param img: image to draw
+
+        """
+        memory = wx.MemoryDC()
+
+        # Calculate Sizes
+        img_sz = img.GetSize()
+        txt_sz = treeCtrl.GetTextExtent(lbl)
+        bitmap = wx.EmptyBitmap(img_sz[0] + 7 + txt_sz[0],
+                                max(img_sz[1], txt_sz[1]) + 2)
+        memory.SelectObject(bitmap)
+
+        if wx.Platform == '__WXMAC__':
+            memory.SetBackground(wx.TRANSPARENT_BRUSH)
+        else:
+            backcolour = treeCtrl.GetBackgroundColour()
+            r, g, b = int(backcolour.Red()), int(backcolour.Green()), int(backcolour.Blue())
+            backcolour = ((r >> 1) + 20, (g >> 1) + 20, (b >> 1) + 20)
+            backcolour = wx.Colour(backcolour[0], backcolour[1], backcolour[2])
+            memory.SetBackground(wx.Brush(backcolour))
+
+        memory.SetBackgroundMode(wx.TRANSPARENT)
+        memory.SetFont(treeCtrl.GetFont())
+        memory.SetTextForeground(treeCtrl.GetForegroundColour())
+        memory.Clear()
+
+        memory.DrawBitmap(img, 2, 1, True)
+
+        textrect = wx.Rect(2 + img_sz[1] + 3, 2, txt_sz[0], txt_sz[1])
+        memory.DrawLabel(lbl, textrect)
+
+        memory.SelectObject(wx.NullBitmap)
+
+        # Gtk and Windows unfortunatly don't do so well with transparent
+        # drawing so this hack corrects the image to have a transparent
+        # background.
+        if wx.Platform != '__WXMAC__':
+            timg = bitmap.ConvertToImage()
+            if not timg.HasAlpha():
+                timg.InitAlpha()
+            for y in xrange(timg.GetHeight()):
+                for x in xrange(timg.GetWidth()):
+                    pix = wx.Colour(timg.GetRed(x, y),
+                                    timg.GetGreen(x, y),
+                                    timg.GetBlue(x, y))
+                    if pix == self._backgroundColour:
+                        timg.SetAlpha(x, y, 0)
+            bitmap = timg.ConvertToBitmap()
+        return bitmap
