@@ -28,8 +28,8 @@
 
 """
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: setup.py 54268 2008-06-16 22:06:53Z CJP $"
-__revision__ = "$Revision: 54268 $"
+__svnid__ = "$Id: setup.py 60240 2009-04-19 05:42:42Z CJP $"
+__revision__ = "$Revision: 60240 $"
 
 #---- Imports ----#
 import os
@@ -58,11 +58,20 @@ CLASSIFIERS = [
             'License :: OSI Approved',
             'Natural Language :: English',
             'Natural Language :: Chinese (Simplified)',
+            'Natural Language :: Chinese (Traditional)',
+            'Natural Language :: Czech',
             'Natural Language :: Dutch',
+            'Natural Language :: French',
             'Natural Language :: German',
+            'Natural Language :: Italian',
             'Natural Language :: Japanese',
+            'Natural Language :: Norwegian',
+            'Natural Language :: Portuguese (Brazilian)',
             'Natural Language :: Russian',
+            'Natural Language :: Serbian',
             'Natural Language :: Spanish',
+            'Natural Language :: Turkish',
+            'Natural Language :: Ukranian',
             'Operating System :: MacOS :: MacOS X',
             'Operating System :: Microsoft :: Windows',
             'Operating System :: POSIX',
@@ -71,22 +80,12 @@ CLASSIFIERS = [
             'Topic :: Text Editors'
             ]
 
-SRC_SCRIPTS = [ ("src", glob.glob("src/*.py")),
-                ("src/autocomp", glob.glob("src/autocomp/*.py")),
-                ("src/extern", ["src/extern/__init__.py", "src/extern/README"]),
-                ("src/syntax", glob.glob("src/syntax/*.py")),
-                ("src/eclib", glob.glob("src/eclib/*.py")),
-                ("scripts", ["scripts/clean_dir.sh"]),
-                ("scripts/i18n", glob.glob("scripts/i18n/*.po")),
-]
-
-DATA_FILES = [("include/python2.5",
+def GenerateBinPackageFiles():
+    """Generate the list of files needed for py2exe/py2app package files"""
+    data = [("include/python2.5",
                glob.glob("include/python2.5/%s/*" % __platform__)),
-              ("pixmaps", ["pixmaps/editra.png", "pixmaps/editra.ico",
-                           "pixmaps/editra.icns", "pixmaps/editra_doc.icns",
-                           "pixmaps/editra_doc.png"]),
               ("pixmaps/theme/Default", ["pixmaps/theme/Default/README"]),
-               ("pixmaps/theme/Tango",["pixmaps/theme/Tango/AUTHORS",
+              ("pixmaps/theme/Tango",["pixmaps/theme/Tango/AUTHORS",
                                       "pixmaps/theme/Tango/COPYING"]),
               ("pixmaps/theme/Tango/toolbar",
                glob.glob("pixmaps/theme/Tango/toolbar/*.png")),
@@ -94,69 +93,79 @@ DATA_FILES = [("include/python2.5",
                glob.glob("pixmaps/theme/Tango/menu/*.png")),
               ("pixmaps/theme/Tango/mime",
                glob.glob("pixmaps/theme/Tango/mime/*.png")),
-              ("plugins", glob.glob("plugins/*.egg")),
-              ("templates", glob.glob("templates/*")),
-              ("locale/de_DE/LC_MESSAGES",
-               ["locale/de_DE/LC_MESSAGES/Editra.mo"]),
-              ("locale/en_US/LC_MESSAGES",
-               ["locale/en_US/LC_MESSAGES/Editra.mo"]),
-              ("locale/es_ES/LC_MESSAGES",
-               ["locale/es_ES/LC_MESSAGES/Editra.mo"]),
-              ("locale/fr_FR/LC_MESSAGES",
-               ["locale/fr_FR/LC_MESSAGES/Editra.mo"]),
-              ("locale/it_IT/LC_MESSAGES",
-               ["locale/it_IT/LC_MESSAGES/Editra.mo"]),
-              ("locale/ja_JP/LC_MESSAGES",
-               ["locale/ja_JP/LC_MESSAGES/Editra.mo"]),
-              ("locale/nl_NL/LC_MESSAGES",
-               ["locale/nl_NL/LC_MESSAGES/Editra.mo"]),
-              ("locale/nn_NO/LC_MESSAGES",
-               ["locale/nn_NO/LC_MESSAGES/Editra.mo"]),
-              ("locale/pt_BR/LC_MESSAGES",
-               ["locale/pt_BR/LC_MESSAGES/Editra.mo"]),
-              ("locale/ru_RU/LC_MESSAGES",
-               ["locale/ru_RU/LC_MESSAGES/Editra.mo"]),
-              ("locale/sr_YU/LC_MESSAGES",
-               ["locale/sr_YU/LC_MESSAGES/Editra.mo"]),
-              ("locale/tr_TR/LC_MESSAGES",
-               ["locale/tr_TR/LC_MESSAGES/Editra.mo"]),
-              ("locale/uk_UA/LC_MESSAGES",
-               ["locale/uk_UA/LC_MESSAGES/Editra.mo"]),
-              ("locale/zh_CN/LC_MESSAGES",
-               ["locale/zh_CN/LC_MESSAGES/Editra.mo"]),
+              ("pixmaps/theme/Tango/other",
+               glob.glob("pixmaps/theme/Tango/other/*.png")),
               ("styles", glob.glob("styles/*.ess")),
+              ("ekeys", glob.glob("ekeys/*.ekeys")),
               ("tests/syntax", glob.glob("tests/syntax/*")),
-#               ("tests/controls", glob.glob("tests/controls/*")),
               ("docs", glob.glob("docs/*.txt")), "AUTHORS", "FAQ", "INSTALL",
               "README","CHANGELOG","COPYING", "NEWS", "THANKS", "TODO",
-              "setup.cfg", "pixmaps/editra_doc.icns"
-             ]
+              "setup.cfg"
+            ]
+
+    # Get the locale files
+    for loc_dir in os.listdir("locale"):
+        tmp = "locale/" + loc_dir + "/LC_MESSAGES"
+        if os.path.isdir(tmp):
+            tmp2 = tmp + "/Editra.mo"
+            if os.path.exists(tmp2):
+                data.append((tmp, [tmp2]))
+
+    # Only bundle the plugins for the running version of python being used for
+    # the build.
+    data.append(("plugins",
+                 glob.glob("plugins/*py%d.%d.egg" % sys.version_info[:2])))
+
+    # Get platform specific icons
+    pixlist = ["pixmaps/editra.png", "pixmaps/editra_doc.png"]
+
+    if "darwin" in sys.platform:
+        data.append("pixmaps/editra_doc.icns")
+        pixlist.extend(["pixmaps/editra.icns", "pixmaps/editra_doc.icns"])
+    elif sys.platform.startswith("win"):
+        pixlist.append("pixmaps/editra.ico")
+
+    data.append(("pixmaps", pixlist))
+
+    return data
+
+def GenerateSrcPackageFiles():
+    """Generate the list of files to include in a source package dist/install"""
+    data = [ "src/*.py", "src/syntax/*.py", "src/autocomp/*.py", 
+             "src/eclib/*.py", "docs/*.txt", "pixmaps/*.png", "pixmaps/*.ico",
+             "src/ebmlib/*.py",
+             "ekeys/*.ekeys",
+             'Editra',
+             "src/extern/*.py", "src/extern/pygments/*.py",
+             "src/extern/pygments/formatters/*.py",
+             "src/extern/pygments/filters/*.py",
+             "src/extern/pygments/lexers/*.py",
+             "src/extern/pygments/styles/*.py",
+             "pixmaps/*.icns",
+             "pixmaps/theme/Default/README",
+             "pixmaps/theme/Tango/AUTHOR",
+             "pixmaps/theme/Tango/COPYING",
+             "pixmaps/theme/Tango/toolbar/*.png",
+             "pixmaps/theme/Tango/menu/*.png",
+             "pixmaps/theme/Tango/mime/*.png",
+             "pixmaps/theme/Default/README",
+             "pixmaps/theme/Tango/other/*.png",
+             "styles/*.ess", "tests/syntax/*",
+             "AUTHORS", "CHANGELOG","COPYING", "FAQ", "INSTALL", "NEWS", 
+             "README", "THANKS", "TODO", "setup.cfg" ]
+
+    # Get the local files
+    for loc_dir in os.listdir("locale"):
+        tmp = "locale/" + loc_dir
+        if os.path.isdir(tmp):
+            tmp = tmp + "/LC_MESSAGES/Editra.mo"
+            if os.path.exists(tmp):
+                data.append(tmp)
+
+    # NOTE: plugins selected to package in build step
+    
+    return data
 
-DATA = [ "src/*.py", "src/syntax/*.py", "src/autocomp/*.py", "src/eclib/*.py",
-         "docs/*.txt", "pixmaps/*.png", "pixmaps/*.ico", 'Editra',
-         "src/extern/*.py", "pixmaps/*.icns", "pixmaps/theme/Default/README",
-         "pixmaps/theme/Tango/AUTHOR", "pixmaps/theme/Tango/COPYING",
-         "pixmaps/theme/Tango/toolbar/*.png", "pixmaps/theme/Tango/menu/*.png",
-         "pixmaps/theme/Tango/mime/*.png", "pixmaps/theme/Default/README",
-         "locale/de_DE/LC_MESSAGES/Editra.mo",
-         "locale/en_US/LC_MESSAGES/Editra.mo",
-         "locale/es_ES/LC_MESSAGES/Editra.mo",
-         "locale/fr_FR/LC_MESSAGES/Editra.mo",
-         "locale/it_IT/LC_MESSAGES/Editra.mo",
-         "locale/ja_JP/LC_MESSAGES/Editra.mo",
-         "locale/nl_NL/LC_MESSAGES/Editra.mo",
-         "locale/nn_NO/LC_MESSAGES/Editra.mo",
-         "locale/pt_BR/LC_MESSAGES/Editra.mo",
-         "locale/ru_RU/LC_MESSAGES/Editra.mo",
-         "locale/sr_YU/LC_MESSAGES/Editra.mo",
-         "locale/tr_TR/LC_MESSAGES/Editra.mo",
-         "locale/uk_UA/LC_MESSAGES/Editra.mo",
-         "locale/zh_CN/LC_MESSAGES/Editra.mo",
-#          "tests/controls/*",
-         "styles/*.ess", "tests/syntax/*",
-         "AUTHORS", "CHANGELOG","COPYING", "FAQ", "INSTALL", "NEWS", "README",
-         "THANKS", "TODO", "setup.cfg", "plugins/*.egg"
-]
 
 DESCRIPTION = "Developer's Text Editor"
 
@@ -190,7 +199,7 @@ ICON = { 'Win' : "pixmaps/editra.ico",
 # or otherwise not able to be found by py2app/exe
 INCLUDES = ['syntax.*', 'ed_log', 'shutil', 'subprocess', 'zipfile',
             'pygments.*', 'pygments.lexers.*', 'pygments.formatters.*',
-            'pygments.filters.*', 'pygments.styles.*']
+            'pygments.filters.*', 'pygments.styles.*', 'ftplib']
 if sys.platform.startswith('win'):
     INCLUDES.extend(['ctypes'])
 else:
@@ -213,7 +222,7 @@ MANIFEST_TEMPLATE = '''
     name="%(prog)s"
     type="win32"
 />
-<description>%(prog)s Program</description>
+<description>%(prog)s</description>
 <dependency>
     <dependentAssembly>
         <assemblyIdentity
@@ -257,14 +266,14 @@ if __platform__ == "win32" and 'py2exe' in sys.argv:
                     "other_resources" : [(RT_MANIFEST, 1,
                                           MANIFEST_TEMPLATE % dict(prog=NAME))],
                   }],
-        description = DESCRIPTION,
+        description = NAME,
         author = AUTHOR,
         author_email = AUTHOR_EMAIL,
         maintainer = AUTHOR,
         maintainer_email = AUTHOR_EMAIL,
         license = LICENSE,
         url = URL,
-        data_files = DATA_FILES,
+        data_files = GenerateBinPackageFiles(),
         )
 
 #---- Setup MacOSX APP ----#
@@ -285,6 +294,14 @@ elif __platform__ == "darwin" and 'py2app' in sys.argv:
                                            CFBundleTypeRole="Editor"
                                           ),
                                      ],
+             CFBundleTypeMIMETypes = ['text/plain',],
+             CFBundleDevelopmentRegion = 'English',
+# TODO Causes errors with the system menu translations and text rendering
+#             CFBundleLocalizations = ['English', 'Spanish', 'French', 'Japanese'],
+#             ['de_DE', 'en_US', 'es_ES', 'fr_FR',
+#                                      'it_IT', 'ja_JP', 'nl_NL', 'nn_NO',
+#                                      'pt_BR', 'ru_RU', 'sr_SR', 'tr_TR',
+#                                      'uk_UA', 'zh_CN'],
        #      NSAppleScriptEnabled="YES",
              NSHumanReadableCopyright = u"Copyright %s 2005-%d" % (AUTHOR, YEAR)
              )
@@ -295,6 +312,9 @@ elif __platform__ == "darwin" and 'py2app' in sys.argv:
                        includes = INCLUDES,
                        plist = PLIST)
 
+    # Add extra mac specific files
+    DATA_FILES = GenerateBinPackageFiles()
+    DATA_FILES.append("scripts/editramac.sh")
 
     # Put extern package on path for py2app
     sys.path.append(os.path.abspath('src/extern'))
@@ -316,13 +336,24 @@ elif __platform__ == "darwin" and 'py2app' in sys.argv:
 
 #---- Other Platform(s)/Source module install ----#
 else:
+
+    # Get the package data
+    DATA = GenerateSrcPackageFiles()
+
     # Force optimization
     if 'install' in sys.argv and ('O1' not in sys.argv or '02' not in sys.argv):
         sys.argv.append('-O2')
 
+        # Install the plugins for this version of Python
+        DATA.append("plugins/*py%d.%d.egg" % sys.version_info[:2])
+
+    # Import proper setup function
     if 'bdist_egg' in sys.argv:
         try:
             from setuptools import setup
+
+            # Only bundle eggs for the given python version
+            DATA.append("plugins/*py%d.%d.egg" % sys.version_info[:2])
         except ImportError:
             print "To build an egg setuptools must be installed"
     else:
