@@ -16,18 +16,28 @@ running Editra.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: Editra.py 53981 2008-06-05 02:59:30Z CJP $"
-__revision__ = "$Revision: 53981 $"
+__svnid__ = "$Id: Editra.py 60549 2009-05-08 01:49:32Z CJP $"
+__revision__ = "$Revision: 60549 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
 import os
 import sys
+import base64
 import locale
 import time
 import getopt
+import shutil
 import wx
 
+# The event handler mixin is now part of wxPython proper, but there hasn't
+# been an official release with it yet, so try to import the official module
+# but fallback to our own copy if it fails.
+try:
+    import wx.lib.eventStack as events
+except:
+    import extern.events as events
+
 # Due to some methods that were added in 2.8.3 being used in a large number
 # of places Editra has become incompatable with wxPython 2.8.1.1 and earlier.
 # TODO: add a message dialog to display this, or look into the cause of the
@@ -55,12 +65,16 @@ import ed_txt
 import ed_event
 import updater
 import plugin
-import extern.events as events
+import ed_ipc
+import ed_msg
 
 #--------------------------------------------------------------------------#
 # Global Variables
 ID_UPDATE_CHECK = wx.NewId()
 
+# Commands (here temporarly)
+APP_CMD_OPEN_WINDOW = u"Editra.OpenWindow"
+
 _ = wx.GetTranslation
 #--------------------------------------------------------------------------#
 
@@ -82,22 +96,57 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         self._lock = False
         self._windows = dict()
 
-        # Setup Locale
-        locale.setlocale(locale.LC_ALL, '')
-        self.locale = wx.Locale(ed_i18n.GetLangId(profiler.Profile_Get('LANG')))
-        if self.locale.GetCanonicalName() in ed_i18n.GetAvailLocales():
-            self.locale.AddCatalogLookupPathPrefix(ed_glob.CONFIG['LANG_DIR'])
-            self.locale.AddCatalog(ed_glob.PROG_NAME)
+        # Disable debug popups
+        wx.Log.EnableLogging(False)
+
+        if ed_glob.SINGLE:
+            # Setup the instance checker
+            instance_name = u"%s-%s" % (self.GetAppName(), wx.GetUserId())
+            self._instance = wx.SingleInstanceChecker(instance_name)
+            if self._instance.IsAnotherRunning():
+                try:
+                    opts, args = getopt.getopt(sys.argv[1:], "dhv",
+                                               ['debug', 'help', 'version'])
+                except getopt.GetoptError, msg:
+                    self._log("[app][err] %s" % str(msg))
+                    args = list()
+
+                if not len(args):
+                    args.append(APP_CMD_OPEN_WINDOW)
+
+                rval = ed_ipc.SendCommands(args, profiler.Profile_Get('SESSION_KEY'))
+                # If sending the command failed then let the editor startup
+                # a new instance
+                if not rval:
+                    self._isfirst = True
+            else:
+                self._log("[app][info] Starting Ipc server...")
+                # Set the session key and save it to the users profile so
+                # that other instances can access the server
+                key = unicode(base64.b64encode(os.urandom(8), 'zZ'))
+                key = wx.GetUserName() + key
+                profiler.Profile_Set('SESSION_KEY', key)
+                profiler.Profile_Set('ISBINARY', hasattr(sys, 'frozen'))
+                path = profiler.Profile_Get('MYPROFILE')
+                profiler.TheProfile.Write(path)
+                try:
+                    self._server = ed_ipc.EdIpcServer(self, profiler.Profile_Get('SESSION_KEY'))
+                    self._server.start()
+                except Exception, msg:
+                    self._log("[app][err] Failed to start ipc server")
+                    self._log("[app][err] %s" % str(msg))
+                    self._server = None
+                self._isfirst = True
         else:
-            del self.locale
-            self.locale = None
+            self._isfirst = True
 
         # Setup Plugins after locale as they may have resource that need to
         # be loaded.
-        self._pluginmgr = plugin.PluginManager()
+        if self._isfirst:
+            self._pluginmgr = plugin.PluginManager()
 
-        self._log("[app][info] Registering Editra's ArtProvider")
-        wx.ArtProvider.PushProvider(ed_art.EditraArt())
+            self._log("[app][info] Registering Editra's ArtProvider")
+            wx.ArtProvider.PushProvider(ed_art.EditraArt())
 
     def AddMessageCatalog(self, name, path):
         """Add a catalog lookup path to the app
@@ -117,9 +166,30 @@ class Editra(wx.App, events.AppEventHandlerMixin):
 
         """
         self.SetAppName(ed_glob.PROG_NAME)
+
         self._log = dev_tool.DEBUGP
         self._log("[app][info] Editra is Initializing")
 
+        # Load user preferences
+        self.profile_updated = InitConfig()
+        self._isfirst = False # Is the first instance
+        self._instance = None
+
+        # Setup Locale
+        locale.setlocale(locale.LC_ALL, '')
+        self.locale = wx.Locale(ed_i18n.GetLangId(profiler.Profile_Get('LANG')))
+        if self.locale.GetCanonicalName() in ed_i18n.GetAvailLocales():
+            self.locale.AddCatalogLookupPathPrefix(ed_glob.CONFIG['LANG_DIR'])
+            self.locale.AddCatalog(ed_glob.PROG_NAME)
+        else:
+            del self.locale
+            self.locale = None
+
+        # Check and set encoding if necessary
+        if not profiler.Profile_Get('ENCODING'):
+            profiler.Profile_Set('ENCODING', locale.getpreferredencoding())
+
+        # Setup the Error Reporter
         if profiler.Profile_Get('REPORTER', 'bool', True):
             sys.excepthook = dev_tool.ExceptionHook
 
@@ -128,9 +198,35 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         self.Bind(wx.EVT_MENU, self.OnNewWindow, id=ed_glob.ID_NEW_WINDOW)
         self.Bind(wx.EVT_MENU, self.OnCloseWindow)
         self.Bind(ed_event.EVT_NOTIFY, self.OnNotify)
+        self.Bind(ed_ipc.EVT_COMMAND_RECV, self.OnCommandRecieved)
+
+        # Splash a warning if version is not a final version
+        if profiler.Profile_Get('APPSPLASH'):
+            import edimage
+            splash_img = edimage.splashwarn.GetBitmap()
+            self.splash = wx.SplashScreen(splash_img, wx.SPLASH_CENTRE_ON_PARENT | \
+                                          wx.SPLASH_NO_TIMEOUT, 0, None, wx.ID_ANY)
+            self.splash.Show()
 
         return True
 
+    def Destroy(self):
+        """Destroy the application"""
+        try:
+            # Cleanup the instance checker
+            del self._instance
+        except AttributeError:
+            pass
+        wx.App.Destroy(self)
+
+    def DestroySplash(self):
+        """Destroy the splash screen"""
+        # If is created and not dead already
+        if getattr(self, 'splash', None) is not None and \
+           isinstance(self.splash, wx.SplashScreen):
+            self.splash.Destroy()
+            self.splash = None
+
     def Exit(self, force=False):
         """Exit the program
         @postcondition: If no toplevel windows are present program will exit.
@@ -138,9 +234,19 @@ class Editra(wx.App, events.AppEventHandlerMixin):
 
         """
         self._pluginmgr.WritePluginConfig()
-        profiler.Profile().Write(profiler.Profile_Get('MYPROFILE'))
+        profiler.TheProfile.Write(profiler.Profile_Get('MYPROFILE'))
         if not self._lock or force:
-            wx.App.Exit(self)
+            if hasattr(self, 'server'):
+                self.server.ShutDown()
+
+            try:
+                # Cleanup the instance checker
+                del self._instance
+            except AttributeError:
+                pass
+
+            # Exit the app
+            wx.App.ExitMainLoop(self)
 
     def GetLocaleObject(self):
         """Get the locale object owned by this app. Use this method to add
@@ -185,11 +291,26 @@ class Editra(wx.App, events.AppEventHandlerMixin):
                 break
 
         if awin is None:
-            if len(self.GetMainWindows()):
-                awin = self.GetMainWindows()[0]
+            awin = self.GetTopWindow()
+            if awin is None or getattr(awin, '__name__', '?') != "MainWindow":
+                if len(self.GetMainWindows()):
+                    awin = self.GetMainWindows()[0]
 
         return awin
 
+    def GetCurrentBuffer(self):
+        """Get the current buffer from the active window or None
+        @return: EditraStc
+
+        """
+        win = self.GetTopWindow()
+        if getattr(win, '__name__', None) != u"MainWindow":
+            win = self.GetActiveWindow()
+            if win is None:
+                return win
+
+        return win.GetNotebook().GetCurrentCtrl()
+
     def GetMainWindows(self):
         """Returns a list of all open main windows
         @return: list of L{MainWindow} instances of this app (list may be empty)
@@ -219,6 +340,13 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         """
         return self._pluginmgr
 
+    def GetProfileUpdated(self):
+        """Was the profile updated 
+        @return: bool
+
+        """
+        return self.profile_updated
+
     def GetWindowInstance(self, wintype):
         """Get an instance of an open window if one exists
         @param wintype: Class type of window to look for
@@ -238,6 +366,13 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         """
         return self._lock
 
+    def IsOnlyInstance(self):
+        """Check if this app is the the first instance that is running
+        @return: bool
+
+        """
+        return self._isfirst
+
     def Lock(self):
         """Locks the app from exiting
         @postcondition: program is locked from exiting
@@ -245,6 +380,10 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         """
         self._lock = True
 
+    def MacNewFile(self):
+        """Stub for future use"""
+        pass
+
     def MacOpenFile(self, filename):
         """Macintosh Specific code for opening files that are associated
         with the editor and double clicked on after the editor is already
@@ -256,16 +395,37 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         window = self.GetTopWindow()
         if getattr(window, '__name__', '') == "MainWindow":
             try:
+                self._log("[app][info] MacOpenFile Fired")
                 encoding = sys.getfilesystemencoding()
                 window.DoOpen(ed_glob.ID_COMMAND_LINE_OPEN,
                               ed_txt.DecodeString(filename, encoding))
-                self._log("[app][info] MacOpenFile Fired")
+
+                # Make sure the window is brought to the front
+                if window.IsIconized():
+                    window.Iconize(False)
+                window.Raise()
+
             except Exception, msg:
                 self._log("[app][err] Failed to open drop file: %s" % str(msg))
                 pass
         else:
             pass
 
+    def MacPrintFile(self, filename):
+        """Stub for future use
+        @param filename: file to print
+
+        """
+        pass
+
+    def MacReopenApp(self):
+        """Handle kAEReopenApplication when dock icons is clicked on"""
+        frame = self.GetTopWindow()
+        if frame is not None:
+            if frame.IsIconized():
+                frame.Iconize(False)
+            frame.Raise()
+
     def OnActivate(self, evt):
         """Activation Event Handler
         @param evt: event that called this handler
@@ -274,6 +434,11 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         """
         if evt.GetActive():
             self._log("[app][info] I'm Awake!!")
+#            frame = self.GetTopWindow()
+#            if frame is not None:
+#                if frame.IsIconized():
+#                    frame.Iconize(False)
+#                frame.Raise()
         else:
             self._log("[app][info] Going to sleep")
         evt.Skip()
@@ -317,16 +482,40 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         else:
             evt.Skip()
 
+    def OnCommandRecieved(self, evt):
+        """Recieve commands from the IPC server
+        @todo: move command processing into own module
+
+        """
+        cmds = evt.GetCommands()
+        for cmdstr in cmds:
+            if u"::" in cmdstr:
+                target, cmd = cmdstr.split(u"::")
+                if target == u"Cmd.EditraStc":
+                    cbuf = self.GetCurrentBuffer()
+                    if cbuf is not None and hasattr(cbuf, cmd):
+                        try:
+                            getattr(cbuf, cmd)()
+                        except:
+                            self._log("[app][err] Invalid Command %s" % cmdstr)
+            else:
+                if cmdstr == APP_CMD_OPEN_WINDOW:
+                    self.OpenNewWindow()
+                elif len(cmdstr):
+                    self.MacOpenFile(cmdstr)
+                else:
+                    self._log("[app][warn] Unknown Command %s" % cmdstr)
+
     def OnCloseWindow(self, evt):
         """Close the currently active window
         @param evt: wx.MenuEvent
 
         """
         if evt.GetId() in [ed_glob.ID_CLOSE, ed_glob.ID_CLOSE_WINDOW]:
-            for window in self._windows.values():
-                if hasattr(window[0], 'IsActive') and window[0].IsActive():
-                    if hasattr(window[0], 'Close'):
-                        window[0].Close()
+            for window in wx.GetTopLevelWindows():
+                if hasattr(window, 'IsActive') and window.IsActive():
+                    if hasattr(window, 'Close'):
+                        window.Close()
                     break
         else:
             evt.Skip()
@@ -334,6 +523,7 @@ class Editra(wx.App, events.AppEventHandlerMixin):
     def OpenNewWindow(self, fname=u'', caller=None):
         """Open a new window
         @keyword fname: Open a file in the new window
+        @return: the new window
 
         """
         frame = ed_main.MainWindow(None, wx.ID_ANY,
@@ -353,6 +543,7 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         # this doesn't happen automatically on windows
         if wx.Platform == '__WXMSW__':
             wx.PostEvent(frame, wx.ActivateEvent(wx.wxEVT_ACTIVATE, True))
+        return frame
 
     def OnNotify(self, evt):
         """Handle notification events
@@ -362,6 +553,7 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         e_val = evt.GetValue()
         if evt.GetId() == ID_UPDATE_CHECK and \
            isinstance(e_val, tuple) and e_val[0]:
+            self.DestroySplash()
             mdlg = wx.MessageDialog(self.GetActiveWindow(),
                                     _("An updated version of Editra is available\n"
                                       "Would you like to download Editra %s now?") %\
@@ -415,7 +607,7 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         @param name: name of window to unregister
 
         """
-        if self._windows.has_key(name):
+        if name in self._windows:
             self._windows.pop(name)
             cur_top = self.GetTopWindow()
             if not len(self._windows):
@@ -452,7 +644,7 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         @param winname: name of window to query
 
         """
-        if self._windows.has_key(winname):
+        if winname in self._windows:
             return self._windows[winname][1]
         else:
             self._log("[app][warn] the window %s has "
@@ -466,18 +658,42 @@ def InitConfig():
     @postcondition: all configuration data is set
 
     """
-    ed_glob.CONFIG['PROFILE_DIR'] = util.ResolvConfigDir("profiles")
+    # Check if a custom config directory was specified on the commandline
+    if ed_glob.CONFIG['CONFIG_BASE'] is not None:
+        # TODO: there is a bug when the first time the config is created
+        #       where the settings will not be saved until second launching.
+        config_base = os.path.abspath(ed_glob.CONFIG['CONFIG_BASE'])
+    else:
+        # Look for a profile directory on the system level. If this directory
+        # exists Use it instead of the user one. This will allow for running
+        # Editra from a portable drive or for system administrators to enforce
+        # settings on a system installed version.
+        config_base = util.ResolvConfigDir(u'.Editra', True) 
+
+    if os.path.exists(config_base):
+        ed_glob.CONFIG['CONFIG_BASE'] = config_base
+        ed_glob.CONFIG['PROFILE_DIR'] = os.path.join(config_base, u"profiles")
+        ed_glob.CONFIG['PROFILE_DIR'] += os.sep
+    else:
+        config_base = wx.StandardPaths.Get().GetUserDataDir()
+        ed_glob.CONFIG['PROFILE_DIR'] = util.ResolvConfigDir(u"profiles")
+
+    # Check for if config directory exists and if profile is from the current
+    # running version of Editra.
     profile_updated = False
-    if util.HasConfigDir():
+    if util.HasConfigDir() and os.path.exists(ed_glob.CONFIG['PROFILE_DIR']):
         if profiler.ProfileIsCurrent():
-            profiler.Profile().Load(profiler.GetProfileStr())
+            pstr = profiler.GetProfileStr()
+            pstr = util.RepairConfigState(pstr)
+            profiler.TheProfile.Load(pstr)
         else:
             dev_tool.DEBUGP("[InitConfig][info] Updating Profile to current version")
 
             # Load and update profile
             pstr = profiler.GetProfileStr()
-            profiler.Profile().Load(pstr)
-            profiler.Profile().Update()
+            pstr = util.RepairConfigState(pstr)
+            profiler.TheProfile.Load(pstr)
+            profiler.TheProfile.Update()
 
             #---- Temporary Profile Adaptions ----#
 
@@ -486,14 +702,31 @@ def InitConfig():
             if mode == 'GUI_DEBUG':
                 profiler.Profile_Set('MODE', 'DEBUG')
 
+            # This key has been removed so clean it from old profiles
+            profiler.Profile_Del('LASTCHECK')
+
+            # Print modes don't use strings anymore
+            if isinstance(profiler.Profile_Get('PRINT_MODE'), basestring):
+                profiler.Profile_Set('PRINT_MODE', ed_glob.PRINT_BLACK_WHITE)
+
+            # Simplifications to eol mode persistance (0.4.28)
+            # Keep for now till plugins are updated
+            #profiler.Profile_Del('EOL') # changed to EOL_MODE
+
+            # After 0.4.65 LAST_SESSION now points a session file and not
+            # to a list of files to open.
+            sess = profiler.Profile_Get('LAST_SESSION')
+            if isinstance(sess, list):
+                profiler.Profile_Set('LAST_SESSION', u'')
+
             #---- End Temporary Profile Adaptions ----#
 
             # Write out updated profile
-            profiler.Profile().Write(pstr)
+            profiler.TheProfile.Write(pstr)
 
             # When upgrading from an older version make sure all
             # config directories are available.
-            for cfg in ["cache", "styles", "plugins", "profiles"]:
+            for cfg in ("cache", "styles", "plugins", "profiles", "sessions"):
                 if not util.HasConfigDir(cfg):
                     util.MakeConfigDir(cfg)
 
@@ -502,77 +735,220 @@ def InitConfig():
         # Fresh install
         util.CreateConfigDir()
 
+        # Check and upgrade installs from old location
+        success = True
+        try:
+            success = UpgradeOldInstall()
+        except Exception, msg:
+            dev_tool.DEBUGP("[InitConfig][err] %s" % msg)
+            success = False
+
+        if not success:
+            old_cdir = u"%s%s.%s%s" % (wx.GetHomeDir(), os.sep,
+                                       ed_glob.PROG_NAME, os.sep)
+            msg = ("Failed to upgrade your old installation\n"
+                   "To retain your old settings you may need to copy some files:\n"
+                   "\nFrom: %s\n\nTo: %s") % (old_cdir, config_base)
+            wx.MessageBox(msg, "Upgrade Failed", style=wx.ICON_WARNING|wx.OK)
+
         # Set default eol for windows
         if wx.Platform == '__WXMSW__':
-            profiler.Profile_Set('EOL', 'Windows (\\r\\n)')
+            profiler.Profile_Set('EOL_MODE', ed_glob.EOL_MODE_CRLF)
+            profiler.Profile_Set('ICONSZ', (16, 16))
+
+    #---- Profile Loaded / Installed ----#
 
     # Set debug mode
-    if 'DEBUG' in profiler.Profile_Get('MODE'):
+    emode = profiler.Profile_Get('MODE')
+    if 'DEBUG' in emode:
         ed_glob.DEBUG = True
+        if emode.startswith('VERBOSE'):
+            ed_glob.VDEBUG = True
 
     # Resolve resource locations
-    ed_glob.CONFIG['CONFIG_DIR'] = util.ResolvConfigDir("")
-    ed_glob.CONFIG['SYSPIX_DIR'] = util.ResolvConfigDir("pixmaps", True)
-    ed_glob.CONFIG['PLUGIN_DIR'] = util.ResolvConfigDir("plugins")
-    ed_glob.CONFIG['THEME_DIR'] = util.ResolvConfigDir(os.path.join("pixmaps",
-                                                                    "theme"))
-    ed_glob.CONFIG['LANG_DIR'] = util.ResolvConfigDir("locale", True)
-    ed_glob.CONFIG['STYLES_DIR'] = util.ResolvConfigDir("styles")
-    ed_glob.CONFIG['SYS_PLUGIN_DIR'] = util.ResolvConfigDir("plugins", True)
-    ed_glob.CONFIG['SYS_STYLES_DIR'] = util.ResolvConfigDir("styles", True)
-    ed_glob.CONFIG['TEST_DIR'] = util.ResolvConfigDir(os.path.join("tests",
-                                                                   "syntax"),
-                                                                   True)
-    if not util.HasConfigDir("cache"):
-        util.MakeConfigDir("cache")
-    ed_glob.CONFIG['CACHE_DIR'] = util.ResolvConfigDir("cache")
+    ed_glob.CONFIG['CONFIG_DIR'] = util.ResolvConfigDir(u"")
+    ed_glob.CONFIG['INSTALL_DIR'] = util.ResolvConfigDir(u"", True)
+    ed_glob.CONFIG['KEYPROF_DIR'] = util.ResolvConfigDir(u"ekeys", True)
+    ed_glob.CONFIG['SYSPIX_DIR'] = util.ResolvConfigDir(u"pixmaps", True)
+    ed_glob.CONFIG['PLUGIN_DIR'] = util.ResolvConfigDir(u"plugins")
+    ed_glob.CONFIG['THEME_DIR'] = util.ResolvConfigDir(os.path.join(u"pixmaps", u"theme"))
+    ed_glob.CONFIG['LANG_DIR'] = util.ResolvConfigDir(u"locale", True)
+    ed_glob.CONFIG['STYLES_DIR'] = util.ResolvConfigDir(u"styles")
+    ed_glob.CONFIG['SYS_PLUGIN_DIR'] = util.ResolvConfigDir(u"plugins", True)
+    ed_glob.CONFIG['SYS_STYLES_DIR'] = util.ResolvConfigDir(u"styles", True)
+    ed_glob.CONFIG['TEST_DIR'] = util.ResolvConfigDir(os.path.join(u"tests", u"syntax"), True)
+
+    # Make sure all standard config directories are there
+    for cfg in ("cache", "styles", "plugins", "profiles", "sessions"):
+        if not util.HasConfigDir(cfg):
+            util.MakeConfigDir(cfg)
+    ed_glob.CONFIG['CACHE_DIR'] = util.ResolvConfigDir(u"cache")
+    ed_glob.CONFIG['SESSION_DIR'] = util.ResolvConfigDir(u"sessions")
 
     return profile_updated
 
 #--------------------------------------------------------------------------#
 
+def UpgradeOldInstall():
+    """Upgrade an old installation and transfer all files if they exist
+    @note: FOR INTERNAL USE ONLY
+    @return: bool (True if success, False if failure)
+
+    """
+    old_cdir = u"%s%s.%s%s" % (wx.GetHomeDir(), os.sep,
+                               ed_glob.PROG_NAME, os.sep)
+    base = ed_glob.CONFIG['CONFIG_BASE']
+    if base is None:
+        base = wx.StandardPaths.Get().GetUserDataDir() + os.sep
+
+    err = 0
+    if os.path.exists(old_cdir) and \
+       base.lower().rstrip(os.sep) != old_cdir.lower().rstrip(os.sep):
+        for item in os.listdir(old_cdir):
+            try:
+                dest = os.path.join(base, item)
+                item = os.path.join(old_cdir, item)
+                if os.path.exists(dest):
+                    if os.path.isdir(dest):
+                        shutil.rmtree(dest, True)
+                    else:
+                        os.remove(dest)
+
+                shutil.move(item, dest)
+            except Exception, msg:
+                util.Log("[Upgrade][err] %s" % msg)
+                err += 1
+                continue
+
+        os.rmdir(old_cdir)
+
+        # Load the copied over profile
+        pstr = profiler.GetProfileStr()
+        prof = os.path.basename(pstr)
+        pstr = os.path.join(base, u"profiles", prof)
+        if os.path.exists(pstr):
+            profiler.TheProfile.Load(pstr)
+            profiler.TheProfile.Update()
+            profiler.UpdateProfileLoader()
+
+        if not err:
+            wx.MessageBox(_("Your profile has been updated to the latest "
+                "version") + u"\n" + \
+              _("Please check the preferences dialog to check "
+                "your preferences"),
+              _("Profile Updated"))
+
+    return not err
+
+#--------------------------------------------------------------------------#
+
+def PrintHelp():
+    """Print command line help
+    @postcondition: Help is printed and program exits
+
+    """
+    print ("Editra - %s - Developers Text Editor\n"
+       "Cody Precord (2005-2009)\n\n"
+       "usage: Editra [arguments] [files... ]\n\n"
+       "Short Arguments:\n"
+       "  -c    Set custom configuration directory at runtime\n"
+       "  -d    Turn on console debugging (-dd for verbose debug)\n"
+       "  -D    Turn off console debugging (overrides preferences)\n"
+       "  -h    Show this help message\n"
+       "  -p    Run Editra in the profiler (outputs to editra.prof).\n"
+       "  -v    Print version number and exit\n"
+       "  -S    Disable single instance checker\n"
+       "\nLong Arguments:\n"
+       "  --confdir arg     Set custom configuration directory at runtime\n"
+       "  --debug           Turn on console debugging\n"
+       "  --help            Show this help message\n"
+       "  --auth            Print the ipc server info\n"
+       "  --version         Print version number and exit\n"
+       "  --profileOut arg  Run Editra in the profier (arg is output file)\n"
+      ) % ed_glob.VERSION
+    os._exit(0)
+
+#--------------------------------------------------------------------------#
+
+def ProcessCommandLine():
+    """Process the command line switches
+    @return: tuple ({switches,}, [args,])
+
+    """
+    try:
+        items, args = getopt.getopt(sys.argv[1:], "dhpvDSc",
+                                   ['debug', 'help', 'version', 'auth',
+                                    'configdir=', 'profileOut='])
+    except getopt.GetoptError, msg:
+        # Raise error to console and exit
+        sys.stderr.write(str(msg) + os.linesep)
+        PrintHelp()
+    
+    # Process command line options
+    opts = dict(items)
+    for opt, value in dict(opts).items():
+        if opt in ['-h', '--help']:
+            PrintHelp()
+        elif opt in ['-v', '--version']:
+            print ed_glob.VERSION
+            os._exit(0)
+        elif opt in ['-d', '--debug'] and '-D' not in opts.keys():
+            # If the debug flag is set more than once go into verbose mode
+            if ed_glob.DEBUG:
+                ed_glob.VDEBUG = True
+            ed_glob.DEBUG = True
+            opts.pop(opt)
+        elif opt == '-D':
+            ed_glob.DEBUG = False
+            ed_glob.VDEBUG = False
+            opts.pop('-D')
+        elif opt == '-S':
+            # Disable single instance checker
+            ed_glob.SINGLE = False
+            opts.pop(opt)
+        elif opt in ['-c', '--configdir']:
+            ed_glob.CONFIG['CONFIG_BASE'] = value
+            opts.pop(opt)
+        elif opt == '--profileOut':
+            opts['-p'] = value
+            opts.pop('--profileOut')
+        else:
+            pass
+
+    # Return any unprocessed arguments
+    return opts, args
+
+#--------------------------------------------------------------------------#
+
 def Main():
     """Configures and Runs an instance of Editra
     @summary: Parses command line options, loads the user profile, creates
               an instance of Editra and starts the main loop.
 
     """
-    try:
-        opts, args = getopt.getopt(sys.argv[1:], "dhv",
-                                   ['debug', 'help', 'version'])
-    except getopt.GetoptError, msg:
-        dev_tool.DEBUGP("[main][err] %s" % str(msg))
-        opts = list()
-        args = list()
+    opts, args = ProcessCommandLine()
 
-    # Process command line options
-    if len(opts):
-        if True in [x[0] in ['-h', '--help'] for x in opts]:
-            print ("Editra - %s - Developers Text Editor\n"
-                   "Cody Precord (2005-2007)\n\n"
-                   "usage: Editra [arguments] [files... ]\n\n"
-                   "Short Arguments:\n"
-                   "  -d         Turn on console debugging\n"
-                   "  -h         Show this help message\n"
-                   "  -v         Print version number and exit\n"
-                   "\nLong Arguments:\n"
-                   "  --debug    Turn on console debugging\n"
-                   "  --help     Show this help message\n"
-                   "  --version  Print version number and exit\n"
-                  ) % ed_glob.VERSION
-            os._exit(0)
+    if '-p' in opts:
+        p_file = opts['-p']
+        opts.pop('-p')
 
-        if True in [x[0] in ['-v', '--version'] for x in opts]:
-            print "%s - v%s - Developers Editor" % (ed_glob.PROG_NAME, \
-                                                    ed_glob.VERSION)
-            os._exit(0)
+        if not len(p_file):
+            # Fall back to default output file
+            p_file = "editra.prof"
+            
+        import hotshot
+        prof = hotshot.Profile(p_file)
+        prof.runcall(_Main, opts, args)
+        prof.close()
+    else:
+        _Main(opts, args)
 
-        if True in [x[0] in ['-d', '--debug'] for x in opts]:
-            ed_glob.DEBUG = True
-
-    # We are ready to run so fire up the config and launch the app
-    profile_updated = InitConfig()
+def _Main(opts, args):
+    """Main method
+    @param opts: Commandline options
+    @param args: Commandline arguments
 
+    """
     # Put extern subpackage on path so that bundled external dependancies
     # can be found if needed.
     if not hasattr(sys, 'frozen'):
@@ -581,25 +957,25 @@ def Main():
             sys.path.append(epath)
 
     # Create Application
-    dev_tool.DEBUGP("[main][info] Initializing Application...")
+    dev_tool.DEBUGP("[main][app] Initializing application...")
     editra_app = Editra(False)
 
-    if profile_updated:
-        # Make sure window iniliazes to default position
-        profiler.Profile_Del('WPOS')
-        wx.MessageBox(_("Your profile has been updated to the latest "
-                        "version") + u"\n" + \
-                      _("Please check the preferences dialog to reset "
-                        "your preferences"),
-                      _("Profile Updated"))
+    # Print ipc server authentication info
+    if '--auth' in opts:
+        opts.pop('--auth')
+        print "port=%d,key=%s" % (ed_ipc.EDPORT,
+                                  profiler.Profile_Get('SESSION_KEY'))
 
-    # Splash a warning if version is not a final version
-    if profiler.Profile_Get('APPSPLASH') and int(ed_glob.VERSION[0]) < 1:
-        import edimage
-        splash_img = edimage.splashwarn.GetBitmap()
-        splash = wx.SplashScreen(splash_img, wx.SPLASH_CENTRE_ON_PARENT | \
-                                 wx.SPLASH_NO_TIMEOUT, 0, None, wx.ID_ANY)
-        splash.Show()
+    # Check if this is the only instance, if its not exit since
+    # any of the opening commands have already been passed to the
+    # master instance
+    if not editra_app.IsOnlyInstance():
+        dev_tool.DEBUGP("[main][info] Second instance exiting...")
+        editra_app.Destroy()
+        os._exit(0)
+
+    # Set the timeout on destroying the splash screen
+    wx.CallLater(2300, editra_app.DestroySplash)
 
     if profiler.Profile_Get('SET_WSIZE'):
         wsize = profiler.Profile_Get('WSIZE')
@@ -614,7 +990,12 @@ def Main():
     # Load Session Data
     # But not if there are command line args for files to open
     if profiler.Profile_Get('SAVE_SESSION', 'bool', False) and not len(args):
-        frame.GetNotebook().LoadSessionFiles()
+        session = profiler.Profile_Get('LAST_SESSION', default=u'')
+        if isinstance(session, list):
+            # Check for format conversion from previous versions
+            profiler.Profile_Set('LAST_SESSION', u'')
+        else:
+            frame.GetNotebook().LoadSessionFile(session)
 
     # Unlike wxMac/wxGTK Windows doesn't post an activate event when a window
     # is first shown, so do it manually to make sure all event handlers get
@@ -622,15 +1003,10 @@ def Main():
     if wx.Platform == '__WXMSW__':
         wx.PostEvent(frame, wx.ActivateEvent(wx.wxEVT_ACTIVATE, True))
 
-    if 'splash' in locals():
-        splash.Destroy()
-
     # Do update check, only check if its been more than a day since the last
     # check
-    tval = sum(time.localtime(time.time())[:3])
-    if profiler.Profile_Get('CHECKUPDATE', default=True) and \
-       tval - profiler.Profile_Get('LASTCHECK', 'int', default=0) > 1:
-        profiler.Profile_Set('LASTCHECK', tval)
+    isadmin = os.access(ed_glob.CONFIG['INSTALL_DIR'], os.R_OK|os.W_OK)
+    if isadmin and profiler.Profile_Get('CHECKUPDATE', default=True):
         uthread = updater.UpdateThread(editra_app, ID_UPDATE_CHECK)
         uthread.start()
 
@@ -642,8 +1018,20 @@ def Main():
         except IndexError:
             dev_tool.DEBUGP("[main][err] IndexError on commandline args")
 
+    # Notify that profile was updated
+    if editra_app.GetProfileUpdated():
+        editra_app.DestroySplash()
+        # Make sure window iniliazes to default position
+        profiler.Profile_Del('WPOS')
+        wx.MessageBox(_("Your profile has been updated to the latest "
+                        "version") + u"\n" + \
+                      _("Please check the preferences dialog to check "
+                        "your preferences"),
+                      _("Profile Updated"))
+
     # 3. Start Applications Main Loop
     dev_tool.DEBUGP("[main][info] Starting MainLoop...")
+    wx.CallAfter(frame.Raise)
     editra_app.MainLoop()
     dev_tool.DEBUGP("[main][info] MainLoop finished exiting application")
     os._exit(0)
