@@ -61,8 +61,16 @@ Requirements:
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: outbuff.py 55938 2008-09-28 21:48:01Z RD $"
-__revision__ = "$Revision: 55938 $"
+__svnid__ = "$Id: outbuff.py 60049 2009-04-07 04:28:52Z CJP $"
+__revision__ = "$Revision: 60049 $"
+
+__all__ = ["OutputBuffer", "OutputBufferEvent", "ProcessBufferMixin",
+           "ProcessThread", "TaskThread", "OPB_STYLE_DEFAULT", "OPB_STYLE_INFO",
+           "OPB_STYLE_WARN", "OPB_STYLE_ERROR", "OPB_STYLE_MAX",
+           "edEVT_PROCESS_START", "EVT_PROCESS_START", "edEVT_TASK_START",
+           "EVT_TASK_START", "edEVT_UPDATE_TEXT", "EVT_UPDATE_TEXT",
+           "edEVT_PROCESS_EXIT", "EVT_PROCESS_EXIT", "edEVT_TASK_COMPLETE",
+           "EVT_TASK_COMPLETE"]
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -161,6 +169,7 @@ class OutputBuffer(wx.stc.StyledTextCtrl):
         self._updating = threading.Condition(self._mutex)
         self._updates = list()
         self._timer = wx.Timer(self)
+        self._line_buffer = -1                
         self._colors = dict(defaultb=(255, 255, 255), defaultf=(0, 0, 0),
                             errorb=(255, 255, 255), errorf=(255, 0, 0),
                             infob=(255, 255, 255), infof=(0, 0, 255),
@@ -188,11 +197,17 @@ class OutputBuffer(wx.stc.StyledTextCtrl):
         self.SetMarginWidth(0, 0)
         self.SetMarginWidth(1, 0)
 
+        # To improve performance at cost of memory cache the document layout
         self.SetLayoutCache(wx.stc.STC_CACHE_DOCUMENT)
         self.SetUndoCollection(False) # Don't keep undo history
         self.SetReadOnly(True)
         self.SetCaretWidth(0)
 
+        if wx.Platform == '__WXMSW__':
+            self.SetEOLMode(wx.stc.STC_EOL_CRLF)
+        else:
+            self.SetEOLMode(wx.stc.STC_EOL_LF)
+
         #self.SetEndAtLastLine(False)
         self.SetVisiblePolicy(1, wx.stc.STC_VISIBLE_STRICT)
 
@@ -201,18 +216,18 @@ class OutputBuffer(wx.stc.StyledTextCtrl):
         self.SetSelBackground(True, highlight)
         self.__SetupStyles()
 
-    def __PutText(self, txt, ind):
-        """
-        @param txt: String to append
-        @param ind: index in update buffer
+    def __FlushBuffer(self):
+        """Flush the update buffer
+        @postcondition: The update buffer is empty
 
         """
         self._updating.acquire()
         self.SetReadOnly(False)
-        start = self.GetLength() # Get position new text will start from
+        txt = u''.join(self._updates[:])
+        start = self.GetLength()
         self.AppendText(txt)
         self.GotoPos(self.GetLength())
-        self._updates = self._updates[ind:]
+        self._updates = list()
         self.ApplyStyles(start, txt)
         self.SetReadOnly(True)
         self._updating.release()
@@ -259,7 +274,8 @@ class OutputBuffer(wx.stc.StyledTextCtrl):
     #---- Public Member Functions ----#
     def AppendUpdate(self, value):
         """Buffer output before adding to window. This method can safely be
-        called from non gui threads to add updates to the buffer.
+        called from non gui threads to add updates to the buffer, that will
+        be displayed durning the next idle period.
         @param value: update string to append to stack
 
         """
@@ -278,6 +294,21 @@ class OutputBuffer(wx.stc.StyledTextCtrl):
         """
         pass
 
+    def CanCopy(self):
+        """Is it possible to copy text right now
+        @return: bool
+
+        """
+        sel = self.GetSelection()
+        return sel[0] != sel[1]
+
+    def CanCut(self):
+        """Is it possible to Cut
+        @return: bool
+
+        """
+        return not self.GetReadOnly()
+
     def Clear(self):
         """Clear the Buffer"""
         self.SetReadOnly(False)
@@ -374,12 +405,30 @@ class OutputBuffer(wx.stc.StyledTextCtrl):
         """
         ind = len(self._updates)
         if ind:
-            wx.CallAfter(self.__PutText, u''.join(self._updates), ind)
+            self.__FlushBuffer()
         elif evt is not None:
             self.DoUpdatesEmpty()
         else:
             pass
 
+    def RefreshBufferedLines(self):
+        """Refresh and readjust the lines in the buffer to fit the current
+        line buffering limits.
+        @postcondition: Oldest lines are removed until we are back within the
+                        buffer limit bounds.
+
+        """
+        if self._line_buffer < 0:
+            return
+
+        while self.GetLineCount() > self._line_buffer:
+            self.SetCurrentPos(0)
+            self.SetReadOnly(False)
+            self.LineDelete()
+            self.SetReadOnly(True)
+
+        self.SetCurrentPos(self.GetLength())
+
     def SetDefaultColor(self, fore=None, back=None):
         """Set the colors for the default text style
         @keyword fore: Foreground Color
@@ -448,6 +497,14 @@ class OutputBuffer(wx.stc.StyledTextCtrl):
         for style in OPB_ALL_STYLES:
             self.StyleSetFont(style, font)
 
+    def SetLineBuffering(self, num):
+        """Set how many lines the buffer should keep for display.
+        @param num: int (-1 == unlimited)
+
+        """
+        self._line_buffer = num
+        self.RefreshBufferedLines()
+
     def SetText(self, text):
         """Set the text that is shown in the buffer
         @param text: text string to set as buffers current value
@@ -585,7 +642,6 @@ class ProcessThread(threading.Thread):
 
     def __DoOneRead(self):
         """Read one line of output and post results.
-        @param proc: process to read from
         @return: bool (True if more), (False if not)
 
         """
@@ -598,9 +654,9 @@ class ProcessThread(threading.Thread):
                 ctypes.windll.kernel32.PeekNamedPipe(handle, None, 0, 0,
                                                      ctypes.byref(avail), None)
                 if avail.value > 0:
-                    read = self._proc.stdout.readline()
+                    read = self._proc.stdout.read(avail.value)
                     if read.endswith(os.linesep):
-                        read = read[:-1 * len(os.linesep)] + "\n"
+                        read = read[:-1 * len(os.linesep)]
                 else:
                     if self._proc.poll() is None:
                         time.sleep(1)
@@ -613,7 +669,6 @@ class ProcessThread(threading.Thread):
             except (subprocess.pywintypes.error, Exception), msg:
                 if msg[0] in (109, errno.ESHUTDOWN):
                     return False
-
         else:
             # OSX and Unix nonblocking pipe read implementation
             if self._proc.stdout is None:
@@ -623,14 +678,17 @@ class ProcessThread(threading.Thread):
             if not self._proc.stdout.closed:
                 fcntl.fcntl(self._proc.stdout,
                             fcntl.F_SETFL,
-                            flags| os.O_NONBLOCK)
+                            flags|os.O_NONBLOCK)
 
             try:
-                if not select.select([self._proc.stdout], [], [], 1)[0]:
-                    return True
+                try:
+                    if not select.select([self._proc.stdout], [], [], 1)[0]:
+                        return True
 
-                read = self._proc.stdout.readline()
-                if read == '':
+                    read = self._proc.stdout.read(4096)
+                    if read == '':
+                        return False
+                except IOError, msg:
                     return False
             finally:
                 if not self._proc.stdout.closed:
@@ -643,7 +701,7 @@ class ProcessThread(threading.Thread):
             result = os.linesep
 
         evt = OutputBufferEvent(edEVT_UPDATE_TEXT, self._parent.GetId(), result)
-        wx.CallAfter(wx.PostEvent, self._parent, evt)
+        wx.PostEvent(self._parent, evt)
         return True
 
     def __KillPid(self, pid):
@@ -668,13 +726,12 @@ class ProcessThread(threading.Thread):
 
             # Try to kill the group
             try:
-                pgid = os.getpgid(pid)
-                os.killpg(pgid, signal.SIGKILL)
+                os.kill(pid, signal.SIGKILL)
             except OSError, msg:
                 pass
 
             # If still alive shoot it again
-            if self._proc.poll() != None:
+            if self._proc.poll() is not None:
                 try:
                     os.kill(-pid, signal.SIGKILL)
                 except OSError, msg:
@@ -682,7 +739,7 @@ class ProcessThread(threading.Thread):
 
             # Try and wait for it to cleanup
             try:
-                os.waitpid(-pid, os.WNOHANG)
+                os.waitpid(pid, os.WNOHANG)
             except OSError, msg:
                 pass
 
@@ -709,20 +766,17 @@ class ProcessThread(threading.Thread):
                                                        self._cmd['args']]])
 
         use_shell = not subprocess.mswindows
-        if use_shell:
-            preexec_fn = os.setsid
-        else:
-            preexec_fn = None
-
-        self._proc = subprocess.Popen(command.strip(), stdout=subprocess.PIPE,
-                                      preexec_fn=preexec_fn,
-                                      stderr=subprocess.STDOUT, shell=use_shell,
-                                      cwd=self._cwd, env=self._env)
+        self._proc = subprocess.Popen(command.strip(),
+                                      stdout=subprocess.PIPE,
+                                      stderr=subprocess.STDOUT,
+                                      shell=use_shell,
+                                      cwd=self._cwd,
+                                      env=self._env)
 
         evt = OutputBufferEvent(edEVT_PROCESS_START,
                                 self._parent.GetId(),
                                 command.strip())
-        wx.CallAfter(wx.PostEvent, self._parent, evt)
+        wx.PostEvent(self._parent, evt)
 
         # Read from stdout while there is output from process
         while True:
@@ -751,7 +805,7 @@ class ProcessThread(threading.Thread):
         # Notify that proccess has exited
         # Pack the exit code as the events value
         evt = OutputBufferEvent(edEVT_PROCESS_EXIT, self._parent.GetId(), result)
-        wx.CallAfter(wx.PostEvent, self._parent, evt)
+        wx.PostEvent(self._parent, evt)
 
     def SetArgs(self, args):
         """Set the args to pass to the command
