@@ -12,22 +12,26 @@ Text/Unicode handling functions and File wrapper class
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_txt.py 55819 2008-09-23 11:40:15Z CJP $"
-__revision__ = "$Revision: 55819 $"
+__svnid__ = "$Id: ed_txt.py 60557 2009-05-09 03:07:55Z CJP $"
+__revision__ = "$Revision: 60557 $"
 
 #--------------------------------------------------------------------------#
 # Imports
-import os
 import sys
 import re
+import time
+import wx
+import threading
 import codecs
 import locale
 import types
 from StringIO import StringIO
 
 # Local Imports
-from util import Log, GetFileModTime
+from util import Log
 from profiler import Profile_Get
+import ed_msg
+import ebmlib
 
 #--------------------------------------------------------------------------#
 # Globals
@@ -48,6 +52,13 @@ BOM = { 'utf-8' : codecs.BOM_UTF8,
 # The first group from this expression will be the encoding.
 RE_MAGIC_COMMENT = re.compile("coding[:=]\s*\"*([-\w.]+)\"*")
 
+# File Load States
+FL_STATE_START   = 0
+FL_STATE_READING = 1
+FL_STATE_PAUSED  = 2
+FL_STATE_END     = 3
+FL_STATE_ABORTED = 4
+
 #--------------------------------------------------------------------------#
 
 class ReadError(Exception):
@@ -60,70 +71,69 @@ class WriteError(Exception):
 
 #--------------------------------------------------------------------------#
 
-class EdFile(object):
+class EdFile(ebmlib.FileObjectImpl):
     """Wrapper for representing a file object that stores data
     about the file encoding and path.
 
     """
-    def __init__(self, path='', modtime=0):
+    def __init__(self, path=u'', modtime=0):
         """Create the file wrapper object
         @param path: the absolute path to the file
 
         """
-        object.__init__(self)
+        ebmlib.FileObjectImpl.__init__(self, path, modtime)
 
         # Attribtues
-        self._handle = None
         self._magic = dict(comment=None, bad=False)
         self.encoding = Profile_Get('ENCODING', default=DEFAULT_ENCODING)
-        self.open = False
-        self.path = path
         self.bom = None
-        self.modtime = modtime
-        self.last_err = None
+        self._mcallback = list()
 
-    def ClearLastError(self):
-        """Reset the error marker on this file"""
-        del self.last_err
-        self.last_err = None
-
-    def Close(self):
-        """Close the file handle
-        @note: this is normally done automatically after a read/write operation
+    def AddModifiedCallback(self, callback):
+        """Set modified callback method
+        @param callback: callable
 
         """
-        try:
-            self._handle.close()
-        except:
-            pass
+        self._mcallback.append(callback)
 
-        self.open = False
+    def CleanUp(self):
+        """Cleanup callback"""
+        pass
 
-    def DoOpen(self, mode):
-        """Opens and creates the internal file object
-        @param mode: mode to open file in
-        @return: True if opened, False if not
-        @postcondition: self._handle is set to the open handle
+    def Clone(self):
+        """Clone the file object
+        @return: EdFile
 
         """
-        if not len(self.path):
-            return False
-
-        try:
-            file_h = open(self.path, mode)
-        except (IOError, OSError), msg:
-            self.last_err = msg
-            return False
-        else:
-            self._handle = file_h
-            self.open = True
-            return True
+        fileobj = EdFile(self._path, self._modtime)
+        fileobj.SetLastError(self.last_err)
+        fileobj.SetEncoding(self.encoding)
+        fileobj.bom = self.bom
+        fileobj._magic = dict(self._magic)
+        for cback in self._mcallback:
+            fileobj.AddModifiedCallback(cback)
+        return fileobj
 
     @property
     def Encoding(self):
         """File encoding property"""
         return self.encoding
 
+    def FireModified(self):
+        """Fire the modified callback(s)"""
+        remove = list()
+        for idx, mcallback in enumerate(self._mcallback):
+            try:
+                mcallback()
+            except:
+                remove.append(idx)
+
+        # Cleanup any bad callbacks
+        if len(remove):
+            remove.reverse()
+            for idx in remove:
+                self._mcallback.pop(idx)
+
     def GetEncoding(self):
         """Get the encoding used by the file it may not be the
         same as the encoding requested at construction time
@@ -132,22 +142,6 @@ class EdFile(object):
         """
         return self.encoding
 
-    def GetExtension(self):
-        """Get the files extension if it has one else simply return the
-        filename minus the path.
-        @return: string file extension (no dot)
-
-        """
-        fname = os.path.split(self.path)
-        return fname[-1].split(os.extsep)[-1].lower()
-
-    def GetLastError(self):
-        """Return the last error that occured when using this file
-        @return: err traceback or None
-
-        """
-        return str(self.last_err).replace("u'", "'")
-
     def GetMagic(self):
         """Get the magic comment if one was present
         @return: string or None
@@ -155,17 +149,6 @@ class EdFile(object):
         """
         return self._magic['comment']
 
-    def GetModtime(self):
-        """Get the timestamp of this files last modification"""
-        return self.modtime
-
-    def GetPath(self):
-        """Get the path of the file
-        @return: string
-
-        """
-        return self.path
-
     def HasBom(self):
         """Return whether the file has a bom byte or not
         @return: bool
@@ -173,56 +156,60 @@ class EdFile(object):
         """
         return self.bom is not None
 
-    def IsOpen(self):
-        """Check if file is open or not
-        @return: bool
-
-        """
-        return self.open
-
-    @property
-    def Modtime(self):
-        """File modification time propery"""
-        return self.GetModtime()
-
-    def Read(self):
+    def Read(self, store=None, chunk=512):
         """Get the contents of the file as a string, automatically handling
         any decoding that may be needed.
 
+        @keyword store: generator method to pass read results through
+        @keyword chunk: read size
+        @note: the store param overrides incremental
         @return: unicode str
 
         """
         if self.DoOpen('rb'):
-            lines = [ self._handle.readline() for x in range(2) ]
-            self._handle.seek(0)
+            lines = [ self.Handle.readline() for x in range(2) ]
+            self.Handle.seek(0)
             enc = None
             if len(lines):
                 enc = CheckBom(lines[0])
                 if enc is None:
                     self.bom = None
-                    enc = CheckMagicComment(lines)
-                    if enc:
-                        self._magic['comment'] = enc
+                    if not self._magic['bad']:
+                        enc = CheckMagicComment(lines)
+                        if enc:
+                            self._magic['comment'] = enc
                 else:
                     Log("[ed_txt][info] File Has %s BOM" % enc)
                     self.bom = unicode(BOM.get(enc, None), enc)
 
             if enc is not None:
                 self.encoding = enc
-
             try:
-                reader = codecs.getreader(self.encoding)(self._handle)
-                txt = reader.read()
+                reader = codecs.getreader(self.encoding)(self.Handle)
+                txt = u''
+                if store is None:
+                    while 1:
+                        tmp = reader.read(chunk)
+                        if not len(tmp):
+                            break
+                        txt += tmp
+                else:
+                    store(reader, chunk)
                 reader.close()
             except Exception, msg:
                 Log("[ed_txt][err] Error while reading with %s" % self.encoding)
                 Log("[ed_txt][err] %s" % msg)
-                self.last_err = str(msg)
+                self.SetLastError(unicode(msg))
                 self.Close()
                 if self._magic['comment']:
                     self._magic['bad'] = True
-                enc, txt = FallbackReader(self.path)
+
+                enc, txt = FallbackReader(self.GetPath())
                 if enc is not None:
+                    Log("[ed_txt][info] Fallback reader succeeded with: %s" % enc)
+                    # TODO: don't clear the error and allow the client to
+                    #       analyze it for error reporting.
+                    self.ClearLastError()
                     self.encoding = enc
                 else:
                     raise UnicodeDecodeError, msg
@@ -231,22 +218,92 @@ class EdFile(object):
                 Log("[ed_txt][info] Stripping %s BOM from text" % self.encoding)
                 txt = txt.replace(self.bom, u'', 1)
 
-            Log("[ed_txt][info] Decoded %s with %s" % (self.path, self.encoding))
-            self.SetModTime(GetFileModTime(self.path))
+            Log("[ed_txt][info] Decoded %s with %s" % (self.GetPath(), self.encoding))
+            self.SetModTime(ebmlib.GetFileModTime(self.GetPath()))
             return txt
         else:
-            raise ReadError, self.last_err
+            raise ReadError, self.GetLastError()
+
+    def ReadAsync(self, control):
+        """Read the file asynchronously on a separate thread
+        @param control: text control to send text to
+
+        """
+        pid = control.GetTopLevelParent().GetId()
+        filesize = ebmlib.GetFileSize(self.GetPath())
+        ed_msg.PostMessage(ed_msg.EDMSG_PROGRESS_STATE, (pid, 1, filesize))
+        thread = FileReadThread(control, self.ReadGenerator, 4096)
+        thread.start()
+
+    def ReadGenerator(self, chunk=512):
+        """Get the contents of the file as a string, automatically handling
+        any decoding that may be needed.
+
+        @keyword chunk: read size
+        @return: unicode str
+
+        """
+        if self.DoOpen('rb'):
+            lines = [ self.Handle.readline() for x in range(2) ]
+            self.Handle.seek(0)
+            enc = None
+            if len(lines):
+                enc = CheckBom(lines[0])
+                if enc is None:
+                    self.bom = None
+                    if not self._magic['bad']:
+                        enc = CheckMagicComment(lines)
+                        if enc:
+                            self._magic['comment'] = enc
+                else:
+                    Log("[ed_txt][info] File Has %s BOM" % enc)
+                    self.bom = unicode(BOM.get(enc, None), enc)
+
+            if enc is not None:
+                self.encoding = enc
+
+            try:
+                reader = codecs.getreader(self.encoding)(self.Handle)
+                while 1:
+                    tmp = reader.read(chunk)
+                    if not len(tmp):
+                        break
+                    yield tmp
+                reader.close()
+            except Exception, msg:
+                Log("[ed_txt][err] Error while reading with %s" % self.encoding)
+                Log("[ed_txt][err] %s" % msg)
+                self.SetLastError(unicode(msg))
+                self.Close()
+                if self._magic['comment']:
+                    self._magic['bad'] = True
+
+                # TODO: handle incremental mode for 
+#                enc, txt = FallbackReader(self.path)
+#                if enc is not None:
+#                    self.encoding = enc
+#                else:
+#                    raise UnicodeDecodeError, msg
+
+            Log("[ed_txt][info] Decoded %s with %s" % (self.GetPath(), self.encoding))
+            self.SetModTime(ebmlib.GetFileModTime(self.GetPath()))
+        else:
+            raise ReadError, self.GetLastError()
+
+    def RemoveModifiedCallback(self, callback):
+        """Remove a registered callback
+        @param callback: callable to remove
+
+        """
+        if callback in self._mcallback:
+            self._mcallback.remove(callback)
 
     def ResetAll(self):
         """Reset all attributes of this file"""
-        self._handle = None
+        super(EdFile, self).ResetAll()
         self._magic = dict(comment=None, bad=False)
         self.encoding = Profile_Get('ENCODING', default=DEFAULT_ENCODING)
-        self.open = False
-        self.path = ''
         self.bom = None
-        self.modtime = 0
-        self.last_err = None
 
     def SetEncoding(self, enc):
         """Explicitly set/change the encoding of the file
@@ -257,20 +314,6 @@ class EdFile(object):
             enc = DEFAULT_ENCODING
         self.encoding = enc
 
-    def SetPath(self, path):
-        """Set the path of the file
-        @param path: absolute path to file
-
-        """
-        self.path = path
-
-    def SetModTime(self, mtime):
-        """Set the modtime of this file
-        @param mtime: long int to set modtime to
-
-        """
-        self.modtime = mtime
-
     def ReadLines(self):
         """Get the contents of the file as a list of lines
         @return: list of strings
@@ -288,8 +331,6 @@ class EdFile(object):
         # Check if a magic comment was added or changed
         tbuff = StringIO(value)
         enc = CheckMagicComment([ tbuff.readline() for x in range(2) ])
-        tbuff.close()
-        del tbuff
 
         # Update encoding if necessary
         if enc is not None:
@@ -297,16 +338,109 @@ class EdFile(object):
 
         # Open and write the file
         if self.DoOpen('wb'):
-            Log("[ed_txt][info] Opened %s, writing as %s" % (self.path, self.encoding))
-            writer = codecs.getwriter(self.encoding)(self._handle)
+            Log("[ed_txt][info] Opened %s, writing as %s" % (self.GetPath(), self.encoding))
+            writer = codecs.getwriter(self.encoding)(self.Handle)
             if self.HasBom():
                 Log("[ed_txt][info] Adding BOM back to text")
-                value = self.bom + value
-            writer.write(value)
+                writer.write(self.bom)
+
+            writer.write(tbuff.getvalue())
+            tbuff.close()
+            del tbuff
             writer.close()
-            Log("[ed_txt][info] %s was written successfully" % self.path)
+            Log("[ed_txt][info] %s was written successfully" % self.GetPath())
         else:
-            raise WriteError, self.last_err
+            raise WriteError, self.GetLastError()
+
+#-----------------------------------------------------------------------------#
+
+class FileReadThread(threading.Thread):
+    """Worker thread for reading text from a file"""
+    def __init__(self, reciever, task, *args, **kwargs):
+        """Create the thread
+        @param reciever: Window to recieve events
+        @param task: generator method to call
+
+        """
+        threading.Thread.__init__(self)
+
+        # Attributes
+        self.cancel = False
+        self._task = task
+        self.reciever = reciever
+        self._args = args
+        self._kwargs = kwargs
+        self.pid = reciever.GetTopLevelParent().GetId()
+
+    def run(self):
+        """Read the text"""
+        evt = FileLoadEvent(edEVT_FILE_LOAD, wx.ID_ANY, None, FL_STATE_START)
+        wx.PostEvent(self.reciever, evt)
+        time.sleep(.75) # give ui a chance to get ready
+
+        count = 1
+        for txt in self._task(*self._args, **self._kwargs):
+            if self.cancel:
+                break
+
+            evt = FileLoadEvent(edEVT_FILE_LOAD, wx.ID_ANY, txt)
+            evt.SetProgress(count * self._args[0])
+            wx.PostEvent(self.reciever, evt)
+            count += 1
+
+        evt = FileLoadEvent(edEVT_FILE_LOAD, wx.ID_ANY, None, FL_STATE_END)
+        wx.PostEvent(self.reciever, evt)
+
+    def Cancel(self):
+        """Cancel the running task"""
+        self.cancel = True
+
+#-----------------------------------------------------------------------------#
+
+edEVT_FILE_LOAD = wx.NewEventType()
+EVT_FILE_LOAD = wx.PyEventBinder(edEVT_FILE_LOAD, 1)
+class FileLoadEvent(wx.PyEvent):
+    """Event to signal that a chunk of text haes been read"""
+    def __init__(self, etype, eid, value=None, state=FL_STATE_READING):
+        """Creates the event object"""
+        wx.PyEvent.__init__(self, eid, etype)
+
+        # Attributes
+        self._state = state
+        self._value = value
+        self._prog = 0
+    
+    def HasText(self):
+        """Returns true if the event has text
+        @return: bool whether the event contains text
+
+        """
+        return self._value is not None
+
+    def GetProgress(self):
+        """Get the current progress of the load"""
+        return self._prog
+
+    def GetState(self):
+        """Get the state of the file load action
+        @return: int (FL_STATE_FOO)
+
+        """
+        return self._state
+
+    def GetValue(self):
+        """Returns the value from the event.
+        @return: the value of this event
+
+        """
+        return self._value
+
+    def SetProgress(self, progress):
+        """Set the number of bytes that have been read
+        @param progress: int
+
+        """
+        self._prog = progress
 
 #-----------------------------------------------------------------------------#
 # Utility Function
@@ -351,7 +485,7 @@ def DecodeString(string, encoding=None):
     @keyword encoding: encoding to decode string with
 
     """
-    if not encoding:
+    if encoding is None:
         encoding = DEFAULT_ENCODING
 
     if not isinstance(string, types.UnicodeType):
