@@ -20,19 +20,24 @@ similar services for manipulating and transforming text.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__cvsid__ = "$Id: generator.py 52855 2008-03-27 14:53:06Z CJP $"
-__revision__ = "$Revision: 52855 $"
+__svnid__ = "$Id: generator.py 60508 2009-05-03 21:54:15Z CJP $"
+__revision__ = "$Revision: 60508 $"
 
 #--------------------------------------------------------------------------#
-# Dependancies
+# Imports
 import wx
 import wx.stc
+import time
+
+# Editra Libraries
 import ed_glob
 import ed_menu
 from ed_style import StyleItem
 import util
 import plugin
-import time
+import ebmlib
+import eclib
+
 #--------------------------------------------------------------------------#
 # Globals
 _ = wx.GetTranslation
@@ -75,6 +80,8 @@ class GeneratorI(plugin.Interface):
         """
         pass
 
+#-----------------------------------------------------------------------------#
+
 class Generator(plugin.Plugin):
     """Plugin Interface Extension Point for Generator
     type plugin objects. Generator objects are used
@@ -90,6 +97,7 @@ class Generator(plugin.Plugin):
         @type menu: wx.Menu
 
         """
+        # Fetch all the menu items for each generator object
         menu_items = list()
         for observer in self.observers:
             try:
@@ -99,6 +107,7 @@ class Generator(plugin.Plugin):
             except Exception, msg:
                 util.Log("[generator][err] %s" % str(msg))
 
+        # Construct the menu
         menu_items.sort()
         genmenu = ed_menu.EdMenu()
         for item in menu_items:
@@ -118,13 +127,15 @@ class Generator(plugin.Plugin):
         """
         gentext = None
         start = time.time()
+        # Find the correct generator and run its generate method on the
+        # given text control.
         for observer in self.observers:
             if observer.GetId() == e_id:
                 gentext = observer.Generate(txt_ctrl)
         util.Log("[generator][info] Generation time %f" % (time.time() - start))
         return gentext
 
-#--------------------------------------------------------------------------#
+#-----------------------------------------------------------------------------#
 
 class Html(plugin.Plugin):
     """Transforms the text from a given Editra stc to a fully
@@ -139,7 +150,7 @@ class Html(plugin.Plugin):
         @param mgr: This generators plugin manager
 
         """
-        plugin.Plugin.__init__(self, mgr)
+        plugin.Plugin.__init__(self)
 
         # Attributes
         self._id = ed_glob.ID_HTML_GEN
@@ -153,13 +164,18 @@ class Html(plugin.Plugin):
         @return: string version of html object
 
         """
+        # Assemble the embedded html
         style = "<style type=\"text/css\">\n%s</style>"
         css = wx.EmptyString
         for key in self.css:
             css += str(self.css[key]) + "\n"
         css = css % self.stc.GetFontDictionary()
         style = style % css
+
+        # Insert the css into the head
         head = self.head.replace('</head>', style + "\n</head>")
+
+        # Assemble the body of the html
         html = "<html>\n%s\n%s\n</html>"
         html = html % (head, self.body)
         return html
@@ -191,7 +207,7 @@ class Html(plugin.Plugin):
                "<meta name=\"Generator\" content=\"Editra/%s\">\n" \
                "<meta http-equiv=\"content-type\" content=\"text/html; " \
                "charset=utf-8\">" \
-               "\n</head>" % (util.GetFileName(self.stc.GetFileName()),
+               "\n</head>" % (ebmlib.GetFileName(self.stc.GetFileName()),
                               ed_glob.VERSION)
 
     def GenerateBody(self):
@@ -241,18 +257,20 @@ class Html(plugin.Plugin):
                 last_id = curr_id
                 style_start = style_end
                 tag = stc.FindTagById(last_id)
-                if not self.css.has_key(tag):
+                if tag not in self.css:
                     s_item = StyleItem()
                     s_item.SetAttrFromStr(stc.GetStyleByName(tag))
                     self.css[tag] = CssItem(tag.split('_')[0], s_item)
+
+        # Case for unstyled documents
         if len(html) == 0:
-            # Case for unstyled documents
             s_item = StyleItem()
             s_item.SetAttrFromStr(stc.GetStyleByName('default_style'))
             self.css['default_style'] = CssItem('default', s_item)
             html.append(self.TransformText(stc.GetText()))
         else:
             self.OptimizeCss()
+
         return "<body class=\"default\">\n<pre>\n%s\n</pre>\n</body>" % \
                                                                    "".join(html)
 
@@ -277,10 +295,15 @@ class Html(plugin.Plugin):
         @postcondition: css is optimized to remove any redundant entries
 
         """
-        if not self.css.has_key('default_style'):
+        # Must have the default style defined
+        if 'default_style' not in self.css:
             return
-        if self.css.has_key('operator_style'):
+
+        # Don't style operators. This is to optimize the html size
+        if 'operator_style' in self.css:
             self.css.pop('operator_style')
+
+        # All other css elements will inheirit from the default
         default = self.css['default_style']
         for key in self.css:
             if key == 'default_style':
@@ -310,7 +333,9 @@ class Html(plugin.Plugin):
         text = text.replace("\"", "&quot;")
         return text
 
-class CssItem(object):
+#-----------------------------------------------------------------------------#
+
+class CssItem:
     """Converts an Edtira StyleItem to a Css item for use in
     generating html.
 
@@ -326,14 +351,17 @@ class CssItem(object):
         @see: L{ed_style}
 
         """
-        object.__init__(self)
+
+        # Attributes
         self._tag = class_tag
         self._back = style_item.GetBack()
         self._fore = style_item.GetFore()
         self._font = style_item.GetFace()
         self._size = style_item.GetSize()
+
         # List of additional style specs
         self._decor = self.ExtractDecorators()
+        self._decor.extend(style_item.GetModifierList())
 
     def __eq__(self, css2):
         """Defines the == operator for the CssItem class
@@ -342,16 +370,14 @@ class CssItem(object):
         @rtype: bool
 
         """
-        if self.__str__() == str(css2):
-            return True
-        else:
-            return False
+        return self.__str__() == str(css2)
 
     def __str__(self):
         """Outputs the css item as a formatted css block
         @return: CssItem as a string
 
         """
+        # Generate the main style attribures
         css = ".%s {\n%s}"
         css_body = wx.EmptyString
         if self._font != wx.EmptyString:
@@ -366,6 +392,8 @@ class CssItem(object):
         if self._back != wx.EmptyString:
             back = self._back.split(',')
             css_body += u"\tbackground-color: %s;\n" % back[0]
+
+        # Add additional style modifiers
         for item in self._decor:
             if item == u'bold':
                 css_body += u"\tfont-weight: %s;\n" % item
@@ -375,6 +403,8 @@ class CssItem(object):
                 css_body += u"\ttext-decoration: %s;\n" % item
             else:
                 pass
+
+        # Format the tag and body into the css def
         if css_body != wx.EmptyString:
             return css % (self._tag, css_body)
         else:
@@ -484,7 +514,9 @@ class LaTeX(plugin.Plugin):
         @param plgmgr: pluginmanger for this object
 
         """
-        plugin.Plugin.__init__(self, plgmgr)
+        plugin.Plugin.__init__(self)
+
+        # Attributes
         self._stc = None
         self._id = ed_glob.ID_TEX_GEN
         self._dstyle = StyleItem()
@@ -574,8 +606,9 @@ class LaTeX(plugin.Plugin):
                     self.RegisterStyleCmd(tag, stc.GetItemByName(tag))
                 tmp = list()
                 start = parse_pos
+
+        # Case for unstyled documents
         if tex == wx.EmptyString:
-            # Case for unstyled documents
             tex.append(self.TransformText(stc.GetText()))
         return "\\begin{document}\n%s\n\\end{document}" % "".join(tex)
 
@@ -600,6 +633,7 @@ class LaTeX(plugin.Plugin):
         @return: the LaTeX document preamble
 
         """
+        # Preamble template
         pre = ("%% \iffalse meta-comment\n"
                "%%\n%% Generated by Editra %s\n"
                "%% This is generator is Very Experimental.\n"
@@ -614,9 +648,13 @@ class LaTeX(plugin.Plugin):
                "\\usepackage{color}\n"
                "\\usepackage{alltt}\n"
                "\\usepackage{times}\n") % ed_glob.VERSION
+
+        # Set the background color
         pre += ("\\pagecolor[rgb]{%s}\n" % \
                 self.HexToRGB(self._dstyle.GetBack()))
         pre += "\\parindent=0in\n\n"
+
+        # Insert all styling commands
         pre += "%% Begin Styling Command Definitions"
         for cmd in self._cmds:
             pre += ("\n" + self._cmds[cmd])
@@ -665,6 +703,8 @@ class LaTeX(plugin.Plugin):
 
         """
         cmd_name = self.CreateCmdName(cmd_name)
+
+        # If we already made a command for this style return
         if cmd_name in self._cmds:
             return
 
@@ -735,7 +775,9 @@ class Rtf(plugin.Plugin):
         @param mgr: plugin manager of this object
 
         """
-        plugin.Plugin.__init__(self, mgr)
+        plugin.Plugin.__init__(self)
+
+        # Attributes
         self._stc = None
         self._id = ed_glob.ID_RTF_GEN
         self._colortbl = RtfColorTbl()
@@ -755,8 +797,11 @@ class Rtf(plugin.Plugin):
         @return: generated rtf marked up text
 
         """
-        if not self._stc:
+        # Buffer hasn't been set
+        if self._stc is None:
             return u''
+
+        # Optimizations
         stc = self._stc
         def_fore = stc.GetDefaultForeColour(as_hex=True)
         self._colortbl.AddColor(def_fore)
@@ -775,9 +820,12 @@ class Rtf(plugin.Plugin):
         AddColor = self._colortbl.AddColor
         GetColorIndex = self._colortbl.GetColorIndex
         GetStyleAt = stc.GetStyleAt
+
+        # Parse all characters/style bytes in document
         for parse_pos in xrange(last_pos + 1):
             sty_id = GetStyleAt(parse_pos)
             end = parse_pos
+
             # If style has changed build the previous section
             if sty_id != last_id:
                 tag = stc.FindTagById(last_id)
@@ -797,6 +845,7 @@ class Rtf(plugin.Plugin):
                                self.TransformText(stc.GetTextRange(start, end)))
                 start = end
             last_id = sty_id
+
         head = "{\\rtf1\\ansi\\deff0{\\fonttbl{\\f0 %s;}}" % \
                 stc.GetDefaultFont().GetFaceName()
         return u"%s%s%s}" % (head, self._colortbl, "".join(tmp_txt))
@@ -844,7 +893,9 @@ class Rtf(plugin.Plugin):
             tmp = tmp + chmap.get(char, char)
         return tmp
 
-class RtfColorTbl(object):
+#-----------------------------------------------------------------------------#
+
+class RtfColorTbl:
     """A storage class to help with generating the color table for
     the Rtf Generator Class.
     @see: Rtf
@@ -855,8 +906,6 @@ class RtfColorTbl(object):
         @summary: creates an object for managing an rtf documents color table
 
         """
-        object.__init__(self)
-
         # Attributes
         self._index = list() # manages the order of the tables keys
         self._tbl = dict()   # map of style item color vals to rtf defs
@@ -879,7 +928,7 @@ class RtfColorTbl(object):
 
         """
         if si_color not in self._index:
-            rgb = util.HexToRGB(si_color.split(u',')[0])
+            rgb = eclib.HexToRGB(si_color.split(u',')[0])
             color = "\\red%d\\green%d\\blue%d;" % tuple(rgb)
             self._index.append(si_color)
             self._tbl[si_color] = color
