@@ -19,13 +19,15 @@ Intefaces:
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: iface.py 54044 2008-06-09 01:55:12Z CJP $"
-__revision__ = "$Revision: 54044 $"
+__svnid__ = "$Id: iface.py 59754 2009-03-23 01:01:11Z CJP $"
+__revision__ = "$Revision: 59754 $"
 
 #--------------------------------------------------------------------------#
-# Dependancies
+# Imports
 import re
 import wx
+
+# Local Imports
 import plugin
 from extern import flatnotebook as FNB
 from profiler import Profile_Get
@@ -106,6 +108,14 @@ class ShelfI(plugin.Interface):
         """
         raise NotImplementedError
 
+    def GetBitmap(self):
+        """Get the bitmap to show in the shelf for this item
+        @return: wx.Bitmap
+        @note: this method is optional
+
+        """
+        return wx.NullBitmap
+
     def GetId(self):
         """Return the id that identifies this item (same as the menuid)
         @return: Item ID
@@ -117,7 +127,7 @@ class ShelfI(plugin.Interface):
     def GetMenuEntry(self, menu):
         """Returns the menu entry associated with this item
         @param menu: The menu this entry will be added to
-        @return: wx.MenuItem
+        @return: wx.MenuItem or None if no menu entry is needed
 
         """
         raise NotImplementedError
@@ -150,7 +160,6 @@ class ShelfI(plugin.Interface):
 
         """
         return True
-        
 
 #-----------------------------------------------------------------------------#
 
@@ -171,6 +180,8 @@ class Shelf(plugin.Plugin):
         self._shelf = None
         self._parent = None
         self._open = dict()
+        self._imgidx = dict()
+        self._imglst = wx.ImageList(16, 16)
 
     def _GetMenu(self):
         """Return the menu of this object
@@ -178,7 +189,8 @@ class Shelf(plugin.Plugin):
 
         """
         menu = ed_menu.EdMenu()
-        menu.Append(ed_glob.ID_SHOW_SHELF, _("Show Shelf") + "\tCtrl+Alt+S", 
+        menu.Append(ed_glob.ID_SHOW_SHELF, _("Show Shelf") + \
+                    ed_menu.EdMenuBar.keybinder.GetBinding(ed_glob.ID_SHOW_SHELF), 
                     _("Show the Shelf"))
         menu.AppendSeparator()
         menu_items = list()
@@ -187,7 +199,7 @@ class Shelf(plugin.Plugin):
             self._open[observer.GetName()] = 0
             try:
                 menu_i = observer.GetMenuEntry(menu)
-                if menu_i:
+                if menu_i is not None:
                     menu_items.append((menu_i.GetItemLabel(), menu_i))
             except Exception, msg:
                 self._log("[shelf][err] %s" % str(msg))
@@ -200,7 +212,7 @@ class Shelf(plugin.Plugin):
             menu.AppendItem(item[1])
         return menu
 
-    def AddItem(self, item, name):
+    def AddItem(self, item, name, imgid=-1):
         """Add an item to the shelfs notebook. This is usefull for interacting
         with the Shelf from outside its interface. It may be necessary to
         call L{EnsureShelfVisible} before or after adding an item if you wish
@@ -209,7 +221,12 @@ class Shelf(plugin.Plugin):
         @param name: Items name used for page text in notebook
 
         """
-        self._shelf.AddPage(item, u"%s - %d" % (name, self._open.get(name, 0)))
+        self._shelf.AddPage(item,
+                            u"%s - %d" % (name, self._open.get(name, 0)))
+
+        # Set the tab icon
+        if imgid >= 0 and Profile_Get('TABICONS', default=True):
+            self._shelf.SetPageImage(self._shelf.GetPageCount()-1, imgid)
         self._open[name] = self._open.get(name, 0) + 1
 
     def CanStockItem(self, item_name):
@@ -233,7 +250,7 @@ class Shelf(plugin.Plugin):
 
         """
         handlers = [ (item.GetId(), self.UpdateShelfMenuUI)
-                     for item in self.observers]
+                     for item in self.observers ]
         return handlers
 
     def Init(self, parent):
@@ -248,10 +265,14 @@ class Shelf(plugin.Plugin):
             return
 
         self._shelf = FNB.FlatNotebook(parent,
-                                       style=FNB.FNB_FF2 | \
-                                             FNB.FNB_X_ON_TAB | \
-                                             FNB.FNB_BACKGROUND_GRADIENT | \
-                                             FNB.FNB_NODRAG)
+                                       style=FNB.FNB_FF2 |
+                                             FNB.FNB_X_ON_TAB |
+                                             FNB.FNB_BACKGROUND_GRADIENT |
+                                             FNB.FNB_NODRAG |
+                                             FNB.FNB_BOTTOM |
+                                             FNB.FNB_NO_X_BUTTON |
+                                             FNB.FNB_MOUSE_MIDDLE_CLOSES_TABS)
+        self._shelf.SetImageList(self._imglst)
         mgr.AddPane(self._shelf, wx.aui.AuiPaneInfo().Name(Shelf.SHELF_NAME).\
                             Caption("Shelf").Bottom().Layer(0).\
                             CloseButton(True).MaximizeButton(False).\
@@ -450,12 +471,22 @@ class Shelf(plugin.Plugin):
 
         name = item.GetName()
         if self.ItemIsOnShelf(name) and \
-            not item.AllowMultiple() or \
-            self._shelf is None:
+            not item.AllowMultiple() or self._shelf is None:
             return
         else:
             self.EnsureShelfVisible()
-            self.AddItem(item.CreateItem(self._shelf), name)
+            item_id = item.GetId()
+            index = -1
+            if hasattr(item, 'GetBitmap'):
+                if item_id in self._imgidx:
+                    index = self._imgidx[item_id]
+                else:
+                    bmp = item.GetBitmap()
+                    if bmp.IsOk():
+                        index = self._imglst.Add(bmp)
+                        self._imgidx[item_id] = index
+
+            self.AddItem(item.CreateItem(self._shelf), name, index)
 
     def ItemIsOnShelf(self, item_name):
         """Check if at least one instance of a given item
@@ -471,6 +502,35 @@ class Shelf(plugin.Plugin):
                 return True
         return False
 
+    def RaiseItem(self, item_name):
+        """Set the selection in the notebook to be the that of the first
+        instance of item_name that is found in the shelf.
+        @param item_name: ShelfI name
+        @return: reference to the selected page or None if no instance is
+
+        """
+        for page in xrange(self._shelf.GetPageCount()):
+            if self._shelf.GetPageText(page).startswith(item_name):
+                self._shelf.SetSelection(page)
+                return self._shelf.GetPage(page)
+        else:
+            return None
+
+    def RaiseWindow(self, window):
+        """Set the selection in the notebook to be the that of the given
+        window. Mostly used internally by items implementing L{ShelfI}.
+        @param window: Window object
+        @return: reference to the selected page or None if no instance is
+
+        """
+        for page in xrange(self._shelf.GetPageCount()):
+            ctrl = self._shelf.GetPage(page)
+            if window == ctrl:
+                self._shelf.SetSelection(page)
+                return ctrl
+        else:
+            return None
+
     def StockShelf(self, i_list):
         """Fill the shelf by opening an ordered list of items
         @param i_list: List of named L{ShelfI} instances
@@ -494,8 +554,6 @@ class Shelf(plugin.Plugin):
             evt.Skip()
             return
 
-        evt.SetMode(wx.UPDATE_UI_PROCESS_SPECIFIED)
-        evt.SetUpdateInterval(200)
         count = self.GetCount(item.GetName())
         if count and not item.AllowMultiple():
             evt.Enable(False)
