@@ -60,8 +60,8 @@ memory is not likely to be very large.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: plugin.py 54279 2008-06-18 19:36:44Z CJP $"
-__revision__ = "$Revision: 54279 $"
+__svnid__ = "$Id: plugin.py 59902 2009-03-28 16:21:50Z CJP $"
+__revision__ = "$Revision: 59902 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -71,7 +71,7 @@ import wx
 
 # Editra Libraries
 import ed_glob
-from ed_txt import DecodeString
+from ed_txt import EncodeString
 import util
 from profiler import CalcVersionValue
 
@@ -178,11 +178,12 @@ class Plugin(object):
     def __name__(self):
         return 'EdPlugin'
 
-    def __new__(cls, *args, **kwargs):
+    def __new__(cls, pluginmgr):
         """Only one instance of each plugin is allowed to exist
         per manager. If an instance of this plugin has already be
         initialized, that instance will be returned. If not this will
         initialize a new instance of the plugin.
+        @keyword mgr: Plugin Manager instance
         @return: a new class object or an existing instance if one
                  exists.
 
@@ -193,7 +194,6 @@ class Plugin(object):
             self.pluginmgr = self
             return self
 
-        pluginmgr = args[0]
         self = pluginmgr.GetPlugins().get(cls)
         if self is None:
             self = super(Plugin, cls).__new__(cls)
@@ -227,6 +227,39 @@ class Plugin(object):
 
 #-----------------------------------------------------------------------------#
 
+class PluginConfigObject(object):
+    """Plugin configuration object. Plugins that wish to provide a
+    configuration panel should implement a subclass of this object
+    in their __init__ module. The __init__ module must also have a
+    function 'GetConfigObject' that returns an instance of this
+    class.
+
+    """
+    def GetConfigPanel(self, parent):
+        """Get the configuration panel for this plugin
+        @param parent: parent window for the panel
+        @return: wxPanel
+
+        """
+        raise NotImplementedError
+
+    def GetBitmap(self):
+        """Get the 32x32 bitmap to show in the config dialog
+        @return: wx.Bitmap
+        @note: Optional if not implemented default icon will be used
+
+        """
+        return wx.NullBitmap
+
+    def GetLabel(self):
+        """Get the display label for the configuration
+        @return: string
+
+        """
+        raise NotImplementedError
+
+#-----------------------------------------------------------------------------#
+
 class PluginData(object):
     """A storage class for representing data about a Plugin
     @see: L{Plugin}
@@ -341,13 +374,13 @@ class PluginManager(object):
         self.LOG = wx.GetApp().GetLog()
         self._config = self.LoadPluginConfig() # Enabled/Disabled Plugins
         self._pi_path = list(set([ed_glob.CONFIG['PLUGIN_DIR'], 
-                             ed_glob.CONFIG['SYS_PLUGIN_DIR']]))
+                                  ed_glob.CONFIG['SYS_PLUGIN_DIR']]))
         sys.path.extend(self._pi_path)
         self._env = self.CreateEnvironment(self._pi_path)
         self._plugins = dict()      # Set of available plugins
         self._enabled = dict()      # Set of enabled plugins
         self._loaded = list()       # List of 
-        self._obsolete = list()     # Obsolete plugins list
+        self._obsolete = dict()     # Obsolete plugins list
         self.InitPlugins(self._env)
         self.RefreshConfig()
 
@@ -422,14 +455,19 @@ class PluginManager(object):
         in path list
         @param path: path(s) to scan for extension points
         @type path: list of path strings
+        @note: pkgutils does not like Unicode! only send encoded strings
 
         """
         if pkg_resources != None:
-            path = [ DecodeString(pname, sys.getfilesystemencoding())
+            path = [ EncodeString(pname, sys.getfilesystemencoding())
                      for pname in path ]
-            env = pkg_resources.Environment(path)
+
+            try:
+                env = pkg_resources.Environment(path)
+            except UnicodeDecodeError, msg:
+                self.LOG("[pluginmgr][err] %s" % msg)
         else:
-            self.LOG("[pluginmgr][warn] setup tools is not installed")
+            self.LOG("[pluginmgr][warn] setuptools is not installed")
             env = dict()
         return env
 
@@ -480,7 +518,7 @@ class PluginManager(object):
     def GetIncompatible(self):
         """Get the list of loaded plugins that are incompatible with the
         current running version of Editra.
-        return: list of strings
+        return: dict(name=module)
 
         """
         return self._obsolete
@@ -506,8 +544,13 @@ class PluginManager(object):
             return
 
         pkg_env = env
+        tmploaded = [ name.lower() for name in self._loaded ]
         for name in pkg_env:
             self.LOG("[pluginmgr][info] Found plugin: %s" % name)
+            if name.lower() in tmploaded:
+                self.LOG("[pluginmgr][info] %s is already loaded" % name)
+                continue
+
             egg = pkg_env[name][0]  # egg is of type Distrobution
             egg.activate()
             for name in egg.get_entry_map(ENTRYPOINT):
@@ -518,10 +561,10 @@ class PluginManager(object):
                         cls = entry_point.load()
                         self._loaded.append(name)
                     else:
+                        self.LOG("[pluginmgr][info] Skip reloading: %s" % name)
                         continue
                 except Exception, msg:
-                    self.LOG("[pluginmgr][err] Couldn't Load %s: %s" % \
-                                                          (str(name), str(msg)))
+                    self.LOG("[pluginmgr][err] Couldn't Load %s: %s" % (name, msg))
                 else:
                     try:
                         # Only initialize plugins that haven't already been
@@ -533,7 +576,11 @@ class PluginManager(object):
                             if minv <= CalcVersionValue(ed_glob.VERSION):
                                 self._plugins[cls] = cls(self)
                             else:
-                                self._obsolete.append(name)
+                                # Save plugins that are not compatible with
+                                # this version to use for notifications.
+                                self._obsolete[name] = cls.__module__
+                        else:
+                            self.LOG("[pluginmgr][info] Skip re-init of %s" % cls)
                     finally:
                         pass
 
