@@ -17,8 +17,8 @@ editor can load to configure the styles of the text.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: style_editor.py 53723 2008-05-23 15:53:53Z CJP $"
-__revision__ = "$Revision: 53723 $"
+__svnid__ = "$Id: style_editor.py 60049 2009-04-07 04:28:52Z CJP $"
+__revision__ = "$Revision: 60049 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -29,11 +29,11 @@ import wx
 # Editra Imports
 import ed_glob
 from profiler import Profile_Get, Profile_Set
-import ed_stc
+import ed_basestc
 from ed_style import StyleItem
 import util
 import syntax.syntax as syntax
-import eclib.colorsetter as colorsetter
+import eclib
 
 # Function Aliases
 _ = wx.GetTranslation
@@ -70,8 +70,8 @@ class StyleEditor(wx.Dialog):
 
         # Attributes
         self.LOG = wx.GetApp().GetLog()
-        self.preview = ed_stc.EditraStc(self, wx.ID_ANY, size=(-1, 200),
-                                        style=wx.SUNKEN_BORDER, use_dt=False)
+        self.preview = ed_basestc.EditraBaseStc(self, wx.ID_ANY, size=(-1, 200),
+                                                style=wx.SUNKEN_BORDER)
         self.styles_orig = self.preview.GetStyleSet()
         self.preview.SetCaretLineVisible(True)
         self.styles_new = DuplicateStyleDict(self.styles_orig)
@@ -136,8 +136,10 @@ class StyleEditor(wx.Dialog):
         b_sizer = wx.BoxSizer(wx.HORIZONTAL)
         ok_b = wx.Button(self, wx.ID_OK, _("Ok"))
         ok_b.SetDefault()
-        b_sizer.AddMany([(wx.Button(self, wx.ID_CANCEL, _("Cancel"))),
-                         (wx.Button(self, wx.ID_SAVE, _("Export"))), (ok_b)])
+        b_sizer.AddMany([(wx.Button(self, wx.ID_CANCEL, _("Cancel")), 0),
+                         ((5, 5), 0),
+                         (wx.Button(self, wx.ID_SAVE, _("Export")), 0),
+                         ((5, 5), 0), (ok_b, 0)])
         sizer.Add(b_sizer, 0, wx.ALIGN_RIGHT |
                   wx.ALIGN_CENTER_VERTICAL | wx.ALL, 5)
 
@@ -155,7 +157,7 @@ class StyleEditor(wx.Dialog):
         self.Bind(wx.EVT_CHECKBOX, self.OnCheck)
         self.Bind(wx.EVT_CLOSE, self.OnClose)
         self.Bind(wx.EVT_LISTBOX, self.OnListBox)
-        self.Bind(colorsetter.EVT_COLORSETTER, self.OnColor)
+        self.Bind(eclib.EVT_COLORSETTER, self.OnColor)
         self.preview.Bind(wx.EVT_LEFT_UP, self.OnTextRegion)
         self.preview.Bind(wx.EVT_KEY_UP, self.OnTextRegion)
     #--- End Init ---#
@@ -269,8 +271,8 @@ class StyleEditor(wx.Dialog):
 
         """
         if ed_glob.CONFIG['STYLES_DIR'] == ed_glob.CONFIG['SYS_STYLES_DIR']:
-            user_config = os.path.join(wx.GetHomeDir(),
-                                        "." + ed_glob.PROG_NAME, 'styles')
+            stdpath = wx.StandardPaths_Get()
+            user_config = os.path.join(stdpath.GetUserDataDir(), 'styles')
             if not os.path.exists(user_config):
                 try:
                     os.mkdir(user_config)
@@ -279,15 +281,20 @@ class StyleEditor(wx.Dialog):
                 else:
                     ed_glob.CONFIG['STYLES_DIR'] = user_config
 
-        dlg = wx.FileDialog(self, _("Export Style Sheet"),
-                            ed_glob.CONFIG['STYLES_DIR'],
-                            wildcard=_("Editra Style Sheet") + " (*.ess)|*.ess",
-                            style=wx.FD_SAVE | wx.OVERWRITE_PROMPT)
-        dlg.CenterOnParent()
-        result = dlg.ShowModal()
+        result = wx.ID_CANCEL
+        ss_c = self.FindWindowById(ed_glob.ID_PREF_SYNTHEME)
+        new_cb = self.FindWindowById(wx.ID_NEW)
+        if new_cb.GetValue():
+            name = ''
+        else:
+            name = ss_c.GetStringSelection()
 
-        if result == wx.ID_OK:
-            sheet_path = dlg.GetPath()
+        fname = wx.GetTextFromUser(_("Enter style sheet name"),
+                                   _("Export Style Sheet"),
+                                   name, self)
+
+        if len(fname):
+            sheet_path = os.path.join(ed_glob.CONFIG['STYLES_DIR'], fname)
             if sheet_path.split(u'.')[-1] != u'ess':
                 sheet_path += u".ess"
 
@@ -301,17 +308,19 @@ class StyleEditor(wx.Dialog):
             else:
                 # Update Style Sheet Control
                 sheet = ".".join(os.path.basename(sheet_path).split(u'.')[:-1])
-                ss_c = self.FindWindowById(ed_glob.ID_PREF_SYNTHEME)
                 ss_c.SetItems(util.GetResourceFiles(u'styles', get_all=True))
                 ss_c.SetStringSelection(sheet)
+                ss_c.Enable()
+                self.FindWindowById(wx.ID_NEW).SetValue(False)
                 self.styles_orig = self.styles_new
                 self.styles_new = DuplicateStyleDict(self.styles_orig)
+                result = wx.ID_OK
 
                 if sheet_path.startswith(ed_glob.CONFIG['STYLES_DIR']) or \
                    sheet_path.startswith(ed_glob.CONFIG['SYS_STYLES_DIR']):
                     # Update editor windows/buffer to use new style sheet
                     UpdateBufferStyles(sheet)
-        dlg.Destroy()
+
         return result
 
     def GenerateStyleSheet(self):
@@ -332,12 +341,21 @@ class StyleEditor(wx.Dialog):
             stage1 = wx.EmptyString
             for attr in ('fore', 'back', 'face', 'size'):
                 ival = item.GetNamedAttr(attr)
+                if attr in ('fore', 'back'):
+                    ival = ival.upper()
+
                 if ival is None or ival == ditem.GetNamedAttr(attr):
                     continue
 
                 stage1 = ''.join((stage1, attr, u':',
                                   ival.replace(',', ' '), u';'))
 
+            # Add any modifiers to the modifier tag
+            modifiers = item.GetModifiers()
+            if len(modifiers):
+                stage1 += (u"modifiers:" + modifiers + u";").replace(',', ' ')
+
+            # If the StyleItem had any set attributes add it to the stylesheet
             if len(stage1):
                 sty_sheet.append(tag + u" {\n")
                 stage2 = u"\t\t" + stage1[0:-1].replace(u";", u";\n\t\t") + u";"
@@ -370,7 +388,7 @@ class StyleEditor(wx.Dialog):
                 self.styles_orig = self.preview.BlankStyleDictionary()
                 self.styles_new = DuplicateStyleDict(self.styles_orig)
                 self.preview.SetStyles('preview', self.styles_new, nomerge=True)
-                self.preview.UpdateBaseStyles()
+                self.preview.UpdateAllStyles('preview')
 
                 # For some reason this causes the text display to refresh
                 # properly when nothing else would work.
@@ -438,7 +456,7 @@ class StyleEditor(wx.Dialog):
         style_id = self.preview.GetStyleAt(self.preview.GetCurrentPos())
         tag_lst = self.FindWindowById(ID_STYLES)
         data = self.preview.FindTagById(style_id)
-        if data != wx.EmptyString and self.styles_new.has_key(data):
+        if data != wx.EmptyString and data in self.styles_new:
             tag_lst.SetStringSelection(data)
             if wx.Platform == '__WXGTK__':
                 tag_lst.SetFirstItemStr(data)
@@ -452,7 +470,7 @@ class StyleEditor(wx.Dialog):
 
         """
         tag = evt.GetEventObject().GetStringSelection()
-        if tag != wx.EmptyString and self.styles_new.has_key(tag):
+        if tag != wx.EmptyString and tag in self.styles_new:
             self.UpdateSettingsPane(self.styles_new[tag])
             self.EnableSettings()
         else:
@@ -540,11 +558,8 @@ class StyleEditor(wx.Dialog):
                 ctrl.SetValue(val_map[sid])
             elif c_type == "wxChoice":
                 ctrl.SetStringSelection(val_map[sid])
-            elif isinstance(ctrl, colorsetter.ColorSetter):
+            elif isinstance(ctrl, eclib.ColorSetter):
                 ctrl.SetLabel(val_map[sid][:7])
-                ctrl.SetValue(wx.Color(int(val_map[sid][1:3], 16),
-                                       int(val_map[sid][3:5], 16),
-                                       int(val_map[sid][5:7], 16)))
         return True
 
     def UpdateStyleSet(self, id_):
@@ -566,7 +581,7 @@ class StyleEditor(wx.Dialog):
             val = ctrl.GetValue()
         elif ctrl_t == 'wxChoice':
             val = ctrl.GetStringSelection()
-        elif isinstance(ctrl, colorsetter.ColorSetter):
+        elif isinstance(ctrl, eclib.ColorSetter):
             val = ctrl.GetLabel()
         else:
             return False
@@ -622,7 +637,7 @@ class SettingsPanel(wx.Panel):
         # Foreground
         fground_sizer = wx.BoxSizer(wx.HORIZONTAL)
         fground_lbl = wx.StaticText(self, label=_("Foreground") + u": ")
-        fground_sel = colorsetter.ColorSetter(self, ID_FORE_COLOR, wx.BLACK)
+        fground_sel = eclib.ColorSetter(self, ID_FORE_COLOR, wx.BLACK)
         fground_sizer.AddMany([((5, 5)),
                                (fground_lbl, 0, wx.ALIGN_CENTER_VERTICAL),
                                ((2, 2), 1, wx.EXPAND),
@@ -634,7 +649,7 @@ class SettingsPanel(wx.Panel):
         # Background
         bground_sizer = wx.BoxSizer(wx.HORIZONTAL)
         bground_lbl = wx.StaticText(self, label=_("Background") + u": ")
-        bground_sel = colorsetter.ColorSetter(self, ID_BACK_COLOR, wx.WHITE)
+        bground_sel = eclib.ColorSetter(self, ID_BACK_COLOR, wx.WHITE)
         bground_sizer.AddMany([((5, 5)),
                                (bground_lbl, 0, wx.ALIGN_CENTER_VERTICAL),
                                ((2, 2), 1, wx.EXPAND),
@@ -718,7 +733,7 @@ def DuplicateStyleDict(style_dict):
 
 def UpdateBufferStyles(sheet):
     """Update the style used in all buffers
-    @sheet: Style sheet to use
+    @param sheet: Style sheet to use
 
     """
     # Only update if the sheet has changed
