@@ -14,8 +14,8 @@ FILE: cpp.py
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: cpp.py 55174 2008-08-22 15:12:27Z CJP $"
-__revision__ = "$Revision: 55174 $"
+__svnid__ = "$Id: cpp.py 58027 2009-01-11 17:58:04Z CJP $"
+__revision__ = "$Revision: 58027 $"
 
 #-----------------------------------------------------------------------------#
 # Imports
@@ -48,13 +48,13 @@ DOC_KEYWORDS = (2, "TODO FIXME XXX author brief bug callgraph category class "
                    "code date def depreciated dir dot dotfile else elseif em "
                    "endcode enddot endif endverbatim example exception file if "
                    "ifnot image include link mainpage name namespace page par "
-                   "paragraph param return retval section struct subpage "
-                   "subsection subsubsection test todo typedef union var "
-                   "verbatim version warning $ @ ~ < > # % HACK")
+                   "paragraph param pre post return retval section struct "
+                   "subpage subsection subsubsection test todo typedef union "
+                   "var verbatim version warning $ @ ~ < > # % HACK")
 
 # CPP Keyword Extensions
 CPP_KEYWORDS = ("and and_eq bitand bitor catch class compl const_cast delete "
-                "dynamic_cast false friend new not not_eq opperator or or_eq "
+                "dynamic_cast false friend new not not_eq operator or or_eq "
                 "private protected public reinterpret_cast static_cast this "
                 "throw try true typeid using xor xor_eq")
 
@@ -113,7 +113,7 @@ SYNTAX_ITEMS = [ ('STC_C_DEFAULT', 'default_style'),
                  ('STC_C_STRING', 'string_style'),
                  ('STC_C_STRINGEOL', 'stringeol_style'),
                  ('STC_C_UUID', 'pre_style'),
-                 ('STC_C_VERBATIM', "number2_style"),
+                 ('STC_C_VERBATIM', 'number2_style'),
                  ('STC_C_WORD', 'keyword_style'),
                  ('STC_C_WORD2', 'keyword2_style') ]
 
