@@ -14,11 +14,14 @@ AUTHOR: Cody Precord
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: tcl.py 52852 2008-03-27 13:45:40Z CJP $"
-__revision__ = "$Revision: 52852 $"
+__svnid__ = "$Id: tcl.py 55180 2008-08-22 17:35:29Z CJP $"
+__revision__ = "$Revision: 55180 $"
 
 #-----------------------------------------------------------------------------#
+# Local Imports
 import synglob
+from cpp import AutoIndenter
+
 #-----------------------------------------------------------------------------#
 
 #---- Keyword Definitions ----#
@@ -149,7 +152,7 @@ SYNTAX_ITEMS = [('STC_TCL_BLOCK_COMMENT', 'comment_style'),
                  ('STC_TCL_NUMBER', 'number_style'),
                  ('STC_TCL_OPERATOR', 'operator_style'),
                  ('STC_TCL_SUBSTITUTION', 'scalar_style'),
-                 ('STC_TCL_SUB_BRACE', 'default_style'), # STYLE NEEDED
+                 ('STC_TCL_SUB_BRACE', 'string_style'), # STYLE NEEDED
                  ('STC_TCL_WORD', 'keyword_style'),        # tcl_kw
                  ('STC_TCL_WORD2', 'keyword2_style'),      # tk_kw
                  ('STC_TCL_WORD3', 'keyword3_style'),      # itcl_kw
