@@ -15,8 +15,8 @@ AUTHOR: Cody Precord
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: xml.py 52852 2008-03-27 13:45:40Z CJP $"
-__revision__ = "$Revision: 52852 $"
+__svnid__ = "$Id: xml.py 59543 2009-03-15 00:28:39Z CJP $"
+__revision__ = "$Revision: 59543 $"
 
 #-----------------------------------------------------------------------------#
 # Dependencies
@@ -47,7 +47,7 @@ def Keywords(lang_id=0):
     @param lang_id: used to select specific subset of keywords
 
     """
-    return [(5, XML_KEYWORDS), (6, SGML_KEYWORDS)]
+    return [(5, XML_KEYWORDS + u" " + SGML_KEYWORDS)]
 
 def SyntaxSpec(lang_id=0):
     """Syntax Specifications
