@@ -12,14 +12,16 @@ This file contains various helper functions and utilities that the program uses.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: util.py 54270 2008-06-17 03:01:48Z CJP $"
-__revision__ = "$Revision: 54270 $"
+__svnid__ = "$Id: util.py 60536 2009-05-07 02:31:37Z CJP $"
+__revision__ = "$Revision: 60536 $"
 
 #--------------------------------------------------------------------------#
-# Dependancies
+# Imports
 import os
 import sys
-import stat
+import types
+import mimetypes
+import encodings
 import codecs
 import urllib2
 import wx
@@ -29,6 +31,11 @@ import ed_glob
 import ed_event
 import ed_crypt
 import dev_tool
+import syntax.syntax as syntax
+import syntax.synglob as synglob
+import eclib # TODO: Temporary till refs to HexToRGB and AdjustColor are
+             #       no longer used anywhere
+import ebmlib
 
 _ = wx.GetTranslation
 #--------------------------------------------------------------------------#
@@ -134,21 +141,26 @@ class DropTargetFT(wx.PyDropTarget):
                is not moved.
 
         """
+        stc = self.window
         if self._tmp is None:
-            return drag_result
+            if hasattr(stc, 'DoDragOver'):
+                val = stc.DoDragOver(x_cord, y_cord, drag_result)
+                self.ScrollBuffer(stc, x_cord, y_cord)
+            drag_result = wx.DragCopy
         else:
-            stc = self.window
-            point = wx.Point(x_cord, y_cord)
-            self._tmp.BeginDrag(point - self._lastp, stc)
-            self._tmp.Hide()
-            stc.GotoPos(stc.PositionFromPoint(point))
-            stc.Refresh()
-            stc.Update()
-            self._tmp.Move(point)
-            self._tmp.Show()
-            self._tmp.RedrawImage(self._lastp, point, True, True)
-            self._lastp = point
-            return drag_result
+            if hasattr(stc, 'DoDragOver'):
+                point = wx.Point(x_cord, y_cord)
+                self._tmp.BeginDrag(point - self._lastp, stc)
+                self._tmp.Hide()
+                stc.DoDragOver(x_cord, y_cord, drag_result)
+                self._tmp.Move(point)
+                self._tmp.Show()
+                self._tmp.RedrawImage(self._lastp, point, True, True)
+                self._lastp = point
+                self.ScrollBuffer(stc, x_cord, y_cord)
+            drag_result = wx.DragCopy
+
+        return drag_result
 
     def OnData(self, x_cord, y_cord, drag_result):
         """Gets and processes the dropped data
@@ -164,8 +176,8 @@ class DropTargetFT(wx.PyDropTarget):
         except wx.PyAssertionError:
             wx.PostEvent(self.window.GetTopLevelParent(), \
                         ed_event.StatusEvent(ed_event.edEVT_STATUS, -1,
-                                             _("Unable to open dropped file or "
-                                               "text")))
+                                             _("Unable to accept dropped file "
+                                               "or text")))
             data = False
             drag_result = wx.DragCancel
 
@@ -174,16 +186,11 @@ class DropTargetFT(wx.PyDropTarget):
             text = self._data['tdata'].GetText()
             if len(files) > 0 and self._data['fcallb'] is not None:
                 self._data['fcallb'](files)
-            elif(len(text) > 0):
-                if hasattr(self.window, 'PositionFromPointClose'):
-                    pos = self.window.PositionFromPointClose(x_cord, y_cord)
+            elif len(text) > 0:
+                if self._data['tcallb'] is not None:
+                    self._data['tcallb'](text)
                 else:
-                    pos = wx.stc.STC_INVALID_POSITION
-
-                if pos != wx.stc.STC_INVALID_POSITION:
-                    self.window.InsertText(pos, text)
-                else:
-                    drag_result = wx.DragCancel
+                    self.window.DoDropText(x_cord, y_cord, text)
         self.InitObjects()
         return drag_result
 
@@ -202,6 +209,28 @@ class DropTargetFT(wx.PyDropTarget):
             except wx.PyAssertionError, msg:
                 Log("[droptargetft][err] %s" % str(msg))
 
+    @staticmethod
+    def ScrollBuffer(stc, x_cord, y_cord):
+        """Scroll the buffer as the dragged text is moved towards the
+        ends.
+        @param stc: StyledTextCtrl
+        @param x_cord: int (x position)
+        @param y_cord: int (y position)
+        @note: currenly does not work on wxMac
+
+        """
+        cline = stc.PositionFromPoint(wx.Point(x_cord, y_cord))
+        if cline != wx.stc.STC_INVALID_POSITION:
+            cline = stc.LineFromPosition(cline)
+            fline = stc.GetFirstVisibleLine()
+            lline = stc.GetLastVisibleLine()
+            if (cline - fline) < 2:
+                stc.ScrollLines(-1)
+            elif lline - cline < 2:
+                stc.ScrollLines(1)
+            else:
+                pass
+
 #---- End FileDropTarget ----#
 
 #---- Misc Common Function Library ----#
@@ -268,29 +297,33 @@ def FilterFiles(file_list):
 
     """
     good = list()
+    checker = ebmlib.FileTypeChecker()
     for path in file_list:
-        # Filter out directories and some common filetypes that can't
-        # be opened.
-        if not os.path.exists(path) or os.path.isdir(path) or \
-           GetExtension(path).lower() in ['gz', 'tar', 'bz2', 'zip', 'rar',
-                                          'ace', 'png', 'jpg', 'gif', 'jpeg',
-                                          'bmp', 'exe', 'pyc', 'pyo', 'psd',
-                                          'a', 'o', 'dll', 'ico', 'icns']:
-            continue
-        else:
+        if not checker.IsBinary(path):
             good.append(path)
     return good
 
-def GetFileModTime(file_name):
-    """Returns the time that the given file was last modified on
-    @param file_name: path of file to get mtime of
+def GetFileType(fname):
+    """Get what the type of the file is as Editra sees it
+    in a formatted string.
+    @param fname: file path
+    @return: string (formatted/translated filetype)
 
     """
-    try:
-        mod_time = os.path.getmtime(file_name)
-    except EnvironmentError:
-        mod_time = 0
-    return mod_time
+    if os.path.isdir(fname):
+        return _("Folder")
+
+    eguess = syntax.GetTypeFromExt(fname.split('.')[-1])
+    if eguess == synglob.LANG_TXT and fname.split('.')[-1] == 'txt':
+        return _("Text Document")
+    elif eguess == synglob.LANG_TXT:
+        mtype = mimetypes.guess_type(fname)[0]
+        if mtype is not None:
+            return mtype
+        else:
+            return _("Unknown")
+    else:
+        return _("%s Source File") % eguess
 
 def GetFileReader(file_name, enc='utf-8'):
     """Returns a file stream reader object for reading the
@@ -329,7 +362,7 @@ def GetFileWriter(file_name, enc='utf-8'):
 
     """
     try:
-        file_h = file(file_name, "wb")
+        file_h = open(file_name, "wb")
     except IOError:
         dev_tool.DEBUGP("[file_writer][err] Failed to open file %s" % file_name)
         return -1
@@ -340,42 +373,10 @@ def GetFileWriter(file_name, enc='utf-8'):
         writer = file_h
     return writer
 
-def GetUniqueName(path, name):
-    """Make a file name that will be unique in case a file of the
-    same name already exists at that path.
-    @param path: Root path to folder of files destination
-    @param name: desired file name base
-    @return: string
-
-    """
-    tmpname = os.path.join(path, name)
-    if os.path.exists(tmpname):
-        if '.' not in name:
-            ext = ''
-            fbase = name
-        else:
-            ext = '.' + name.split('.')[-1]
-            fbase = name[:-1 * len(ext)]
-
-        inc = len([x for x in os.listdir(path) if x.startswith(fbase)])
-        tmpname = os.path.join(path, "%s-%d%s" % (fbase, inc, ext))
-        while os.path.exists(tmpname):
-            inc = inc + 1
-            tmpname = os.path.join(path, "%s-%d%s" % (fbase, inc, ext))
-
-    return tmpname
-
-def GetPathName(path):
-    """Gets the path minus filename
-    @param path: full path to get base of
-
-    """
-    return os.path.split(path)[0]
-
 def GetFileManagerCmd():
     """Get the file manager open command for the current os. Under linux
-    it will check for nautilus and konqueror and return which one it finds
-    first or 'nautilus' (Gnome) if it finds neither.
+    it will check for xdg-open, nautilus, konqueror, and Thunar, it will then
+    return which one it finds first or 'nautilus' it finds nothing.
     @return: string
 
     """
@@ -386,126 +387,54 @@ def GetFileManagerCmd():
     else:
         # Check for common linux filemanagers returning first one found
         #          Gnome/ubuntu KDE/kubuntu  xubuntu
-        for cmd in ('nautilus', 'konqueror', 'Thunar'):
+        for cmd in ('xdg-open', 'nautilus', 'konqueror', 'Thunar'):
             result = os.system("which %s > /dev/null" % cmd)
             if result == 0:
                 return cmd
         else:
             return 'nautilus'
 
-def GetFileName(path):
-    """Gets last atom on end of string as filename
-    @param path: full path to get filename from
-
-    """
-    return os.path.split(path)[-1]
-
-def GetFileSize(path):
-    """Get the size of the file at a given path
-    @param path: Path to file
-    @return: long
-
-    """
-    try:
-        return os.stat(path)[stat.ST_SIZE]
-    except:
-        return 0
-
-def GetExtension(file_str):
-    """Gets last atom at end of string as extension if
-    no extension whole string is returned
-    @param file_str: path or file name to get extension from
-
-    """
-    return file_str.split('.')[-1]
-
-def MakeNewFile(path, name):
-    """Make a new file at the given path with the given name.
-    If the file already exists, the given name will be changed to
-    a unique name in the form of name + -NUMBER + .extension
-    @param path: path to directory to create file in
-    @param name: desired name of file
-    @return: Tuple of (success?, Path of new file OR Error message)
-
-    """
-    if not os.path.isdir(path):
-        path = os.path.dirname(path)
-    fname = GetUniqueName(path, name)
-
-    try:
-        open(fname, 'w').close()
-    except (IOError, OSError), msg:
-        return (False, str(msg))
-
-    return (True, fname)
-
-def MakeNewFolder(path, name):
-    """Make a new folder at the given path with the given name.
-    If the folder already exists, the given name will be changed to
-    a unique name in the form of name + -NUMBER.
-    @param path: path to create folder on
-    @param name: desired name for folder
-    @return: Tuple of (success?, new dirname OR Error message)
-
-    """
-    if not os.path.isdir(path):
-        path = os.path.dirname(path)
-    folder = GetUniqueName(path, name)
-    try:
-        os.mkdir(folder)
-    except (OSError, IOError), msg:
-        return (False, str(msg))
-
-    return (True, folder)
-
-def ResolvAbsPath(rel_path):
-    """Takes a relative path and converts it to an
-    absolute path.
-    @param rel_path: path to construct absolute path for
-
-    """
-    cwd = os.getcwd()
-    pieces = rel_path.split(os.sep)
-    cut = 0
-
-    for token in pieces:
-        if token == "..":
-            cut += 1
-
-        if cut > 0:
-            rpath = os.sep.join(pieces[cut:])
-            cut *= -1
-            cwd = cwd.split(os.sep)
-            apath = os.sep.join(cwd[0:cut])
-        else:
-            rpath = rel_path
-            apath = cwd
-
-    return apath + os.sep + rpath
-
 def HasConfigDir(loc=u""):
     """ Checks if the user has a config directory and returns True
     if the config directory exists or False if it does not.
     @return: whether config dir in question exists on an expected path
 
     """
-    if os.path.exists(u"%s%s.%s%s%s" % (wx.GetHomeDir(), os.sep,
-                                        ed_glob.PROG_NAME, os.sep, loc)):
-        return True
-    else:
-        return False
+    cbase = ed_glob.CONFIG['CONFIG_BASE']
+    if cbase is None:
+        cbase = wx.StandardPaths_Get().GetUserDataDir() + os.sep
+
+    to_check = os.path.join(cbase, loc)
+    return os.path.exists(to_check)
 
 def MakeConfigDir(name):
     """Makes a user config directory
     @param name: name of config directory to make in user config dir
 
     """
-    config_dir = wx.GetHomeDir() + os.sep + u"." + ed_glob.PROG_NAME
+    cbase = ed_glob.CONFIG['CONFIG_BASE']
+    if cbase is None:
+        cbase = wx.StandardPaths_Get().GetUserDataDir()
+
     try:
-        os.mkdir(config_dir + os.sep + name)
+        os.mkdir(cbase + os.sep + name)
     except (OSError, IOError):
         pass
 
+def RepairConfigState(path):
+    """Repair the state of profile path, updating and creating it
+    it does not exist.
+    @param path: path of profile
+
+    """
+    if os.path.isabs(path) and os.path.exists(path):
+        return path
+    else:
+        # Need to fix some stuff up
+        CreateConfigDir()
+        import profiler
+        return profiler.Profile_Get("MYPROFILE")
+
 def CreateConfigDir():
     """ Creates the user config directory its default sub
     directories and any of the default config files.
@@ -513,10 +442,13 @@ def CreateConfigDir():
 
     """
     #---- Resolve Paths ----#
-    config_dir = u"%s%s.%s" % (wx.GetHomeDir(), os.sep, ed_glob.PROG_NAME)
-    profile_dir = u"%s%sprofiles" % (config_dir, os.sep)
-    dest_file = u"%s%sdefault.ppb" % (profile_dir, os.sep)
-    ext_cfg = ["cache", "styles", "plugins"]
+    config_dir = ed_glob.CONFIG['CONFIG_BASE']
+    if config_dir is None:
+        config_dir = wx.StandardPaths_Get().GetUserDataDir() + os.sep
+
+    profile_dir = os.path.join(config_dir, u"profiles")
+    dest_file = os.path.join(profile_dir, u"default.ppb")
+    ext_cfg = [u"cache", u"styles", u"plugins"]
 
     #---- Create Directories ----#
     if not os.path.exists(config_dir):
@@ -530,8 +462,9 @@ def CreateConfigDir():
             MakeConfigDir(cfg)
 
     import profiler
-    profiler.Profile().LoadDefaults()
+    profiler.TheProfile.LoadDefaults()
     profiler.Profile_Set("MYPROFILE", dest_file)
+    profiler.TheProfile.Write(dest_file)
     profiler.UpdateProfileLoader()
 
 def ResolvConfigDir(config_dir, sys_only=False):
@@ -547,26 +480,36 @@ def ResolvConfigDir(config_dir, sys_only=False):
            the code has proven itself.
 
     """
+    stdpath = wx.StandardPaths_Get()
+
+    # Try to get a User config directory
     if not sys_only:
-        # Try to look for a user dir
-        user_config = u"%s%s.%s%s%s" % (wx.GetHomeDir(), os.sep,
-                                        ed_glob.PROG_NAME, os.sep,
-                                        config_dir)
+        user_config = ed_glob.CONFIG['CONFIG_BASE']
+        if user_config is None:
+            user_config = stdpath.GetUserDataDir() + os.sep
+
+        user_config = os.path.join(user_config, config_dir)
+
         if os.path.exists(user_config):
             return user_config + os.sep
 
     # The following lines are used only when Editra is being run as a
     # source package. If the found path does not exist then Editra is
     # running as as a built package.
-    path = __file__
-    path = os.sep.join(path.split(os.sep)[:-2])
-    path =  path + os.sep + config_dir + os.sep
-    if os.path.exists(path):
-        return path
+    if not hasattr(sys, 'frozen'):
+        path = __file__
+        path = os.sep.join(path.split(os.sep)[:-2])
+        path =  path + os.sep + config_dir + os.sep
+        if os.path.exists(path):
+            if not isinstance(path, types.UnicodeType):
+                path = unicode(path, sys.getfilesystemencoding())
+            return path
 
     # If we get here we need to do some platform dependant lookup
     # to find everything.
     path = sys.argv[0]
+    if not isinstance(path, types.UnicodeType):
+        path = unicode(path, sys.getfilesystemencoding())
 
     # If it is a link get the real path
     if os.path.islink(path):
@@ -586,28 +529,34 @@ def ResolvConfigDir(config_dir, sys_only=False):
             pieces = pro_path.split(os.sep)
             pro_path = os.sep.join(pieces[:-1])
         else:
-            pro_path = ResolvAbsPath(pro_path)
+            pro_path = os.path.abspath(pro_path)
+    elif os.sys.platform == "darwin":
+        # On OS X the config directories are in the applet under Resources
+        pro_path = stdpath.GetResourcesDir()
+        pro_path = os.path.join(pro_path, config_dir)
     else:
         pro_path = os.sep.join(pieces[:-2])
 
         if pro_path.startswith(os.sep):
             pass
-        elif pro_path == "":
+        elif pro_path == u"":
             pro_path = os.getcwd()
             pieces = pro_path.split(os.sep)
             if pieces[-1] not in [ed_glob.PROG_NAME.lower(), ed_glob.PROG_NAME]:
                 pro_path = os.sep.join(pieces[:-1])
         else:
-            pro_path = ResolvAbsPath(pro_path)
+            pro_path = os.path.abspath(pro_path)
 
-    if os.sys.platform == "darwin":
-        # On OS X the config directories are in the applet under Resources
-        pro_path = u"%s%sResources%s%s%s" % (pro_path, os.sep, os.sep,
-                                             config_dir, os.sep)
-    else:
+    if os.sys.platform != "darwin":
         pro_path = pro_path + os.sep + config_dir + os.sep
 
-    return os.path.normpath(pro_path) + os.sep
+    path = os.path.normpath(pro_path) + os.sep
+
+    # Make sure path is unicode
+    if not isinstance(path, types.UnicodeType):
+        path = unicode(path, sys.getdefaultencoding())
+
+    return path
 
 def GetResources(resource):
     """Returns a list of resource directories from a given toplevel config dir
@@ -673,6 +622,17 @@ def GetResourceFiles(resource, trim=True, get_all=False,
         
         return list(set(rec_list))
 
+def GetAllEncodings():
+    """Get all encodings found on the system
+    @return: list of strings
+
+    """
+    elist = encodings.aliases.aliases.values()
+    elist = list(set(elist))
+    elist.sort()
+    elist = [ enc for enc in elist if not enc.endswith('codec') ]
+    return elist
+
 def Log(msg):
     """Push the message to the apps log
     @param msg: message string to log
@@ -707,45 +667,6 @@ def GetProxyOpener(proxy_set):
     return opener
 
 #---- GUI helper functions ----#
-def AdjustColour(color, percent, alpha=wx.ALPHA_OPAQUE):
-    """ Brighten/Darken input colour by percent and adjust alpha
-    channel if needed. Returns the modified color.
-    @param color: color object to adjust
-    @type color: wx.Color
-    @param percent: percent to adjust +(brighten) or -(darken)
-    @type percent: int
-    @keyword alpha: Value to adjust alpha channel to
-
-    """
-    radj, gadj, badj = [ int(val * (abs(percent) / 100.))
-                         for val in color.Get() ]
-
-    if percent < 0:
-        radj, gadj, badj = [ val * -1 for val in [radj, gadj, badj] ]
-    else:
-        radj, gadj, badj = [ val or percent for val in [radj, gadj, badj] ]
-
-    red = min(color.Red() + radj, 255)
-    green = min(color.Green() + gadj, 255)
-    blue = min(color.Blue() + badj, 255)
-    return wx.Colour(red, green, blue, alpha)
-
-def HexToRGB(hex_str):
-    """Returns a list of red/green/blue values from a
-    hex string.
-    @param hex_str: hex string to convert to rgb
-
-    """
-    hexval = hex_str
-    if hexval[0] == u"#":
-        hexval = hexval[1:]
-    ldiff = 6 - len(hexval)
-    hexval += ldiff * u"0"
-    # Convert hex values to integer
-    red = int(hexval[0:2], 16)
-    green = int(hexval[2:4], 16)
-    blue = int(hexval[4:], 16)
-    return [red, green, blue]
 
 def SetWindowIcon(window):
     """Sets the given windows icon to be the programs
