@@ -2,7 +2,7 @@
 # Purpose:      XMLTree class
 # Author:       Roman Rolinsky <rolinsky@femagsoft.com>
 # Created:      31.05.2007
-# RCS-ID:       $Id: XMLTree.py 51998 2008-02-22 15:01:37Z ROL $
+# RCS-ID:       $Id: XMLTree.py 54812 2008-07-29 13:39:00Z ROL $
 
 from globals import *
 from model import Model
@@ -113,7 +113,7 @@ class XMLTree(wx.TreeCtrl):
 
     def FlushSubtree(self, item, node):
         '''Update all items after changes in model.'''
-        if item == self.root:
+        if item is None or item == self.root:
             self.Flush()
             return
         self.DeleteChildren(item)
