@@ -4,7 +4,7 @@
 #
 # Author:      Robin Dunn
 #
-# RCS-ID:      $Id: img2py.py 53756 2008-05-26 19:01:06Z RD $
+# RCS-ID:      $Id: img2py.py 60088 2009-04-10 20:16:37Z RD $
 # Copyright:   (c) 2002 by Total Control Software
 # Licence:     wxWindows license
 #----------------------------------------------------------------------
@@ -90,6 +90,11 @@ import tempfile
 import wx
 import img2img
 
+try:
+    b64encode = base64.b64encode
+except AttributeError:
+    b64encode = base64.encodestring
+
 app = None
 DEFAULT_APPEND = False
 DEFAULT_COMPRESSED = True
@@ -103,6 +108,22 @@ DEFAULT_COMPATIBLE = True   # True for 2.8, change to False in 2.9...
 # "index.append('Image name')"
 indexPattern = re.compile(r"\s*index.append\('(.+)'\)\s*")
 
+
+def convert(fileName, maskClr, outputDir, outputName, outType, outExt):
+    # if the file is already the right type then just use it directly
+    if maskClr == DEFAULT_MASKCLR and fileName.upper().endswith(outExt.upper()):
+        if outputName:
+            newname = outputName
+        else:
+            newname = os.path.join(outputDir,
+                                   os.path.basename(os.path.splitext(fileName)[0]) + outExt)
+        file(newname, "wb").write(file(fileName).read())
+        return 1, "ok"
+  
+    else:
+        return img2img.convert(fileName, maskClr, outputDir, outputName, outType, outExt)
+    
+    
 def img2py(image_file, python_file,
            append=DEFAULT_APPEND,
            compressed=DEFAULT_COMPRESSED,
@@ -110,13 +131,20 @@ def img2py(image_file, python_file,
            imgName=DEFAULT_IMGNAME,
            icon=DEFAULT_ICON,
            catalog=DEFAULT_CATALOG,
-           functionCompatibile=DEFAULT_COMPATIBLE):
+           functionCompatible=DEFAULT_COMPATIBLE,
+           functionCompatibile=-1,   # typo version for backward compatibility
+           ):
     """
     Converts an image file to a data structure written in a Python file
     --image_file: string; the path of the source image file
     --python_file: string; the path of the destination python file
     --other arguments: they are equivalent to the command-line arguments
     """
+    
+    # was the typo version used?
+    if functionCompatibile != -1:
+        functionCompatible = functionCompatibile
+        
     global app
     if not wx.GetApp():
         app = wx.PySimpleApp()
@@ -124,13 +152,13 @@ def img2py(image_file, python_file,
     # convert the image file to a temporary file
     tfname = tempfile.mktemp()
     try:
-        ok, msg = img2img.convert(image_file, maskClr, None, tfname, wx.BITMAP_TYPE_PNG, ".png")
+        ok, msg = convert(image_file, maskClr, None, tfname, wx.BITMAP_TYPE_PNG, ".png")
         if not ok:
             print msg
             return
 
         lines = []
-        data = base64.b64encode(open(tfname, "rb").read())
+        data = b64encode(open(tfname, "rb").read())
         while data:
             part = data[:72]
             data = data[72:]
@@ -213,7 +241,7 @@ def img2py(image_file, python_file,
             out.write("index.append('%s')\n" % imgName)
             out.write("catalog['%s'] = %s\n" % (imgName, varName))
 
-        if functionCompatibile:
+        if functionCompatible:
             out.write("get%sData = %s.GetData\n" % (varName, varName))
             out.write("get%sImage = %s.GetImage\n" % (varName, varName))
             out.write("get%sBitmap = %s.GetBitmap\n" % (varName, varName))
